@@ -19,7 +19,12 @@
 #include <windows.h>	/* LoadLibrary() */
 #endif
 
+#ifdef SDL
+#include "SDL.h"
+#endif
+
 #include <assert.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
@@ -38,6 +43,7 @@
 #include "packets.h"
 #include "rand.h"
 #include "support.h"
+#include "timing.h"
 #include "version.h"
 
 #include "agents.h"
@@ -49,11 +55,12 @@
 #include "climisc.h"
 #include "clinet.h"
 #include "cma_core.h"		/* kludge */
+#include "connectdlg_common.h"  /* client_kill_server() */
 #include "connectdlg_g.h"
 #include "control.h" 
 #include "dialogs_g.h"
 #include "diplodlg_g.h"
-#include "goto.h"
+#include "ggzclient.h"
 #include "gui_main_g.h"
 #include "helpdata.h"		/* boot_help_texts() */
 #include "mapctrl_g.h"
@@ -66,6 +73,7 @@
 #include "plrdlg_g.h"
 #include "repodlgs_g.h"
 #include "tilespec.h"
+#include "themes_common.h"
 
 #include "civclient.h"
 
@@ -84,10 +92,9 @@ char password[MAX_LEN_PASSWORD] = "\0";
 char metaserver[512] = "\0";
 int  server_port = -1;
 bool auto_connect = FALSE; /* TRUE = skip "Connect to Freeciv Server" dialog */
+bool in_ggz = FALSE;
 
-static enum client_states client_state = CLIENT_BOOT_STATE;
-
-int seconds_to_turndone;
+static enum client_states civclient_state = C_S_INITIAL;
 
 /* TRUE if an end turn request is blocked by busy agents */
 bool waiting_for_end_turn = FALSE;
@@ -98,6 +105,11 @@ bool waiting_for_end_turn = FALSE;
  */
 bool turn_done_sent = FALSE;
 
+/*
+ * TRUE between receiving PACKET_END_TURN and PACKET_BEGIN_TURN
+ */
+static bool server_busy = FALSE;
+
 /**************************************************************************
   Convert a text string from the internal to the data encoding, when it
   is written to the network.
@@ -154,6 +166,38 @@ static void charsets_init(void)
   dio_set_get_conv_callback(get_conv);
 }
 
+/**************************************************************************
+ This is called at program exit.
+**************************************************************************/
+static void at_exit(void)
+{
+  client_kill_server(TRUE);
+  my_shutdown_network();
+}
+
+/**************************************************************************
+  Called only by set_client_state() below.
+**************************************************************************/
+static void client_game_init(void)
+{
+  game_init();
+  attribute_init();
+  agents_init();
+  control_init();
+}
+
+/**************************************************************************
+  Called by set_client_state() and client_exit() below.
+**************************************************************************/
+static void client_game_free(void)
+{
+  control_done();
+  free_help_texts();
+  attribute_free();
+  agents_free();
+  game_free();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -163,6 +207,7 @@ int main(int argc, char *argv[])
   int ui_options = 0;
   bool ui_separator = FALSE;
   char *option=NULL;
+  bool user_tileset = FALSE;
 
   /* Load win32 post-crash debugger */
 #ifdef WIN32_NATIVE
@@ -177,6 +222,7 @@ int main(int argc, char *argv[])
 
   init_nls();
   audio_init();
+  init_character_encodings(gui_character_encoding, gui_use_transliteration);
 
   /* default argument values are set in options.c */
   loglevel=LOG_NORMAL;
@@ -184,89 +230,99 @@ int main(int argc, char *argv[])
   i = 1;
 
   while (i < argc) {
-   if (ui_separator) {
-     argv[1 + ui_options] = argv[i];
-     ui_options++;
-   } else if (is_option("--help", argv[i])) {
-    fc_fprintf(stderr, _("Usage: %s [option ...]\n"
-		      "Valid options are:\n"), argv[0]);
-    fc_fprintf(stderr, _("  -a, --autoconnect\tSkip connect dialog\n"));
+    if (ui_separator) {
+      argv[1 + ui_options] = argv[i];
+      ui_options++;
+    } else if (is_option("--help", argv[i])) {
+      fc_fprintf(stderr, _("Usage: %s [option ...]\n"
+			   "Valid options are:\n"), argv[0]);
+      fc_fprintf(stderr, _("  -a, --autoconnect\tSkip connect dialog\n"));
 #ifdef DEBUG
-    fc_fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 4,"
-                                  " or 4:file1,min,max:...)\n"));
+      fc_fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 4,"
+			   " or 4:file1,min,max:...)\n"));
 #else
-    fc_fprintf(stderr,
-	       _("  -d, --debug NUM\tSet debug log level (0 to 3)\n"));
+      fc_fprintf(stderr,
+		 _("  -d, --debug NUM\tSet debug log level (0 to 3)\n"));
 #endif
-    fc_fprintf(stderr,
-	       _("  -h, --help\t\tPrint a summary of the options\n"));
-    fc_fprintf(stderr, _("  -l, --log FILE\tUse FILE as logfile "
-                      "(spawned server also uses this)\n"));
-    fc_fprintf(stderr, _("  -m, --meta HOST\t"
-		      "Connect to the metaserver at HOST\n"));
-    fc_fprintf(stderr, _("  -n, --name NAME\tUse NAME as name\n"));
-    fc_fprintf(stderr,
-	       _("  -p, --port PORT\tConnect to server port PORT\n"));
-    fc_fprintf(stderr,
-	       _("  -P, --Plugin PLUGIN\tUse PLUGIN for sound output %s\n"),
-	    audio_get_all_plugin_names());
-    fc_fprintf(stderr, _("  -r, --read FILE\tRead startup script FILE "
-                      "(for spawned server only)\n"));
-    fc_fprintf(stderr,
-	       _("  -s, --server HOST\tConnect to the server at HOST\n"));
-    fc_fprintf(stderr, _("  -S, --Sound FILE\tRead sound tags from FILE\n"));
-    fc_fprintf(stderr, _("  -t, --tiles FILE\t"
-		      "Use data file FILE.tilespec for tiles\n"));
-    fc_fprintf(stderr, _("  -v, --version\t\tPrint the version number\n"));
-    fc_fprintf(stderr, _("      --\t\t"
-		      "Pass any following options to the UI.\n"
-		      "\t\t\tTry \"%s -- --help\" for more.\n"), argv[0]);
-    exit(EXIT_SUCCESS);
-   } else if (is_option("--version",argv[i])) {
-    fc_fprintf(stderr, "%s %s\n", freeciv_name_version(), client_string);
-    exit(EXIT_SUCCESS);
-   } else if ((option = get_option("--log",argv,&i,argc))) {
-      logfile = mystrdup(option); /* never free()d */
-   } else  if ((option = get_option("--read", argv, &i, argc)))
-      scriptfile = mystrdup(option); /* never free()d */
-   else if ((option = get_option("--name",argv,&i,argc)))
+      fc_fprintf(stderr,
+		 _("  -h, --help\t\tPrint a summary of the options\n"));
+      fc_fprintf(stderr, _("  -l, --log FILE\tUse FILE as logfile "
+			   "(spawned server also uses this)\n"));
+      fc_fprintf(stderr, _("  -m, --meta HOST\t"
+			   "Connect to the metaserver at HOST\n"));
+      fc_fprintf(stderr, _("  -n, --name NAME\tUse NAME as name\n"));
+      fc_fprintf(stderr,
+		 _("  -p, --port PORT\tConnect to server port PORT (usually with -a)\n"));
+      fc_fprintf(stderr,
+		 _("  -P, --Plugin PLUGIN\tUse PLUGIN for sound output %s\n"),
+		 audio_get_all_plugin_names());
+      fc_fprintf(stderr, _("  -r, --read FILE\tRead startup script FILE "
+			   "(for spawned server only)\n"));
+      fc_fprintf(stderr,
+		 _("  -s, --server HOST\tConnect to the server at HOST (usually with -a)\n"));
+      fc_fprintf(stderr,
+		 _("  -S, --Sound FILE\tRead sound tags from FILE\n"));
+      fc_fprintf(stderr, _("  -t, --tiles FILE\t"
+			   "Use data file FILE.tilespec for tiles\n"));
+      fc_fprintf(stderr, _("  -v, --version\t\tPrint the version number\n"));
+      fc_fprintf(stderr, _("      --\t\t"
+			   "Pass any following options to the UI.\n"
+			   "\t\t\tTry \"%s -- --help\" for more.\n"), argv[0]);
+      exit(EXIT_SUCCESS);
+    } else if (is_option("--version",argv[i])) {
+      fc_fprintf(stderr, "%s %s\n", freeciv_name_version(), client_string);
+      exit(EXIT_SUCCESS);
+    } else if ((option = get_option_malloc("--log", argv, &i, argc))) {
+      logfile = option; /* never free()d */
+    } else  if ((option = get_option_malloc("--read", argv, &i, argc))) {
+      scriptfile = option; /* never free()d */
+    } else if ((option = get_option_malloc("--name", argv, &i, argc))) {
       sz_strlcpy(user_name, option);
-   else if ((option = get_option("--meta",argv,&i,argc)))
+      free(option);
+    } else if ((option = get_option_malloc("--meta", argv, &i, argc))) {
       sz_strlcpy(metaserver, option);
-   else if ((option = get_option("--Sound", argv, &i, argc)))
+      free(option);
+    } else if ((option = get_option_malloc("--Sound", argv, &i, argc))) {
       sz_strlcpy(sound_set_name, option);
-   else if ((option = get_option("--Plugin", argv, &i, argc)))
+      free(option);
+    } else if ((option = get_option_malloc("--Plugin", argv, &i, argc))) {
       sz_strlcpy(sound_plugin_name, option);
-   else if ((option = get_option("--port",argv,&i,argc))) {
-     if(sscanf(option, "%d", &server_port) != 1) {
-       fc_fprintf(stderr,
-		  _("Invalid port \"%s\" specified with --port option.\n"),
-		  option);
-       fc_fprintf(stderr, _("Try using --help.\n"));
+      free(option);
+    } else if ((option = get_option_malloc("--port",argv,&i,argc))) {
+      if (sscanf(option, "%d", &server_port) != 1) {
+	fc_fprintf(stderr,
+		   _("Invalid port \"%s\" specified with --port option.\n"),
+		   option);
+	fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
-     }
-   } else if ((option = get_option("--server",argv,&i,argc)))
+      }
+      free(option);
+    } else if ((option = get_option_malloc("--server", argv, &i, argc))) {
       sz_strlcpy(server_host, option);
-   else if (is_option("--autoconnect",argv[i]))
+      free(option);
+    } else if (is_option("--autoconnect", argv[i])) {
       auto_connect = TRUE;
-   else if ((option = get_option("--debug",argv,&i,argc))) {
-      loglevel=log_parse_level_str(option);
-      if (loglevel==-1) {
+    } else if ((option = get_option_malloc("--debug", argv, &i, argc))) {
+      loglevel = log_parse_level_str(option);
+      if (loglevel == -1) {
 	fc_fprintf(stderr,
 		   _("Invalid debug level \"%s\" specified with --debug "
 		     "option.\n"), option);
 	fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
       }
-   } else if ((option = get_option("--tiles", argv, &i, argc)))
+      free(option);
+    } else if ((option = get_option_malloc("--tiles", argv, &i, argc))) {
       sz_strlcpy(tileset_name, option);
-   else if (is_option("--", argv[i])) {
-     ui_separator = TRUE;
-   } else { 
+      free(option);
+      user_tileset = TRUE;
+    } else if (is_option("--", argv[i])) {
+      ui_separator = TRUE;
+    } else {
       fc_fprintf(stderr, _("Unrecognized option: \"%s\"\n"), argv[i]);
       exit(EXIT_FAILURE);
-   }
-   i++;
+    }
+    i++;
   } /* of while */
 
   /* Remove all options except those intended for the UI. */
@@ -280,7 +336,7 @@ int main(int argc, char *argv[])
 
   /* after log_init: */
 
-  sz_strlcpy(default_user_name, user_username());
+  (void)user_username(default_user_name, MAX_LEN_NAME);
   if (!is_valid_username(default_user_name)) {
     char buf[sizeof(default_user_name)];
 
@@ -295,17 +351,20 @@ int main(int argc, char *argv[])
 
   /* initialization */
 
-  conn_list_init(&game.all_connections);
-  conn_list_init(&game.est_connections);
-  conn_list_init(&game.game_connections);
+  game.all_connections = conn_list_new();
+  game.est_connections = conn_list_new();
 
   ui_init();
   charsets_init();
   my_init_network();
+
+  /* register exit handler */ 
+  atexit(at_exit);
+
   chatline_common_init();
-  init_messages_where();
-  init_city_report_data();
+  message_options_init();
   init_player_dlg_common();
+  init_themes();
   settable_options_init();
 
   load_general_options();
@@ -325,25 +384,24 @@ int main(int argc, char *argv[])
     sz_strlcpy(metaserver, default_metaserver); 
   if (server_port == -1) server_port = default_server_port;
 
-
   /* This seed is not saved anywhere; randoms in the client should
      have cosmetic effects only (eg city name suggestions).  --dwp */
   mysrand(time(NULL));
-
+  helpdata_init();
   boot_help_texts();
-  if (!tilespec_read_toplevel(tileset_name)) {
-    /* get tile sizes etc */
-    exit(EXIT_FAILURE);
-  }
+
+  tilespec_try_read(tileset_name, user_tileset);
 
   audio_real_init(sound_set_name, sound_plugin_name);
   audio_play_music("music_start", NULL);
 
+  ggz_initialize();
+
   /* run gui-specific client */
   ui_main(argc, argv);
 
   /* termination */
-  ui_exit();
+  client_exit();
   
   /* not reached */
   return EXIT_SUCCESS;
@@ -352,14 +410,26 @@ int main(int argc, char *argv[])
 /**************************************************************************
 ...
 **************************************************************************/
-void ui_exit(void)
+void client_exit(void)
 {
   attribute_flush();
   client_remove_all_cli_conn();
-  my_shutdown_network();
 
+  if (save_options_on_exit) {
+    save_options();
+  }
+  
+  tileset_free(tileset);
+  
+  ui_exit();
+  
+  chatline_common_done();
+  message_options_free();
   client_game_free();
 
+  helpdata_done(); /* client_exit() unlinks help text list */
+  conn_list_free(game.all_connections);
+  conn_list_free(game.est_connections);
   exit(EXIT_SUCCESS);
 }
 
@@ -367,7 +437,7 @@ void ui_exit(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void handle_packet_input(void *packet, int type)
+void client_packet_input(void *packet, int type)
 {
   if (!client_handle_packet(type, packet)) {
     freelog(LOG_ERROR, "Received unknown packet (type %d) from server!",
@@ -409,20 +479,11 @@ void send_turn_done(void)
 
   attribute_flush();
 
-  send_packet_player_turn_done(&aconnection);
+  dsend_packet_player_phase_done(&aconnection, game.info.turn);
 
   update_turn_done_button_state();
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-void send_goto_unit(struct unit *punit, struct tile *dest_tile)
-{
-  dsend_packet_unit_goto(&aconnection, punit->id,
-			 dest_tile->x, dest_tile->y);
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -431,99 +492,81 @@ void send_report_request(enum report_type type)
   dsend_packet_report_req(&aconnection, type);
 }
 
-/**************************************************************************
- called whenever client is changed to pre-game state.
-**************************************************************************/
-void client_game_init()
-{
-  game_init();
-  attribute_init();
-  agents_init();
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void client_game_free()
-{
-  free_client_goto();
-  free_help_texts();
-  attribute_free();
-  agents_free();
-  game_free();
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
 void set_client_state(enum client_states newstate)
 {
-  bool connect_error = (client_state == CLIENT_PRE_GAME_STATE)
-      && (newstate == CLIENT_PRE_GAME_STATE);
-  enum client_states oldstate = client_state;
+  bool connect_error = (C_S_PREPARING == civclient_state)
+      && (C_S_PREPARING == newstate);
+  enum client_states oldstate = civclient_state;
 
-  if (newstate == CLIENT_GAME_OVER_STATE) {
+  if (C_S_OVER == newstate) {
     /*
      * Extra kludge for end-game handling of the CMA.
      */
-    city_list_iterate(game.player_ptr->cities, pcity) {
-      if (cma_is_city_under_agent(pcity, NULL)) {
-        cma_release_city(pcity);
-      }
-    } city_list_iterate_end;
+    if (game.player_ptr) {
+      city_list_iterate(game.player_ptr->cities, pcity) {
+	if (cma_is_city_under_agent(pcity, NULL)) {
+	  cma_release_city(pcity);
+	}
+      } city_list_iterate_end;
+    }
     popdown_all_city_dialogs();
     popdown_all_game_dialogs();
     set_unit_focus(NULL);
   }
 
-  if (client_state != newstate) {
+  if (civclient_state != newstate) {
 
     /* If changing from pre-game state to _either_ select race
-       or running state, then we have finished getting ruleset data,
-       and should translate data, for joining running game or for
-       selecting nations.  (Want translated nation names in nation
-       select dialog.)
+       or running state, then we have finished getting ruleset data.
     */
-    if (client_state==CLIENT_PRE_GAME_STATE
-	&& (newstate==CLIENT_SELECT_RACE_STATE
-	    || newstate==CLIENT_GAME_RUNNING_STATE)) {
-      translate_data_names();
+    if (C_S_PREPARING == civclient_state
+	&& C_S_RUNNING == newstate) {
       audio_stop();		/* stop intro sound loop */
     }
       
-    client_state=newstate;
+    civclient_state = newstate;
 
-    if (client_state == CLIENT_GAME_RUNNING_STATE) {
+    switch (civclient_state) {
+    case C_S_RUNNING:
+      init_city_report_game_data();
       load_ruleset_specific_options();
       create_event(NULL, E_GAME_START, _("Game started."));
       precalc_tech_data();
-      update_research(game.player_ptr);
+      if (game.player_ptr) {
+	update_research(game.player_ptr);
+      }
       role_unit_precalcs();
       boot_help_texts();	/* reboot */
       can_slide = FALSE;
       update_unit_focus();
       can_slide = TRUE;
       set_client_page(PAGE_GAME);
-    }
-    else if (client_state == CLIENT_PRE_GAME_STATE) {
+      break;
+    case C_S_PREPARING:
       popdown_all_city_dialogs();
-      popdown_all_game_dialogs();
       close_all_diplomacy_dialogs();
-      set_unit_focus(NULL);
+      popdown_all_game_dialogs();
       clear_notify_window();
-      if (oldstate != CLIENT_BOOT_STATE) {
+      if (C_S_INITIAL != oldstate) {
 	client_game_free();
       }
       client_game_init();
-      if (!aconnection.established) {
-	set_client_page(PAGE_MAIN);
+      set_unit_focus(NULL);
+      if (!aconnection.established && !with_ggz) {
+	set_client_page(in_ggz ? PAGE_GGZ : PAGE_MAIN);
       } else {
 	set_client_page(PAGE_START);
       }
-    }
+      break;
+    default:
+      break;
+    };
     update_menus();
   }
-  if (!aconnection.established && client_state == CLIENT_PRE_GAME_STATE) {
+  if (!aconnection.established && C_S_PREPARING == civclient_state) {
     gui_server_connect();
     if (auto_connect) {
       if (connect_error) {
@@ -531,7 +574,7 @@ void set_client_state(enum client_states newstate)
 		_("There was an error while auto connecting; aborting."));
 	exit(EXIT_FAILURE);
       } else {
-	server_autoconnect();
+	start_autoconnecting_to_server();
 	auto_connect = FALSE;	/* don't try this again */
       }
     } 
@@ -544,9 +587,9 @@ void set_client_state(enum client_states newstate)
 /**************************************************************************
 ...
 **************************************************************************/
-enum client_states get_client_state(void)
+enum client_states client_state(void)
 {
-  return client_state;
+  return civclient_state;
 }
 
 /**************************************************************************
@@ -555,11 +598,10 @@ enum client_states get_client_state(void)
 void client_remove_cli_conn(struct connection *pconn)
 {
   if (pconn->player) {
-    conn_list_unlink(&pconn->player->connections, pconn);
+    conn_list_unlink(pconn->player->connections, pconn);
   }
-  conn_list_unlink(&game.all_connections, pconn);
-  conn_list_unlink(&game.est_connections, pconn);
-  conn_list_unlink(&game.game_connections, pconn);
+  conn_list_unlink(game.all_connections, pconn);
+  conn_list_unlink(game.est_connections, pconn);
   assert(pconn != &aconnection);
   free(pconn);
 }
@@ -570,15 +612,12 @@ void client_remove_cli_conn(struct connection *pconn)
 **************************************************************************/
 void client_remove_all_cli_conn(void)
 {
-  while (conn_list_size(&game.all_connections) > 0) {
-    struct connection *pconn = conn_list_get(&game.all_connections, 0);
+  while (conn_list_size(game.all_connections) > 0) {
+    struct connection *pconn = conn_list_get(game.all_connections, 0);
     client_remove_cli_conn(pconn);
   }
 }
 
-void dealloc_id(int id); /* double kludge (suppress a possible warning) */
-void dealloc_id(int id) { }/* kludge */
-
 /**************************************************************************
 ..
 **************************************************************************/
@@ -604,49 +643,97 @@ bool client_is_observer(void)
   return aconnection.established && aconnection.observer;
 }
 
+/* Seconds_to_turndone is the number of seconds the server has told us
+ * are left.  The timer tells exactly how much time has passed since the
+ * server gave us that data. */
+static double seconds_to_turndone = 0.0;
+static struct timer *turndone_timer;
+
+/* This value shows what value the timeout label is currently showing for
+ * the seconds-to-turndone. */
+static int seconds_shown_to_turndone;
+
 /**************************************************************************
- This function should be called every 500ms. It lets the unit blink
- and update the timeout.
+  Reset the number of seconds to turndone from an "authentic" source.
+
+  The seconds are taken as a double even though most callers will just
+  know an integer value.
 **************************************************************************/
-void real_timer_callback(void)
+void set_seconds_to_turndone(double seconds)
 {
-  static bool flip = FALSE;
+  if (game.info.timeout > 0) {
+    seconds_to_turndone = seconds;
+    turndone_timer = renew_timer_start(turndone_timer, TIMER_USER,
+				       TIMER_ACTIVE);
 
-  if (get_client_state() != CLIENT_GAME_RUNNING_STATE) {
-    return;
+    /* Maybe we should do an update_timeout_label here, but it doesn't
+     * seem to be necessary. */
+    seconds_shown_to_turndone = ceil(seconds) + 0.1;
+  }
+}
+
+/**************************************************************************
+  Return the number of seconds until turn-done.  Don't call this unless
+  game.info.timeout != 0.
+**************************************************************************/
+int get_seconds_to_turndone(void)
+{
+  if (game.info.timeout > 0) {
+    return seconds_shown_to_turndone;
+  } else {
+    /* This shouldn't happen. */
+    return FC_INFINITY;
+  }
+}
+
+/**************************************************************************
+  This function should be called at least once per second.  It does various
+  updates (idle animations and timeout updates).  It returns the number of
+  seconds until it should be called again.
+**************************************************************************/
+double real_timer_callback(void)
+{
+  double time_until_next_call = 1.0;
+
+  {
+    double autoconnect_time = try_to_autoconnect();
+    time_until_next_call = MIN(time_until_next_call, autoconnect_time);
   }
 
-  if (game.player_ptr->is_connected && game.player_ptr->is_alive &&
-      !game.player_ptr->turn_done) {
-    int is_waiting = 0, is_moving = 0;
+  if (C_S_RUNNING != client_state()) {
+    return time_until_next_call;
+  }
 
-    players_iterate(pplayer) {
-      if (pplayer->is_alive && pplayer->is_connected) {
-	if (pplayer->turn_done) {
-	  is_waiting++;
-	} else {
-	  is_moving++;
-	}
-      }
-    } players_iterate_end;
+  {
+    double blink_time = blink_turn_done_button();
 
-    if (is_moving == 1 && is_waiting > 0) {
-      update_turn_done_button(FALSE);	/* stress the slow player! */
+    time_until_next_call = MIN(time_until_next_call, blink_time);
+  }
+
+  if (get_num_units_in_focus() > 0) {
+    double blink_time = blink_active_unit();
+
+    time_until_next_call = MIN(time_until_next_call, blink_time);
+  }
+
+  /* It is possible to have game.info.timeout > 0 but !turndone_timer, in the
+   * first moments after the timeout is set. */
+  if (game.info.timeout > 0 && turndone_timer) {
+    double seconds = seconds_to_turndone - read_timer_seconds(turndone_timer);
+    int iseconds = ceil(seconds) + 0.1; /* Turn should end right on 0. */
+
+    if (iseconds < seconds_shown_to_turndone) {
+      seconds_shown_to_turndone = iseconds;
+      update_timeout_label();
     }
+
+    time_until_next_call = MIN(time_until_next_call,
+			       seconds - floor(seconds) + 0.001);
   }
 
-  blink_active_unit();
-
-  if (flip) {
-    update_timeout_label();
-    if (seconds_to_turndone > 0) {
-      seconds_to_turndone--;
-    } else {
-      seconds_to_turndone = 0;
-    }
-  }
-
-  flip = !flip;
+  /* Make sure we wait at least 50 ms, otherwise we may not give any other
+   * code time to run. */
+  return MAX(time_until_next_call, 0.05);
 }
 
 /**************************************************************************
@@ -656,27 +743,31 @@ void real_timer_callback(void)
 **************************************************************************/
 bool can_client_issue_orders(void)
 {
-  return (!client_is_observer()
-	  && get_client_state() == CLIENT_GAME_RUNNING_STATE);
+  return (game.player_ptr
+	  && !client_is_observer()
+	  && C_S_RUNNING == client_state());
 }
 
 /**************************************************************************
   Returns TRUE iff the client can do diplomatic meetings with another 
   given player.
 **************************************************************************/
-bool can_meet_with_player(struct player *pplayer)
+bool can_meet_with_player(const struct player *pplayer)
 {
-  return (could_meet_with_player(game.player_ptr, pplayer)
-          && can_client_issue_orders());
+  return (can_client_issue_orders()
+	  && game.player_ptr
+	  && could_meet_with_player(game.player_ptr, pplayer));
 }
 
 /**************************************************************************
   Returns TRUE iff the client can get intelligence from another 
   given player.
 **************************************************************************/
-bool can_intel_with_player(struct player *pplayer)
+bool can_intel_with_player(const struct player *pplayer)
 {
-  return could_intel_with_player(game.player_ptr, pplayer);
+  return (client_is_observer()
+	  || (game.player_ptr
+	      && could_intel_with_player(game.player_ptr, pplayer)));
 }
 
 /**************************************************************************
@@ -686,6 +777,30 @@ bool can_intel_with_player(struct player *pplayer)
 **************************************************************************/
 bool can_client_change_view(void)
 {
-  return (get_client_state() == CLIENT_GAME_RUNNING_STATE
-	  || get_client_state() == CLIENT_GAME_OVER_STATE);
+  return ((game.player_ptr || client_is_observer())
+	  && (C_S_RUNNING == client_state()
+	      || C_S_OVER == client_state()));
+}
+
+/**************************************************************************
+  Sets if server is considered busy. Currently it is considered busy
+  between turns.
+**************************************************************************/
+void set_server_busy(bool busy)
+{
+  if (busy != server_busy) {
+    /* server_busy value will change */
+    server_busy = busy;
+
+    /* This may mean that we have to change from or to wait cursor */
+    control_mouse_cursor(NULL);
+  }
+}
+
+/**************************************************************************
+  Returns if server is considered busy at the moment
+**************************************************************************/
+bool is_server_busy(void)
+{
+  return server_busy;
 }
