@@ -33,6 +33,7 @@
 #include "control.h"
 #include "goto.h"
 #include "text.h"
+#include "unitlist.h"
 
 #include "gui_text.h"
 
@@ -134,17 +135,21 @@ static void real_add(char **buffer, size_t * buffer_size, const char *format,
 ****************************************************************************/
 const char *mapview_get_terrain_tooltip_text(struct tile *ptile)
 {
-  int infrastructure = get_tile_infrastructure_set(ptile);
+  int count;
+    
+  bv_special infrastructure = get_tile_infrastructure_set(ptile, &count);
+               
   INIT;
 
 #ifdef DEBUG
   add_line(_("Location: (%d, %d) [%d]"),
-	   ptile->x, ptile->y, ptile->continent);
+	   TILE_XY(ptile),
+	   ptile->continent);
 #endif
-  add_line("%s", map_get_tile_info_text(ptile));
-  if (infrastructure) {
+  add_line("%s", tile_get_info_text(ptile));
+  if (count > 0) {
     add_line("%s",
-	     map_get_infrastructure_text(infrastructure));
+	     get_infrastructure_text(infrastructure));
   }
   RETURN;
 }
@@ -158,41 +163,30 @@ static void calc_effect(enum unit_activity activity, struct tile *ptile,
   struct tile backup = *ptile;
   int stats_before[3], stats_after[3];
 
-  stats_before[0] = get_food_tile(ptile);
-  stats_before[1] = get_shields_tile(ptile);
-  stats_before[2] = get_trade_tile(ptile);
+  stats_before[0] = get_output_tile(ptile, O_FOOD);
+  stats_before[1] = get_output_tile(ptile, O_SHIELD);
+  stats_before[2] = get_output_tile(ptile, O_TRADE);
 
   /* BEWARE UGLY HACK AHEAD */
 
   switch (activity) {
   case ACTIVITY_ROAD:
-    map_set_special(ptile, S_ROAD);
-    break;
   case ACTIVITY_RAILROAD:
-    map_set_special(ptile, S_RAILROAD);
-    break;
   case ACTIVITY_MINE:
-    map_mine_tile(ptile);
-    break;
-
   case ACTIVITY_IRRIGATE:
-    map_irrigate_tile(ptile);
-    break;
-
   case ACTIVITY_TRANSFORM:
-    map_transform_tile(ptile);
+    tile_apply_activity(ptile, activity);
     break;
   default:
     assert(0);
   }
 
-  stats_after[0] = get_food_tile(ptile);
-  stats_after[1] = get_shields_tile(ptile);
-  stats_after[2] = get_trade_tile(ptile);
+  stats_after[0] = get_output_tile(ptile, O_FOOD);
+  stats_after[1] = get_output_tile(ptile, O_SHIELD);
+  stats_after[2] = get_output_tile(ptile, O_TRADE);
 
   ptile->terrain = backup.terrain;
   ptile->special = backup.special;
-  reset_move_costs(ptile);
   /* hopefully everything is now back in place */
 
   diff[0] = stats_after[0] - stats_before[0];
@@ -304,23 +298,23 @@ const char *mapview_get_unit_action_tooltip(struct unit *punit,
   } else {
 #if 0
   ttype = punit->tile->terrain;
-  tinfo = get_tile_type(ttype);
+  tinfo = terrain_by_number(ttype);
   if ((tinfo->irrigation_result != T_LAST)
       && (tinfo->irrigation_result != ttype)) {
     my_snprintf(irrtext, sizeof(irrtext), irrfmt,
-		(get_tile_type(tinfo->irrigation_result))->terrain_name);
-  } else if (map_has_special(punit->tile, S_IRRIGATION)
+		terrain_name_translation(tinfo->irrigation_result));
+  } else if (tile_has_special(punit->tile, S_IRRIGATION)
 	     && player_knows_techs_with_flag(game.player_ptr, TF_FARMLAND)) {
     sz_strlcpy(irrtext, _("Bu_ild Farmland"));
   }
   if ((tinfo->mining_result != T_LAST) && (tinfo->mining_result != ttype)) {
     my_snprintf(mintext, sizeof(mintext), minfmt,
-		(get_tile_type(tinfo->mining_result))->terrain_name);
+		terrain_name_translation(tinfo->mining_result));
   }
   if ((tinfo->transform_result != T_LAST)
       && (tinfo->transform_result != ttype)) {
     my_snprintf(transtext, sizeof(transtext), transfmt,
-		(get_tile_type(tinfo->transform_result))->terrain_name);
+		terrain_name_translation(tinfo->transform_result));
   }
 
   menus_rename("<main>/_Orders/Build _Irrigation", irrtext);
@@ -347,18 +341,17 @@ const char *mapview_get_city_action_tooltip(struct city *pcity,
   if (strcmp(action, "city_buy") == 0) {
     const char *name;
 
-    if (pcity->is_building_unit) {
-      name = get_unit_type(pcity->currently_building)->name;
+    if (pcity->production.is_unit) {
+      name = utype_name_translation(utype_by_number(pcity->production.value));
     } else {
-      name = get_impr_name_ex(pcity, pcity->currently_building);
+      name = get_impr_name_ex(pcity, pcity->production.value);
     }
 
     add_line(_("Buy production"));
     add_line(_("Cost: %d (%d in treasury)"),
 	     city_buy_cost(pcity), game.player_ptr->economic.gold);
     add_line(_("Producting: %s (%d turns)"), name,
-	     city_turns_to_build(pcity, pcity->currently_building,
-				 pcity->is_building_unit, TRUE));
+	     city_turns_to_build(pcity, pcity->production, TRUE));
   } else {
     add_line("tooltip for action %s isn't written yet", action);
     freelog(LOG_NORMAL,
@@ -367,59 +360,6 @@ const char *mapview_get_city_action_tooltip(struct city *pcity,
   RETURN;
 }  
 
-/************************************************************************
-  Text to popup on middle-click
-************************************************************************/
-const char *mapview_get_terrain_info_text(struct tile *ptile)
-{
-  const char *activity_text = concat_tile_activity_text(ptile);
-  const char *diplo_nation_plural_adjectives[DS_LAST] =
-    {Q_("?nation:Neutral"), Q_("?nation:Hostile"),
-     "" /* unused, DS_CEASEFIRE*/,
-     Q_("?nation:Peaceful"), Q_("?nation:Friendly"), 
-     Q_("?nation:Mysterious")};
-  INIT;
-
-  add_line(_("Terrain: %s"),
-	   map_get_tile_info_text(ptile));
-  add_line(_("Food/Prod/Trade: %s"),
-	   map_get_tile_fpt_text(ptile));
-  if (tile_has_special(ptile, S_HUT)) {
-    add_line(_("Minor Tribe Village"));
-  }
-  if (game.borders > 0) {
-    struct player *owner = map_get_owner(ptile);
-    struct player_diplstate *ds = game.player_ptr->diplstates;
-
-    if (owner == game.player_ptr){
-      add_line(_("Our Territory"));
-    } else if (owner) {
-      if (ds[owner->player_no].type == DS_CEASEFIRE) {
-	int turns = ds[owner->player_no].turns_left;
-
-	add_line(PL_("%s territory (%d turn ceasefire)",
-				       "%s territory (%d turn ceasefire)",
-				       turns),
-		 get_nation_name(owner->nation), turns);
-      } else {
-	add_line(_("Territory of the %s %s"),
-		 diplo_nation_plural_adjectives[ds[owner->player_no].type],
-		 get_nation_name_plural(owner->nation));
-      }
-    } else {
-      add_line(_("Unclaimed territory"));
-    }
-  }
-  if (get_tile_infrastructure_set(ptile)) {
-    add_line(_("Infrastructure: %s"),
-	     map_get_infrastructure_text(ptile->special));
-  }
-  if (strlen(activity_text)) {
-    add_line(_("Activity: %s"), activity_text);
-  }
-  RETURN;
-}
-
 /****************************************************************************
   Get a short tooltip for a city.
 ****************************************************************************/
@@ -428,8 +368,8 @@ const char *mapview_get_city_tooltip_text(struct city *pcity)
   struct player *owner = city_owner(pcity);
   INIT;
 
-  add_line("%s", pcity->name);
-  add_line("%s", owner->name);
+  add_line("%s", city_name(pcity));
+  add_line("%s", player_name(owner));
   RETURN;
 }
 
@@ -438,11 +378,10 @@ const char *mapview_get_city_tooltip_text(struct city *pcity)
 ****************************************************************************/
 const char *mapview_get_city_info_text(struct city *pcity)
 {
-  struct player *owner = city_owner(pcity);
   INIT;
 
-  add_line(_("City: %s (%s)"), pcity->name,
-	   get_nation_name(owner->nation));
+  add_line(_("City: %s (%s)"), city_name(pcity),
+	   nation_adjective_for_player(city_owner(pcity)));
   if (city_got_citywalls(pcity)) {
     add(_(" with City Walls"));
   }
@@ -459,20 +398,21 @@ const char *mapview_get_unit_tooltip_text(struct unit *punit)
       player_find_city_by_id(game.player_ptr, punit->homecity);
   INIT;
 
-  add("%s", ptype->name);
+  add("%s", utype_name_translation(ptype));
   if (ptype->veteran[punit->veteran].name[0] != '\0') {
     add(" (%s)", ptype->veteran[punit->veteran].name);
   }
   add("\n");
   add_line("%s", unit_activity_text(punit));
   if (pcity) {
-    add_line("%s", pcity->name);
+    add_line("%s", city_name(pcity));
   }
   RETURN;
 }
 
 /****************************************************************************
   Get a longer tooltip for a unit.
+  FIXME: should use same method as client/text.c popup_info_text()
 ****************************************************************************/
 const char *mapview_get_unit_info_text(struct unit *punit)
 {
@@ -484,21 +424,31 @@ const char *mapview_get_unit_info_text(struct unit *punit)
     add_line(_("Activity: %s"), activity_text);
   }
   if (punit) {
-    char tmp[64] = { 0 };
+    struct player *owner = unit_owner(punit);
     struct unit_type *ptype = unit_type(punit);
 
-    if (punit->owner == game.player_idx) {
-      struct city *pcity =
-	  player_find_city_by_id(game.player_ptr, punit->homecity);
+    if (owner == game.player_ptr) {
+      struct city *pcity = player_find_city_by_id(owner, punit->homecity);
 
       if (pcity){
-	my_snprintf(tmp, sizeof(tmp), "/%s", pcity->name);
+	/* TRANS: "Unit: Musketeers (Polish, Warsaw)" */
+	add_line(&str, _("Unit: %s (%s, %s)"),
+		 utype_name_translation(ptype),
+		 nation_adjective_for_player(owner),
+		 city_name(pcity));
+      } else {
+	/* TRANS: "Unit: Musketeers (Polish)" */
+	add_line(&str, _("Unit: %s (%s)"),
+		 utype_name_translation(ptype),
+		 nation_adjective_for_player(owner));
       }
-    }
-    add_line(_("Unit: %s(%s%s)"), ptype->name,
-	     get_nation_name(unit_owner(punit)->nation), tmp);
-    if (punit->owner != game.player_idx) {
-      struct unit *apunit = get_unit_in_focus();
+    } else {
+      struct unit *apunit = head_of_units_in_focus();  /* FIXME, need best in stack */
+
+      /* TRANS: "Unit: Musketeers (Polish)" */
+      add_line(&str, _("Unit: %s (%s)"),
+               utype_name_translation(ptype),
+               nation_adjective_for_player(owner));
 
       if (apunit) {
 	/* chance to win when active unit is attacking the selected unit */
