@@ -52,8 +52,10 @@
 #include "cityrep.h"
 
 #define NEG_VAL(x)  ((x)<0 ? (x) : (-x))
-#define CMA_NONE	(-1)
-#define CMA_CUSTOM	(-2)
+
+/* Some versions of gcc have problems with negative values here (PR#39722). */
+#define CMA_NONE	(10000)
+#define CMA_CUSTOM	(10001)
 
 enum city_operation_type {
   CO_CHANGE, CO_LAST, CO_NEXT, CO_FIRST, CO_NONE
@@ -66,7 +68,8 @@ static void city_model_init(void);
 static void city_activated_callback(GtkTreeView *view, GtkTreePath *path,
 				    GtkTreeViewColumn *col, gpointer data);
 
-static void city_command_callback(struct gui_dialog *dlg, int response);
+static void city_command_callback(struct gui_dialog *dlg, int response,
+                                  gpointer data);
 
 static void city_selection_changed_callback(GtkTreeSelection *selection);
 
@@ -93,8 +96,7 @@ static void popup_first_menu(GtkMenuShell *menu, gpointer data);
 static void popup_next_menu(GtkMenuShell *menu, gpointer data);
 
 static GtkWidget *city_center_command, *city_popup_command, *city_buy_command;
-static GtkWidget *city_change_command;
-static GtkWidget *city_last_command, *city_first_command, *city_next_command;
+static GtkWidget *city_production_command;
 
 
 static GtkWidget *change_improvements_item;
@@ -138,7 +140,7 @@ bool select_menu_cached;
 /****************************************************************
  Return text line for the column headers for the city report
 *****************************************************************/
-static void get_city_table_header(char *text[], int n)
+static void get_city_table_header(char **text, int n)
 {
   struct city_report_spec *spec;
   int i;
@@ -157,29 +159,23 @@ static void get_city_table_header(char *text[], int n)
 ****************************************************************/
 
 /****************************************************************
-...
+ Popup the city report dialog, and optionally raise it.
 ****************************************************************/
-void popup_city_report_dialog(bool make_modal)
+void popup_city_report_dialog(bool raise)
 {
   if(!city_dialog_shell) {
-    city_dialog_shell_is_modal = make_modal;
+    city_dialog_shell_is_modal = FALSE;
     
-    create_city_report_dialog(make_modal);
+    create_city_report_dialog(FALSE);
 
     select_menu_cached = FALSE;
   }
 
   gui_dialog_present(city_dialog_shell);
   hilite_cities_from_canvas();
-}
-
-/****************************************************************
- Raises the city report dialog.
-****************************************************************/
-void raise_city_report_dialog(void)
-{
-  popup_city_report_dialog(FALSE);
-  gui_dialog_raise(city_dialog_shell);
+  if (raise) {
+    gui_dialog_raise(city_dialog_shell);
+  }
 }
 
 /****************************************************************
@@ -192,12 +188,6 @@ void popdown_city_report_dialog(void)
   }
 }
 
-
-/****************************************************************
-...
-*****************************************************************/
-typedef bool (*TestCityFunc)(struct city *, gint);
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -211,9 +201,9 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 					     int size)
 {
   GtkWidget *menu;
-  cid cids[U_LAST + B_LAST];
-  struct item items[U_LAST + B_LAST];
-  int i, item, cids_used;
+  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct item items[MAX_NUM_PRODUCTION_TARGETS];
+  int i, item, targets_used;
   char *row[4];
   char buf[4][64];
   
@@ -249,15 +239,18 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
     }
 
     data = (struct city **)g_ptr_array_free(selected, FALSE);
-    cids_used = collect_cids1(cids, data, num_selected, append_units,
-        		      append_wonders, TRUE, test_func);
+    targets_used
+      = collect_production_targets(targets, data, num_selected, append_units,
+				   append_wonders, TRUE, test_func);
     g_free(data);
   } else {
-    cids_used = collect_cids1(cids, NULL, 0, append_units,
-			      append_wonders, FALSE, test_func);
+    targets_used = collect_production_targets(targets, NULL, 0, append_units,
+					      append_wonders, FALSE,
+					      test_func);
   }
 
-  name_and_sort_items(cids, cids_used, items, city_operation != CO_NONE, NULL);
+  name_and_sort_items(targets, targets_used, items,
+		      city_operation != CO_NONE, NULL);
 
   for (i = 0; i < 4; i++) {
     row[i] = buf[i];
@@ -271,13 +264,12 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
     group[i] = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
   }
   
-  for (item = 0; item < cids_used; item++) {
-    cid cid = items[item].cid;
+  for (item = 0; item < targets_used; item++) {
+    struct city_production target = items[item].item;
     GtkWidget *menu_item, *hbox, *label;
     char txt[256];
 
-    get_city_dialog_production_row(row, sizeof(buf[0]), cid_id(cid),
-				   cid_is_unit(cid), NULL);
+    get_city_dialog_production_row(row, sizeof(buf[0]), target, NULL);
 
     menu_item = gtk_menu_item_new();
     hbox = gtk_hbox_new(FALSE, 18);
@@ -306,7 +298,7 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
     g_signal_connect(menu_item, "activate", callback,
-		     GINT_TO_POINTER(items[item].cid));
+		     GINT_TO_POINTER(cid_encode(target)));
   }
   
   for (i = 0; i < 3; i++) {
@@ -315,7 +307,7 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
   
   gtk_widget_show_all(menu);
 
-  gtk_widget_set_sensitive(GTK_WIDGET(parent_item), (cids_used > 0));
+  gtk_widget_set_sensitive(GTK_WIDGET(parent_item), (targets_used > 0));
 }
 
 /****************************************************************
@@ -324,12 +316,12 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 static void impr_or_unit_iterate(GtkTreeModel *model, GtkTreePath *path,
 				 GtkTreeIter *it, gpointer data)
 {
-  cid cid = GPOINTER_TO_INT(data);
+  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
   gint id;
 
   gtk_tree_model_get(model, it, 1, &id, -1);
 
-  city_change_production(find_city_by_id(id), cid_is_unit(cid), cid_id(cid));
+  city_change_production(game_find_city_by_number(id), target);
 }
 
 /****************************************************************
@@ -343,14 +335,14 @@ static void worklist_last_impr_or_unit_iterate(GtkTreeModel *model,
 						 GtkTreeIter *it, 
 						 gpointer data)
 {
-  cid cid = GPOINTER_TO_INT(data);
+  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
   gint id;
   struct city *pcity;  
 
   gtk_tree_model_get(model, it, 1, &id, -1);
-  pcity = find_city_by_id(id);
+  pcity = game_find_city_by_number(id);
 
-  (void) city_queue_insert(pcity, -1, cid_is_unit(cid), cid_id(cid));
+  (void) city_queue_insert(pcity, -1, target);
   /* perhaps should warn the user if not successful? */
 }
 
@@ -367,14 +359,14 @@ static void worklist_first_impr_or_unit_iterate(GtkTreeModel *model,
 						 GtkTreeIter *it, 
 						 gpointer data)
 {
-  cid cid = GPOINTER_TO_INT(data);
+  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
   gint id;
   struct city *pcity;  
 
   gtk_tree_model_get(model, it, 1, &id, -1);
-  pcity = find_city_by_id(id);
+  pcity = game_find_city_by_number(id);
 
-  (void) city_queue_insert(pcity, 0, cid_is_unit(cid), cid_id(cid));
+  (void) city_queue_insert(pcity, 0, target);
   /* perhaps should warn the user if not successful? */
 }
 
@@ -391,12 +383,12 @@ static void worklist_next_impr_or_unit_iterate(GtkTreeModel *model,
 {
   struct city *pcity;
   gint id;
-  cid cid = GPOINTER_TO_INT(data);
+  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
 
   gtk_tree_model_get(model, it, 1, &id, -1);
-  pcity = find_city_by_id(id);
+  pcity = game_find_city_by_number(id);
 
-  (void) city_queue_insert(pcity, 1, cid_is_unit(cid), cid_id(cid));
+  (void) city_queue_insert(pcity, 1, target);
   /* perhaps should warn the user if not successful? */
 }
 
@@ -405,7 +397,7 @@ static void worklist_next_impr_or_unit_iterate(GtkTreeModel *model,
 *****************************************************************/
 static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
 {
-  cid cid = GPOINTER_TO_INT(data);
+  struct city_production target = cid_decode(GPOINTER_TO_INT(data));
   GObject *parent = G_OBJECT(w->parent);
   TestCityFunc test_func = g_object_get_data(parent, "freeciv_test_func");
   enum city_operation_type city_operation = 
@@ -424,7 +416,7 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
       itree_get(&it, 0, &res, -1);
       pcity = res;
 
-      if (test_func(pcity, cid)) {
+      if (test_func(pcity, target)) {
 	itree_select(city_selection, &it);
       }
     }
@@ -434,21 +426,22 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
     case CO_LAST:
       gtk_tree_selection_selected_foreach(city_selection,
 					  worklist_last_impr_or_unit_iterate,
-					  GINT_TO_POINTER(cid));
+					  GINT_TO_POINTER(cid_encode(target)));
       break;
     case CO_CHANGE:
       gtk_tree_selection_selected_foreach(city_selection,
-					  impr_or_unit_iterate, GINT_TO_POINTER(cid));
+					  impr_or_unit_iterate,
+					  GINT_TO_POINTER(cid_encode(target)));
       break;
     case CO_FIRST:
       gtk_tree_selection_selected_foreach(city_selection,
 					  worklist_first_impr_or_unit_iterate,
-					  GINT_TO_POINTER(cid));
+					  GINT_TO_POINTER(cid_encode(target)));
       break;
     case CO_NEXT:
       gtk_tree_selection_selected_foreach(city_selection,
 					  worklist_next_impr_or_unit_iterate,
-					  GINT_TO_POINTER(cid));
+					  GINT_TO_POINTER(cid_encode(target)));
       break;
     default:
       assert(FALSE); /* should never get here. */
@@ -544,16 +537,24 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
   GtkWidget *w;
 
   gtk_menu_item_remove_submenu(parent_item);
+  if (!can_client_issue_orders()) {
+    return;
+  }
   menu = gtk_menu_new();
   gtk_menu_item_set_submenu(parent_item, menu);
 
   if (change_cma) {
-    for (i = -1; i < cmafec_preset_num(); i++) {
-      w = (i == -1 ? gtk_menu_item_new_with_label(_("none"))
-	   : gtk_menu_item_new_with_label(cmafec_preset_get_descr(i)));
+    w = gtk_menu_item_new_with_label(_("none"));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), w);
+    g_signal_connect(w, "activate", G_CALLBACK(select_cma_callback),
+		     GINT_TO_POINTER(CMA_NONE));
+
+    for (i = 0; i < cmafec_preset_num(); i++) {
+      w = gtk_menu_item_new_with_label(cmafec_preset_get_descr(i));
       gtk_menu_shell_append(GTK_MENU_SHELL(menu), w);
       g_signal_connect(w, "activate", G_CALLBACK(select_cma_callback),
 		       GINT_TO_POINTER(i));
+      assert(GPOINTER_TO_INT(GINT_TO_POINTER(i)) == i);
     }
   } else {
     /* search for a "none" */
@@ -659,7 +660,7 @@ static void update_view_menu(GtkWidget *show_item)
   int i;
 
   menu = gtk_menu_new();
-  for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {
+  for(i=0, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {
     item = gtk_check_menu_item_new_with_label(spec->explanation);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), spec->show);
@@ -673,38 +674,46 @@ static void update_view_menu(GtkWidget *show_item)
 *****************************************************************/
 static GtkWidget *create_city_report_menubar(void)
 {
-  GtkWidget *menubar, *item;
+  GtkWidget *vbox, *sep, *menubar, *menu, *item;
+
+  vbox = gtk_vbox_new(FALSE, 0);
+  sep = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
 
   menubar = gtk_menu_bar_new();
+  gtk_box_pack_start(GTK_BOX(vbox), menubar, TRUE, TRUE, 0);
   
-  item = gtk_menu_item_new_with_mnemonic(_("Chan_ge"));
+  item = gtk_menu_item_new_with_mnemonic(_("_Production"));
+  city_production_command = item;
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
-  city_change_command = item;
+
+  menu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+
+  item = gtk_menu_item_new_with_mnemonic(_("Chan_ge"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   create_change_menu(item);
 
   item = gtk_menu_item_new_with_mnemonic(_("Add _First"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
-  city_first_command = item;
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   create_first_menu(item);
 
   item = gtk_menu_item_new_with_mnemonic(_("Add _Next"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
-  city_next_command = item;
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   create_next_menu(item);
 
   item = gtk_menu_item_new_with_mnemonic(_("Add _Last"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
-  city_last_command = item;
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   create_last_menu(item);
 
   item = gtk_menu_item_new_with_mnemonic(_("_Select"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   create_select_menu(item);
 
-  item = gtk_menu_item_new_with_mnemonic(_("S_how"));
+  item = gtk_menu_item_new_with_mnemonic(_("_Display"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   update_view_menu(item);
-  return menubar;
+  return vbox;
 }
 
 /****************************************************************
@@ -728,7 +737,8 @@ static void cityrep_cell_data_func(GtkTreeViewColumn *col,
   g_value_unset(&value);
 
   sp = &city_report_specs[n];
-  my_snprintf(buf, sizeof(buf), "%*s", NEG_VAL(sp->width), (sp->func)(pcity));
+  my_snprintf(buf, sizeof(buf), "%*s", NEG_VAL(sp->width),
+	      (sp->func)(pcity, sp->data));
 
   g_value_init(&value, G_TYPE_STRING);
   g_value_set_string(&value, buf);
@@ -760,8 +770,10 @@ static gint cityrep_sort_func(GtkTreeModel *model,
   g_value_unset(&value);
 
   sp = &city_report_specs[n];
-  my_snprintf(buf1, sizeof(buf1), "%*s",NEG_VAL(sp->width),(sp->func)(pcity1));
-  my_snprintf(buf2, sizeof(buf2), "%*s",NEG_VAL(sp->width),(sp->func)(pcity2));
+  my_snprintf(buf1, sizeof(buf1), "%*s", NEG_VAL(sp->width),
+	      (sp->func)(pcity1, sp->data));
+  my_snprintf(buf2, sizeof(buf2), "%*s", NEG_VAL(sp->width),
+	      (sp->func)(pcity2, sp->data));
 
   return cityrepfield_compare(buf1, buf2);
 }
@@ -771,14 +783,14 @@ static gint cityrep_sort_func(GtkTreeModel *model,
 *****************************************************************/
 static void create_city_report_dialog(bool make_modal)
 {
-  static char *titles [NUM_CREPORT_COLS];
-  static char  buf    [NUM_CREPORT_COLS][64];
+  static char **titles;
+  static char (*buf)[128];
   struct city_report_spec *spec;
 
   GtkWidget *w, *sw, *menubar;
   int i;
-  
-  gui_dialog_new(&city_dialog_shell, GTK_NOTEBOOK(top_notebook));
+
+  gui_dialog_new(&city_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL);
   gui_dialog_set_title(city_dialog_shell, _("Cities"));
 
   gui_dialog_set_default_size(city_dialog_shell, -1, 420);
@@ -788,8 +800,7 @@ static void create_city_report_dialog(bool make_modal)
 
   /* menubar */
   menubar = create_city_report_menubar();
-  gtk_box_pack_start(GTK_BOX(city_dialog_shell->vbox),
-	menubar, FALSE, FALSE, 0);
+  gui_dialog_add_widget(city_dialog_shell, menubar);
 
   /* buttons */
   w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_FIT,
@@ -797,7 +808,7 @@ static void create_city_report_dialog(bool make_modal)
   city_center_command = w;
 
   w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_IN,
-      _("_Popup"), CITY_POPUP);
+      _("_Inspect"), CITY_POPUP);
   city_popup_command = w;
 
   w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_EXECUTE,
@@ -811,9 +822,11 @@ static void create_city_report_dialog(bool make_modal)
 				  GTK_RESPONSE_CLOSE);
 
   /* tree view */
-  for (i=0; i<NUM_CREPORT_COLS; i++)
+  buf = fc_realloc(buf, NUM_CREPORT_COLS * sizeof(buf[0]));
+  titles = fc_realloc(titles, NUM_CREPORT_COLS * sizeof(titles[0]));
+  for (i = 0; i < NUM_CREPORT_COLS; i++) {
     titles[i] = buf[i];
-
+  }
   get_city_table_header(titles, sizeof(buf[0]));
 
   city_model = gtk_list_store_new(2, G_TYPE_POINTER, G_TYPE_INT);
@@ -937,8 +950,8 @@ static void same_island_iterate(GtkTreeModel *model, GtkTreePath *path,
     itree_get(&it, 0, &res, -1);
     pcity = res;
 
-    if (map_get_continent(pcity->tile)
-	== map_get_continent(selectedcity->tile)) {
+    if (tile_get_continent(pcity->tile)
+	== tile_get_continent(selectedcity->tile)) {
       itree_select(city_selection, &it);
     }
   }
@@ -970,11 +983,11 @@ static void city_select_building_callback(GtkMenuItem *item, gpointer data)
     itree_get(&it, 0, &res, -1);
     pcity = res;
 
-    if ( (which == TYPE_UNIT && pcity->is_building_unit)
-         || (which == TYPE_NORMAL_IMPROVEMENT && !pcity->is_building_unit
-             && !is_wonder(pcity->currently_building))
-         || (which == TYPE_WONDER && !pcity->is_building_unit
-             && is_wonder(pcity->currently_building)) ) {
+    if ( (which == TYPE_UNIT && pcity->production.is_unit)
+         || (which == TYPE_NORMAL_IMPROVEMENT && !pcity->production.is_unit
+             && !is_wonder(pcity->production.value))
+         || (which == TYPE_WONDER && !pcity->production.is_unit
+             && is_wonder(pcity->production.value)) ) {
       itree_select(city_selection, &it);
     }
   }
@@ -1023,13 +1036,14 @@ static void popup_iterate(GtkTreeModel *model, GtkTreePath *path,
     center_tile_mapcanvas(pcity->tile);
   }
 
-  popup_city_dialog(pcity, 0);
+  popup_city_dialog(pcity);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-static void city_command_callback(struct gui_dialog *dlg, int response)
+static void city_command_callback(struct gui_dialog *dlg, int response,
+                                  gpointer data)
 {
   switch (response) {
   case CITY_CENTER:
@@ -1095,8 +1109,7 @@ static void update_row(GtkTreeIter *row, struct city *pcity)
 *****************************************************************/
 static void city_model_init(void)
 {
-  if (city_dialog_shell && !is_report_dialogs_frozen()) {
-
+  if (city_dialog_shell && !is_report_dialogs_frozen() && game.player_ptr) {
     city_list_iterate(game.player_ptr->cities, pcity) {
       GtkTreeIter it;
 
@@ -1134,14 +1147,16 @@ void city_report_dialog_update(void)
     /* update. */
     gtk_list_store_clear(city_model);
 
-    city_list_iterate(game.player_ptr->cities, pcity) {
-      gtk_list_store_append(city_model, &it);
-      update_row(&it, pcity);
+    if (game.player_ptr) {
+      city_list_iterate(game.player_ptr->cities, pcity) {
+	gtk_list_store_append(city_model, &it);
+	update_row(&it, pcity);
 
-      if (g_hash_table_remove(copy, pcity)) {
-	gtk_tree_selection_select_iter(city_selection, &it);
-      }
-    } city_list_iterate_end;
+	if (g_hash_table_remove(copy, pcity)) {
+	  gtk_tree_selection_select_iter(city_selection, &it);
+	}
+      } city_list_iterate_end;
+    }
 
     /* free the selection. */
     g_hash_table_destroy(copy);
@@ -1203,7 +1218,7 @@ static void create_change_menu(GtkWidget *item)
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_improvements_item);
   change_wonders_item = gtk_menu_item_new_with_label(_("Wonders"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_wonders_item);
-  change_cma_item = gtk_menu_item_new_with_label(_("CMA"));
+  change_cma_item = gtk_menu_item_new_with_label(_("Citizen Governor"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_cma_item);
 }
 
@@ -1476,16 +1491,19 @@ static void create_select_menu(GtkWidget *item)
 	gtk_menu_item_new_with_label(_("Available Wonders"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), select_wonders_item);
   select_cma_item =
-	gtk_menu_item_new_with_label(_("CMA"));
+	gtk_menu_item_new_with_label(_("Citizen Governor"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), select_cma_item);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-static bool city_building_impr_or_unit(struct city *pcity, cid cid)
+static bool city_building_impr_or_unit(const struct city *pcity,
+				       struct city_production target)
 {
-  return (cid == cid_encode_from_city(pcity));
+  struct city_production target2 = pcity->production;
+
+  return target.is_unit == target2.is_unit && target.value == target2.value;
 }
 
 /****************************************************************
@@ -1558,18 +1576,13 @@ static void city_selection_changed_callback(GtkTreeSelection *selection)
   n = gtk_tree_selection_count_selected_rows(selection);
 
   if (n == 0) {
-    gtk_widget_set_sensitive(city_change_command, FALSE);
-    gtk_widget_set_sensitive(city_last_command, FALSE);
-    gtk_widget_set_sensitive(city_first_command, FALSE);
-    gtk_widget_set_sensitive(city_next_command, FALSE);
+    gtk_widget_set_sensitive(city_production_command, FALSE);
     gtk_widget_set_sensitive(city_center_command, FALSE);
     gtk_widget_set_sensitive(city_popup_command, FALSE);
     gtk_widget_set_sensitive(city_buy_command, FALSE);
   } else {
-    gtk_widget_set_sensitive(city_change_command, can_client_issue_orders());
-    gtk_widget_set_sensitive(city_last_command, can_client_issue_orders());
-    gtk_widget_set_sensitive(city_first_command, can_client_issue_orders());
-    gtk_widget_set_sensitive(city_next_command, can_client_issue_orders());
+    gtk_widget_set_sensitive(city_production_command,
+			     can_client_issue_orders());
     gtk_widget_set_sensitive(city_center_command, TRUE);
     gtk_widget_set_sensitive(city_popup_command, TRUE);
     gtk_widget_set_sensitive(city_buy_command, can_client_issue_orders());
