@@ -20,34 +20,36 @@
 
 #include <gtk/gtk.h>
 
+#include "mem.h"
+#include "shared.h"
+#include "support.h"
+
+#include "diptreaty.h"
 #include "fcintl.h"
 #include "game.h"
 #include "government.h"
 #include "map.h"
-#include "mem.h"
 #include "packets.h"
 #include "player.h"
-#include "shared.h"
-#include "support.h"
 
 #include "chatline.h"
+#include "civclient.h"
 #include "climisc.h"
 #include "clinet.h"
-#include "diptreaty.h"
-#include "gui_main.h"
-#include "gui_stuff.h"
-#include "mapview.h"
 #include "options.h"
 
 #include "diplodlg.h"
+#include "gui_main.h"
+#include "gui_stuff.h"
+#include "mapview.h"
+#include "plrdlg.h"
 
 #define MAX_NUM_CLAUSES 64
 
 struct Diplomacy_dialog {
   struct Treaty treaty;
+  struct gui_dialog* dialog;
   
-  GtkWidget *shell;
-
   GtkWidget *menu0;
   GtkWidget *menu1;
 
@@ -65,14 +67,13 @@ struct Diplomacy_dialog {
     TYPED_LIST_ITERATE(struct Diplomacy_dialog, dialoglist, pdialog)
 #define dialog_list_iterate_end  LIST_ITERATE_END
 
-static struct dialog_list dialog_list;
-static bool dialog_list_list_has_been_initialised = FALSE;
+static struct dialog_list *dialog_list;
 
 static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 						 struct player *plr1);
 
 static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id);
-static void popup_diplomacy_dialog(int other_player_id);
+static void popup_diplomacy_dialog(int other_player_id, int initiated_from);
 static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data);
 static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data);
@@ -86,6 +87,8 @@ static void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 static void diplo_dialog_returnkey(GtkWidget *w, gpointer data);
 
+#define RESPONSE_CANCEL_MEETING 100
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -102,6 +105,7 @@ void handle_diplomacy_accept_treaty(int counterpart, bool I_accepted,
   pdialog->treaty.accept1 = other_accepted;
 
   update_diplomacy_dialog(pdialog);
+  gui_dialog_alert(pdialog->dialog);
 }
 
 /****************************************************************
@@ -109,7 +113,7 @@ void handle_diplomacy_accept_treaty(int counterpart, bool I_accepted,
 *****************************************************************/
 void handle_diplomacy_init_meeting(int counterpart, int initiated_from)
 {
-  popup_diplomacy_dialog(counterpart);
+  popup_diplomacy_dialog(counterpart, initiated_from);
 }
 
 
@@ -139,8 +143,9 @@ void handle_diplomacy_create_clause(int counterpart, int giver,
     return;
   }
 
-  add_clause(&pdialog->treaty, get_player(giver), type, value);
+  add_clause(&pdialog->treaty, player_by_number(giver), type, value);
   update_diplomacy_dialog(pdialog);
+  gui_dialog_alert(pdialog->dialog);  
 }
 
 /****************************************************************
@@ -155,17 +160,22 @@ void handle_diplomacy_remove_clause(int counterpart, int giver,
     return;
   }
 
-  remove_clause(&pdialog->treaty, get_player(giver), type, value);
+  remove_clause(&pdialog->treaty, player_by_number(giver), type, value);
   update_diplomacy_dialog(pdialog);
+  gui_dialog_alert(pdialog->dialog);
 }
 
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-static void popup_diplomacy_dialog(int other_player_id)
+static void popup_diplomacy_dialog(int other_player_id, int initiated_from)
 {
   struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(other_player_id);
 
+  if (!can_client_issue_orders()) {
+    return;
+  }
+
   if (game.player_ptr->ai.control) {
     return;			/* Don't show if we are AI controlled. */
   }
@@ -173,10 +183,18 @@ static void popup_diplomacy_dialog(int other_player_id)
   if (!pdialog) {
     pdialog =
 	create_diplomacy_dialog(game.player_ptr,
-				get_player(other_player_id));
+				player_by_number(other_player_id));
   }
 
-  gtk_window_present(GTK_WINDOW(pdialog->shell));
+  gui_dialog_present(pdialog->dialog);
+  /* We initated the meeting - Make the tab active */
+  if (initiated_from == player_number(game.player_ptr)) {
+    gui_dialog_raise(pdialog->dialog);
+    
+    if (players_dialog_shell != NULL) {
+      gui_dialog_set_return_dialog(pdialog->dialog, players_dialog_shell);
+    }
+  }
 }
 
 /****************************************************************
@@ -235,19 +253,19 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
 
     menu = gtk_menu_new();
 
-    for (i = 1, flag = FALSE; i < game.num_tech_types; i++) {
+    for (i = 1, flag = FALSE; i < game.control.num_tech_types; i++) {
       if (get_invention(plr0, i) == TECH_KNOWN
 	  && (get_invention(plr1, i) == TECH_UNKNOWN
 	      || get_invention(plr1, i) == TECH_REACHABLE)
           && tech_is_available(plr1, i)) {
 	item
-	  = gtk_menu_item_new_with_label(get_tech_name(game.player_ptr, i));
+	  = gtk_menu_item_new_with_label(advance_name_for_player(game.player_ptr, i));
 
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect(item, "activate",
 			 G_CALLBACK(diplomacy_dialog_tech_callback),
-			 GINT_TO_POINTER((plr0->player_no << 24) |
-					 (plr1->player_no << 16) |
+			 GINT_TO_POINTER((player_number(plr0) << 24) |
+					 (player_number(plr1) << 16) |
 					 i));
 	flag = TRUE;
       }
@@ -271,7 +289,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
 			      - Kris Bubendorfer
   *****************************************************************/
   {
-    int i = 0, j = 0, n = city_list_size(&plr0->cities);
+    int i = 0, j = 0, n = city_list_size(plr0->cities);
     struct city **city_list_ptrs;
 
     if (n > 0) {
@@ -292,13 +310,13 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
     menu = gtk_menu_new();
 
     for (j = 0; j < i; j++) {
-      item = gtk_menu_item_new_with_label(city_list_ptrs[j]->name);
+      item = gtk_menu_item_new_with_label(city_name(city_list_ptrs[j]));
 
       gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
       g_signal_connect(item, "activate",
 		       G_CALLBACK(diplomacy_dialog_city_callback),
-			 GINT_TO_POINTER((plr0->player_no << 24) |
-					 (plr1->player_no << 16) |
+			 GINT_TO_POINTER((player_number(plr0) << 24) |
+					 (player_number(plr1) << 16) |
 					 city_list_ptrs[j]->id));
     }
     free(city_list_ptrs);
@@ -378,9 +396,8 @@ static void row_callback(GtkTreeView *view, GtkTreePath *path,
   clause_list_iterate(pdialog->treaty.clauses, pclause) {
     if (i == index[0]) {
       dsend_packet_diplomacy_remove_clause_req(&aconnection,
-					       pdialog->treaty.plr1->
-					       player_no,
-					       pclause->from->player_no,
+					       player_number(pdialog->treaty.plr1),
+					       player_number(pclause->from),
 					       pclause->type,
 					       pclause->value);
       return;
@@ -393,18 +410,18 @@ static void row_callback(GtkTreeView *view, GtkTreePath *path,
 /****************************************************************
 ...
 *****************************************************************/
-static void diplomacy_destroy(GtkWidget *w, gpointer data)
+static void diplomacy_destroy(struct Diplomacy_dialog* pdialog)
 {
-  struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *)data;
-
-  dialog_list_unlink(&dialog_list, pdialog);
+  gui_dialog_destroy(pdialog->dialog);
+  dialog_list_unlink(dialog_list, pdialog);
   free(pdialog);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-static void diplomacy_response(GtkWidget *w, gint response, gpointer data)
+static void diplomacy_response(struct gui_dialog *dlg, int response,
+                               gpointer data)
 {
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *)data;
 
@@ -413,15 +430,12 @@ static void diplomacy_response(GtkWidget *w, gint response, gpointer data)
 
 
     dsend_packet_diplomacy_accept_treaty_req(&aconnection,
-					     pdialog->treaty.plr1->
-					     player_no);
+					     player_number(pdialog->treaty.plr1));
     break;
-
   default:
     dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
-					      pdialog->treaty.plr1->
-					      player_no);
-    gtk_widget_destroy(w);
+					      player_number(pdialog->treaty.plr1));
+    diplomacy_destroy(pdialog);
     break; 
   }
 }
@@ -432,7 +446,7 @@ static void diplomacy_response(GtkWidget *w, gint response, gpointer data)
 static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 							struct player *plr1)
 {
-  GtkWidget *shell, *vbox, *bottom, *hbox, *table;
+  GtkWidget *vbox, *bottom, *hbox, *table;
   GtkWidget *label, *sw, *view, *image, *spin;
   GtkWidget *menubar, *menuitem, *menu;
   GtkListStore *store;
@@ -443,26 +457,24 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 
   pdialog = fc_malloc(sizeof(*pdialog));
 
-  dialog_list_insert(&dialog_list, pdialog);
+  dialog_list_prepend(dialog_list, pdialog);
   init_treaty(&pdialog->treaty, plr0, plr1);
 
-  shell = gtk_dialog_new_with_buttons(_("Diplomacy meeting"),
-				      NULL,
-				      0,
-				      NULL);
-  pdialog->shell = shell;
-  setup_dialog(shell, toplevel);
-  g_signal_connect(shell, "destroy",
-		   G_CALLBACK(diplomacy_destroy), pdialog);
-  g_signal_connect(shell, "response",
-		   G_CALLBACK(diplomacy_response), pdialog);
+  gui_dialog_new(&(pdialog->dialog), GTK_NOTEBOOK(top_notebook), pdialog);
+  
+  my_snprintf(buf, sizeof(buf),
+              _("Diplomacy: %s"),
+	      nation_plural_for_player(plr1));
 
-  gtk_dialog_add_button(GTK_DIALOG(shell), _("_Cancel meeting"),
-			GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button(GTK_DIALOG(shell), _("Accept _treaty"),
-			GTK_RESPONSE_ACCEPT);
+  gui_dialog_set_title(pdialog->dialog, buf);
+  gui_dialog_response_set_callback(pdialog->dialog, diplomacy_response);
 
-  vbox = GTK_DIALOG(shell)->vbox;
+  gui_dialog_add_stockbutton(pdialog->dialog, GTK_STOCK_CANCEL,
+			     _("_Cancel meeting"), RESPONSE_CANCEL_MEETING);
+  gui_dialog_add_stockbutton(pdialog->dialog, GTK_STOCK_DND,
+			     _("Accept _treaty"), GTK_RESPONSE_ACCEPT);
+
+  vbox = pdialog->dialog->vbox;
 
 
   /* clauses. */
@@ -493,34 +505,52 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
     "yalign", 0.5,
     NULL);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+
   gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 2);
   gtk_widget_show_all(vbox);
 
 
   /* bottom area. */
   bottom = gtk_hbox_new(TRUE, 18);
-  gtk_box_pack_start(GTK_BOX(vbox), bottom, FALSE, FALSE, 10);
+  gtk_box_pack_start(GTK_BOX(vbox), bottom, FALSE, FALSE, 20);
+
 
   /* us. */
   vbox = gtk_vbox_new(FALSE, 18);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 2);
   gtk_box_pack_start(GTK_BOX(bottom), vbox, TRUE, TRUE, 0);
 
+  label = gtk_label_new(NULL);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+  my_snprintf(buf, sizeof(buf),
+	      "<span size=\"large\"><u>%s</u></span>",
+              nation_plural_for_player(plr0));
+  gtk_label_set_markup(GTK_LABEL(label), buf);
+  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
+
   hbox = gtk_hbox_new(FALSE, 12);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+  
+  /* our flag */
+  image =
+      gtk_image_new_from_pixbuf(sprite_get_pixbuf
+				(get_nation_flag_sprite
+				 (tileset, nation_of_player(plr0))));
+  gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);    
 
   label = gtk_label_new(NULL);
-  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
   my_snprintf(buf, sizeof(buf),
-	      "<span size=\"large\" weight=\"bold\">%s</span>",
-	      get_nation_name(plr0->nation));
+	      "<span size=\"large\" weight=\"bold\">%s %s</span>",
+	      ruler_title_translation(plr0),
+	      player_name(plr0));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 
   image = gtk_image_new();
   pdialog->image0 = image;
   gtk_box_pack_end(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-
+  
   table = gtk_table_new(2, 2, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(table), 6);
   gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
@@ -529,8 +559,8 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
   gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 0, 1);
   g_object_set_data(G_OBJECT(spin), "plr", plr0);
-  g_signal_connect_after(spin, "activate",
-      			 G_CALLBACK(diplo_dialog_returnkey), pdialog);
+  g_signal_connect_after(spin, "value-changed",
+			 G_CALLBACK(diplo_dialog_returnkey), pdialog);
 
   label = g_object_new(GTK_TYPE_LABEL,
     "use-underline", TRUE,
@@ -562,21 +592,39 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 2);
   gtk_box_pack_start(GTK_BOX(bottom), vbox, TRUE, TRUE, 0);
 
+
+  label = gtk_label_new(NULL);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+  my_snprintf(buf, sizeof(buf),
+	      "<span size=\"large\"><u>%s</u></span>",
+              nation_plural_for_player(plr1));
+  gtk_label_set_markup(GTK_LABEL(label), buf);
+  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
+
   hbox = gtk_hbox_new(FALSE, 12);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
+  /* Their flag */
+  image =
+      gtk_image_new_from_pixbuf(sprite_get_pixbuf
+				(get_nation_flag_sprite
+				 (tileset, nation_of_player(plr1))));
+  gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);    
+
+
   label = gtk_label_new(NULL);
-  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
   my_snprintf(buf, sizeof(buf),
-	      "<span size=\"large\" weight=\"bold\">%s</span>",
-	      get_nation_name(plr1->nation));
+	      "<span size=\"large\" weight=\"bold\">%s %s</span>",
+	      ruler_title_translation(plr1),
+	      player_name(plr1));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 
   image = gtk_image_new();
   pdialog->image1 = image;
   gtk_box_pack_end(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-
+  
   table = gtk_table_new(2, 2, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(table), 6);
   gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
@@ -585,8 +633,8 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
   gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 0, 1);
   g_object_set_data(G_OBJECT(spin), "plr", plr1);
-  g_signal_connect_after(spin, "activate",
-      			 G_CALLBACK(diplo_dialog_returnkey), pdialog);
+  g_signal_connect_after(spin, "value-changed",
+			 G_CALLBACK(diplo_dialog_returnkey), pdialog);
 
   label = g_object_new(GTK_TYPE_LABEL,
     "use-underline", TRUE,
@@ -610,6 +658,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), menu);
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), menuitem);
   g_object_set_data(G_OBJECT(menu), "plr", plr1);
+  
   g_signal_connect(menu, "show", G_CALLBACK(popup_add_menu), pdialog);
   
   gtk_widget_show_all(bottom);
@@ -617,6 +666,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   g_signal_connect(view, "row_activated", G_CALLBACK(row_callback), pdialog);
 
   update_diplomacy_dialog(pdialog);
+  gui_dialog_show_all(pdialog->dialog);
 
   return pdialog;
 }
@@ -628,6 +678,7 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
   GtkListStore *store;
   GtkTreeIter it;
+  bool blank = TRUE;
 
   store = pdialog->store;
 
@@ -639,14 +690,20 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 
     gtk_list_store_append(store, &it);
     gtk_list_store_set(store, &it, 0, buf, -1);
+    blank = FALSE;
   } clause_list_iterate_end;
 
-  gtk_image_set_from_pixmap(GTK_IMAGE(pdialog->image0),
-			    get_thumb_pixmap(pdialog->treaty.accept0),
-			    NULL);
-  gtk_image_set_from_pixmap(GTK_IMAGE(pdialog->image1),
-			    get_thumb_pixmap(pdialog->treaty.accept1),
-			    NULL);
+  if (blank) {
+    gtk_list_store_append(store, &it);
+    gtk_list_store_set(store, &it, 0,
+		       _("--- This treaty is blank. "
+		 	 "Please add some clauses. ---"), -1);
+  }
+
+  gtk_image_set_from_pixbuf(GTK_IMAGE(pdialog->image0),
+			    get_thumb_pixbuf(pdialog->treaty.accept0));
+  gtk_image_set_from_pixbuf(GTK_IMAGE(pdialog->image1),
+			    get_thumb_pixbuf(pdialog->treaty.accept1));
 }
 
 /****************************************************************
@@ -658,7 +715,7 @@ static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int tech = choice & 0xffff;
 
-  if (giver == game.player_idx) {
+  if (giver == game.info.player_idx) {
     other = dest;
   } else {
     other = giver;
@@ -678,7 +735,7 @@ static void diplomacy_dialog_city_callback(GtkWidget * w, gpointer data)
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int city = choice & 0xffff;
 
-  if (giver == game.player_idx) {
+  if (giver == game.info.player_idx) {
     other = dest;
   } else {
     other = giver;
@@ -699,8 +756,8 @@ static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
   pgiver = (struct player *)g_object_get_data(G_OBJECT(w), "plr");
 
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_MAP, 0);
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_MAP, 0);
 }
 
 /****************************************************************
@@ -714,8 +771,8 @@ static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
   pgiver = (struct player *)g_object_get_data(G_OBJECT(w), "plr");
 
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_SEAMAP,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_SEAMAP,
 					   0);
 }
 
@@ -728,8 +785,8 @@ static void diplomacy_dialog_add_pact_clause(GtkWidget *w, gpointer data,
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *)data;
 
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pdialog->treaty.plr0->player_no,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pdialog->treaty.plr0),
 					   type, 0);
 }
 
@@ -767,8 +824,8 @@ static void diplomacy_dialog_vision_callback(GtkWidget *w, gpointer data)
       (struct player *) g_object_get_data(G_OBJECT(w), "plr");
 
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_VISION,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_VISION,
 					   0);
 }
 
@@ -782,8 +839,8 @@ static void diplomacy_dialog_embassy_callback(GtkWidget *w, gpointer data)
       (struct player *) g_object_get_data(G_OBJECT(w), "plr");
 
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_EMBASSY,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_EMBASSY,
 					   0);
 }
 
@@ -793,7 +850,23 @@ static void diplomacy_dialog_embassy_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
-  gtk_widget_destroy(pdialog->shell);
+  diplomacy_destroy(pdialog);
+}
+
+/*****************************************************************
+...
+*****************************************************************/
+void diplomacy_dialog_init()
+{
+  dialog_list = dialog_list_new();
+}
+
+/*****************************************************************
+...
+*****************************************************************/
+void diplomacy_dialog_done()
+{
+  dialog_list_free(dialog_list);
 }
 
 /*****************************************************************
@@ -801,12 +874,7 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = get_player(other_player_id);
-
-  if(!dialog_list_list_has_been_initialised) {
-    dialog_list_init(&dialog_list);
-    dialog_list_list_has_been_initialised = TRUE;
-  }
+  struct player *plr0 = game.player_ptr, *plr1 = player_by_number(other_player_id);
 
   dialog_list_iterate(dialog_list, pdialog) {
     if ((pdialog->treaty.plr0 == plr0 && pdialog->treaty.plr1 == plr1) ||
@@ -830,12 +898,11 @@ static void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
   
   if (amount >= 0 && amount <= pgiver->economic.gold) {
     dsend_packet_diplomacy_create_clause_req(&aconnection,
-					     pdialog->treaty.plr1->
-					     player_no, pgiver->player_no,
+					     player_number(pdialog->treaty.plr1),
+					     player_number(pgiver),
 					     CLAUSE_GOLD, amount);
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), 0.0);
   } else {
-    append_output_window(_("Game: Invalid amount of gold specified."));
+    append_output_window(_("Invalid amount of gold specified."));
   }
 }
 
@@ -844,11 +911,7 @@ static void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
 *****************************************************************/
 void close_all_diplomacy_dialogs(void)
 {
-  if (!dialog_list_list_has_been_initialised) {
-    return;
-  }
-
-  while (dialog_list_size(&dialog_list) > 0) {
-    close_diplomacy_dialog(dialog_list_get(&dialog_list, 0));
+  while (dialog_list_size(dialog_list) > 0) {
+    close_diplomacy_dialog(dialog_list_get(dialog_list, 0));
   }
 }
