@@ -24,6 +24,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "fcintl.h"
+#include "log.h"
 #include "mem.h"
 #include "support.h"
 
@@ -36,6 +37,8 @@
 
 static GList *dialog_list;
 
+static GtkSizeGroup *gui_action;
+
 
 /**************************************************************************
 ...
@@ -76,8 +79,11 @@ GtkWidget *gtk_stockbutton_new(const gchar *stock, const gchar *label_text)
 
   label = gtk_label_new_with_mnemonic(label_text);
   gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
+  g_object_set_data(G_OBJECT(button), "label", label);
 
   image = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_BUTTON);
+  g_object_set_data(G_OBJECT(button), "image", image);
+
   hbox = gtk_hbox_new(FALSE, 2);
 
   align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
@@ -91,6 +97,18 @@ GtkWidget *gtk_stockbutton_new(const gchar *stock, const gchar *label_text)
   return button;
 }
 
+/**************************************************************************
+  Changes the lable (with mnemonic) on an existing stockbutton.  See
+  gtk_stockbutton_new.
+**************************************************************************/
+void gtk_stockbutton_set_label(GtkWidget *button, const gchar *label_text)
+{
+  GtkWidget *label;
+
+  label = g_object_get_data(G_OBJECT(button), "label");
+  gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), label_text);
+}
+
 /**************************************************************************
   Returns gettext-converted list of n strings.  The individual strings
   in the list are as returned by gettext().  In case of no NLS, the strings
@@ -107,7 +125,7 @@ void intl_slist(int n, const char **s, bool *done)
 
   if (!*done) {
     for(i=0; i<n; i++) {
-      s[i] = _(s[i]);
+      s[i] = Q_(s[i]);
     }
 
     *done = TRUE;
@@ -274,14 +292,15 @@ void setup_dialog(GtkWidget *shell, GtkWidget *parent)
 static void gui_dialog_response(struct gui_dialog *dlg, int response)
 {
   if (dlg->response_callback) {
-    (*dlg->response_callback)(dlg, response);
+    (*dlg->response_callback)(dlg, response, dlg->user_data);
   }
 }
 
 /**************************************************************************
   Default dialog response handler. Destroys the dialog.
 **************************************************************************/
-static void gui_dialog_destroyed(struct gui_dialog *dlg, int response)
+static void gui_dialog_destroyed(struct gui_dialog *dlg, int response,
+                                 gpointer data)
 {
   gui_dialog_destroy(dlg);
 }
@@ -298,17 +317,37 @@ static void gui_dialog_destroy_handler(GtkWidget *w, struct gui_dialog *dlg)
     g_signal_handler_disconnect(notebook, handler_id);
   }
 
+  g_object_unref(dlg->gui_button);
+
   if (*(dlg->source)) {
     *(dlg->source) = NULL;
   }
 
   dialog_list = g_list_remove(dialog_list, dlg);
+  
+  /* Raise the return dialog set by gui_dialog_set_return_dialog() */
+  if (dlg->return_dialog_id != -1) {
+    GList *it;
+    for (it = dialog_list; it; it = g_list_next(it)) {
+      struct gui_dialog * adialog = (struct gui_dialog *)it->data;
+      if (adialog->id == dlg->return_dialog_id) {
+        gui_dialog_raise(adialog);
+	break;
+      }
+    }
+  }
+  
+  if (dlg->title) {
+    free(dlg->title);
+  }
+  
   free(dlg);
 }
 
 /**************************************************************************
   Emit a delete event response on dialog deletion in case the end-user
   needs to know when a deletion took place.
+  Popup dialog version
 **************************************************************************/
 static gint gui_dialog_delete_handler(GtkWidget *widget,
 				      GdkEventAny *ev, gpointer data)
@@ -322,6 +361,31 @@ static gint gui_dialog_delete_handler(GtkWidget *widget,
   return FALSE;
 }
 
+/**************************************************************************
+  Emit a delete event response on dialog deletion in case the end-user
+  needs to know when a deletion took place.
+  TAB version
+**************************************************************************/
+static gint gui_dialog_delete_tab_handler(struct gui_dialog* dlg)
+{
+  GtkWidget* notebook;
+  int n;
+  
+  notebook = dlg->v.tab.notebook;
+  n = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
+  if (gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), n)
+      != dlg->v.tab.child) {
+    gui_dialog_set_return_dialog(dlg, NULL);
+  }			                                  
+  
+  /* emit response signal. */
+  gui_dialog_response(dlg, GTK_RESPONSE_DELETE_EVENT);
+                                                                               
+  /* do the destroy by default. */
+  return FALSE;
+}
+
+
 /**************************************************************************
   Allow the user to close a dialog using Escape or CTRL+W.
 **************************************************************************/
@@ -360,22 +424,91 @@ static void gui_dialog_switch_page_handler(GtkNotebook *notebook,
   }
 }
 
+/**************************************************************************
+  Changes a tab into a window.
+**************************************************************************/
+static void gui_dialog_detach(struct gui_dialog* dlg)
+{
+  gint n;
+  GtkWidget *window, *notebook;
+  gulong handler_id;
+
+  if (dlg->type != GUI_DIALOG_TAB) {
+    return;
+  }
+  dlg->type = GUI_DIALOG_WINDOW;
+  
+  /* Create a new reference to the main widget, so it won't be 
+   * destroyed in gtk_notebook_remove_page() */
+  g_object_ref(dlg->vbox);
+
+  /* Remove widget from the notebook */
+  notebook = dlg->v.tab.notebook;
+  handler_id = dlg->v.tab.handler_id;
+  g_signal_handler_disconnect(notebook, handler_id);
+
+  n = gtk_notebook_page_num(GTK_NOTEBOOK(dlg->v.tab.notebook), dlg->vbox);
+  gtk_notebook_remove_page(GTK_NOTEBOOK(dlg->v.tab.notebook), n);
+
+
+  /* Create window and put the widget inside */
+  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(window), dlg->title);
+  setup_dialog(window, toplevel);
+
+  gtk_container_add(GTK_CONTAINER(window), dlg->vbox);
+  dlg->v.window = window;
+  g_signal_connect(window, "delete_event",
+    G_CALLBACK(gui_dialog_delete_handler), dlg);
+	
+  gtk_window_set_default_size(GTK_WINDOW(dlg->v.window),
+                              dlg->default_width,
+			      dlg->default_height);    
+  gtk_widget_show_all(window);
+}
+
+/***************************************************************************
+  Someone has clicked on a label in a notebook
+***************************************************************************/
+static gboolean click_on_tab_callback(GtkWidget* w,
+                                     GdkEventButton* button,
+				     gpointer data)
+{
+  if (button->type != GDK_2BUTTON_PRESS) {
+    return FALSE;
+  }
+  if (button->button != 1) {
+    return FALSE;
+  }
+  gui_dialog_detach((struct gui_dialog*) data);
+  return TRUE;
+}
+
+
 /**************************************************************************
   Creates a new dialog. It will be a tab or a window depending on the
   current user setting of 'enable_tabs'.
   Sets pdlg to point to the dialog once it is create, Zeroes pdlg on
   dialog destruction.
+  user_data will be passed through response function
 **************************************************************************/
-void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook)
+void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
+                    gpointer user_data)
 {
   struct gui_dialog *dlg;
   GtkWidget *vbox, *action_area;
+  static int dialog_id_counter;
 
   dlg = fc_malloc(sizeof(*dlg));
   dialog_list = g_list_prepend(dialog_list, dlg);
 
   dlg->source = pdlg;
   *pdlg = dlg;
+  dlg->user_data = user_data;
+  dlg->title = NULL;
+  
+  dlg->default_width = 200;
+  dlg->default_height = 300;
 
   if (enable_tabs) {
     dlg->type = GUI_DIALOG_TAB;
@@ -383,16 +516,17 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook)
     dlg->type = GUI_DIALOG_WINDOW;
   }
 
+  if (!gui_action) {
+    gui_action = gtk_size_group_new(GTK_SIZE_GROUP_VERTICAL);
+  }
+  dlg->gui_button = gtk_size_group_new(GTK_SIZE_GROUP_BOTH);
+
   if (enable_tabs && notebook == GTK_NOTEBOOK(bottom_notebook)) {
     vbox = gtk_hbox_new(FALSE, 0);
-    action_area = gtk_vbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area),
-	GTK_BUTTONBOX_SPREAD);
+    action_area = gtk_vbox_new(FALSE, 2);
   } else {
     vbox = gtk_vbox_new(FALSE, 0);
-    action_area = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area),
-	GTK_BUTTONBOX_END);
+    action_area = gtk_hbox_new(FALSE, 2);
   }
 
   gtk_widget_show(vbox);
@@ -415,11 +549,14 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook)
 
       gtk_container_add(GTK_CONTAINER(window), vbox);
       dlg->v.window = window;
+      g_signal_connect(window, "delete_event",
+        G_CALLBACK(gui_dialog_delete_handler), dlg);
+      
     }
     break;
   case GUI_DIALOG_TAB:
     {
-      GtkWidget *hbox, *label, *image, *button;
+      GtkWidget *hbox, *label, *image, *button, *event_box;
       gint w, h;
       char buf[256];
 
@@ -435,7 +572,7 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook)
       button = gtk_button_new();
       gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
       g_signal_connect_swapped(button, "clicked",
-	  G_CALLBACK(gui_dialog_destroy), dlg);
+	  G_CALLBACK(gui_dialog_delete_tab_handler), dlg);
 
       my_snprintf(buf, sizeof(buf), _("Close Tab:\n%s"), _("Ctrl+W"));
       gtk_tooltips_set_tip(main_tips, button, buf, "");
@@ -447,14 +584,22 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook)
       gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
       gtk_widget_show_all(hbox);
+      
+      event_box = gtk_event_box_new();
+      gtk_container_add(GTK_CONTAINER(event_box), hbox);
 
-      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, hbox);
+      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, event_box);
       dlg->v.tab.handler_id =
 	g_signal_connect(notebook, "switch_page",
 	    G_CALLBACK(gui_dialog_switch_page_handler), dlg);
+      dlg->v.tab.child = vbox;
 
       dlg->v.tab.label = label;
       dlg->v.tab.notebook = GTK_WIDGET(notebook);
+      
+      gtk_widget_add_events(event_box, GDK_BUTTON2_MOTION_MASK);
+      g_signal_connect(event_box, "button-press-event",
+                       G_CALLBACK(click_on_tab_callback), dlg);
     }
     break;
   }
@@ -463,11 +608,13 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook)
   dlg->action_area = action_area;
 
   dlg->response_callback = gui_dialog_destroyed;
+  
+  dlg->id = dialog_id_counter;
+  dialog_id_counter++;
+  dlg->return_dialog_id = -1;
 
   g_signal_connect(vbox, "destroy",
       G_CALLBACK(gui_dialog_destroy_handler), dlg);
-  g_signal_connect(vbox, "delete_event",
-      G_CALLBACK(gui_dialog_delete_handler), dlg);
   g_signal_connect(vbox, "key_press_event",
       G_CALLBACK(gui_dialog_key_press_handler), dlg);
 
@@ -509,6 +656,8 @@ static void gui_dialog_pack_button(struct gui_dialog *dlg, GtkWidget *button,
   }
 
   gtk_box_pack_end(GTK_BOX(dlg->action_area), button, FALSE, TRUE, 0);
+  gtk_size_group_add_widget(gui_action, button);
+  gtk_size_group_add_widget(dlg->gui_button, button);
 }
 
 /**************************************************************************
@@ -542,6 +691,18 @@ GtkWidget *gui_dialog_add_button(struct gui_dialog *dlg,
   return button;
 }
 
+/**************************************************************************
+  Adds a widget to a dialog.
+**************************************************************************/
+GtkWidget *gui_dialog_add_widget(struct gui_dialog *dlg,
+				 GtkWidget *widget)
+{
+  gtk_box_pack_start(GTK_BOX(dlg->action_area), widget, FALSE, TRUE, 0);
+  gtk_size_group_add_widget(gui_action, widget);
+
+  return widget;
+}
+
 /**************************************************************************
   Changes the default dialog response.
 **************************************************************************/
@@ -554,11 +715,14 @@ void gui_dialog_set_default_response(struct gui_dialog *dlg, int response)
 
   for (list = children; list; list = g_list_next(list)) {
     GtkWidget *button = list->data;
-    gpointer data = g_object_get_data(G_OBJECT(button),
-	"gui-dialog-response-data");
 
-    if (response == GPOINTER_TO_INT(data)) {
-      gtk_widget_grab_default(button);
+    if (GTK_IS_BUTTON(button)) {
+      gpointer data = g_object_get_data(G_OBJECT(button),
+	  "gui-dialog-response-data");
+
+      if (response == GPOINTER_TO_INT(data)) {
+	gtk_widget_grab_default(button);
+      }
     }
   }
 
@@ -578,11 +742,14 @@ void gui_dialog_set_response_sensitive(struct gui_dialog *dlg,
 
   for (list = children; list; list = g_list_next(list)) {
     GtkWidget *button = list->data;
-    gpointer data = g_object_get_data(G_OBJECT(button),
-	"gui-dialog-response-data");
 
-    if (response == GPOINTER_TO_INT(data)) {
-      gtk_widget_set_sensitive(button, setting);
+    if (GTK_IS_BUTTON(button)) {
+      gpointer data = g_object_get_data(G_OBJECT(button),
+	  "gui-dialog-response-data");
+
+      if (response == GPOINTER_TO_INT(data)) {
+	gtk_widget_set_sensitive(button, setting);
+      }
     }
   }
 
@@ -613,14 +780,20 @@ void gui_dialog_show_all(struct gui_dialog *dlg)
 
     for (list = children; list; list = g_list_next(list)) {
       GtkWidget *button = list->data;
-      gpointer data = g_object_get_data(G_OBJECT(button),
-	  "gui-dialog-response-data");
-      int response = GPOINTER_TO_INT(data);
 
-      if (response != GTK_RESPONSE_CLOSE && response != GTK_RESPONSE_CANCEL) {
+      if (!GTK_IS_BUTTON(button)) {
 	num_visible++;
       } else {
-	gtk_widget_hide(button);
+	gpointer data = g_object_get_data(G_OBJECT(button),
+	    "gui-dialog-response-data");
+	int response = GPOINTER_TO_INT(data);
+
+	if (response != GTK_RESPONSE_CLOSE
+	    && response != GTK_RESPONSE_CANCEL) {
+	  num_visible++;
+	} else {
+	  gtk_widget_hide(button);
+	}
       }
     }
     g_list_free(children);
@@ -650,9 +823,11 @@ void gui_dialog_present(struct gui_dialog *dlg)
 
       if (current != n) {
 	GtkWidget *label = dlg->v.tab.label;
+	GdkColormap *cmap = gtk_widget_get_default_colormap();
+	GdkColor color = {.red = 255 << 8, .green = 0, .blue = 0};
 
-	gtk_widget_modify_fg(label, GTK_STATE_ACTIVE,
-	    colors_standard[COLOR_STD_RED]);
+	gdk_rgb_find_color(cmap, &color);
+	gtk_widget_modify_fg(label, GTK_STATE_ACTIVE, &color);
       }
     }
     break;
@@ -698,9 +873,11 @@ void gui_dialog_alert(struct gui_dialog *dlg)
 
       if (current != n) {
 	GtkWidget *label = dlg->v.tab.label;
+	GdkColormap *cmap = gtk_widget_get_default_colormap();
+	GdkColor color = {.red = 0, .green = 0, .blue = 255 << 8};
 
-	gtk_widget_modify_fg(label, GTK_STATE_ACTIVE,
-	    colors_standard[COLOR_STD_RACE9]);
+	gdk_rgb_find_color(cmap, &color);
+	gtk_widget_modify_fg(label, GTK_STATE_ACTIVE, &color);
       }
     }
     break;
@@ -712,6 +889,8 @@ void gui_dialog_alert(struct gui_dialog *dlg)
 **************************************************************************/
 void gui_dialog_set_default_size(struct gui_dialog *dlg, int width, int height)
 {
+  dlg->default_width = width;
+  dlg->default_height = height;
   switch (dlg->type) {
   case GUI_DIALOG_WINDOW:
     gtk_window_set_default_size(GTK_WINDOW(dlg->v.window), width, height);
@@ -726,6 +905,10 @@ void gui_dialog_set_default_size(struct gui_dialog *dlg, int width, int height)
 **************************************************************************/
 void gui_dialog_set_title(struct gui_dialog *dlg, const char *title)
 {
+  if (dlg->title) {
+    free(dlg->title);
+  }
+  dlg->title = mystrdup(title);
   switch (dlg->type) {
   case GUI_DIALOG_WINDOW:
     gtk_window_set_title(GTK_WINDOW(dlg->v.window), title);
@@ -779,3 +962,15 @@ void gui_dialog_response_set_callback(struct gui_dialog *dlg,
   dlg->response_callback = fun;
 }
 
+/**************************************************************************
+  When the dlg dialog is destroyed the return_dialog will be raised
+**************************************************************************/
+void gui_dialog_set_return_dialog(struct gui_dialog *dlg,
+                                  struct gui_dialog *return_dialog)
+{
+  if (return_dialog == NULL) {
+    dlg->return_dialog_id = -1;
+  } else {
+    dlg->return_dialog_id = return_dialog->id;
+  }
+}
