@@ -102,13 +102,13 @@ static const char *help_ulabel_name[5][5] =
 static const char *help_tlabel_name[4][5] =
 {
     { N_("Move/Defense:"),	NULL, NULL, N_("Food/Res/Trade:"),	NULL },
-    { N_("Sp1 F/R/T:"),		NULL, NULL, N_("Sp2 F/R/T:"),		NULL },
+    { N_("Resources:"),		NULL, NULL, NULL,			NULL },
     { N_("Road Rslt/Time:"),	NULL, NULL, N_("Irrig. Rslt/Time:"),	NULL },
     { N_("Mine Rslt/Time:"),	NULL, NULL, N_("Trans. Rslt/Time:"),	NULL }
 };
 
-/* HACK: we use a static string for convenience. */
-static char long_buffer[64000];
+#define REQ_NONE _("None")
+#define REQ_NEVER _("(Never)")
 
 
 static void create_help_dialog(void);
@@ -187,7 +187,7 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   GValue        value = { 0, };
 
   if (advances[tech].req[0] == A_LAST && advances[tech].req[1] == A_LAST) {
-    bg = COLOR_STD_RED;
+    bg = COLOR_REQTREE_UNREACHABLE;
 
     gtk_tree_store_append(tstore, &l, parent);
     help_advances[tech] = TRUE;
@@ -200,16 +200,22 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
     gtk_tree_store_set(tstore, &l,
 		       1, -1,
 		       2, tech,
-		       3, colors_standard[bg],
+		       3, &get_color(tileset, bg)->color
 		       -1);
     return;
   }
 
+  bg = COLOR_REQTREE_BACKGROUND;
   switch (get_invention(game.player_ptr, tech)) {
-  case TECH_UNKNOWN:	      bg = COLOR_STD_RED;	      break;
-  case TECH_KNOWN:	      bg = COLOR_STD_GROUND;	      break;
-  case TECH_REACHABLE:        bg = COLOR_STD_YELLOW;	      break;
-  default:		      bg = COLOR_STD_WHITE;	      break;
+  case TECH_UNKNOWN:
+    bg = COLOR_REQTREE_UNREACHABLE;
+    break;
+  case TECH_KNOWN:
+    bg = COLOR_REQTREE_KNOWN;
+    break;
+  case TECH_REACHABLE:
+    bg = COLOR_REQTREE_REACHABLE;
+    break;
   }
   turns_to_tech = num_unknown_techs_for_goal(game.player_ptr, tech);
 
@@ -220,14 +226,14 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   help_advances[tech] = TRUE;
 
   g_value_init(&value, G_TYPE_STRING);
-  g_value_set_static_string(&value, get_tech_name(game.player_ptr, tech));
+  g_value_set_static_string(&value, advance_name_for_player(game.player_ptr, tech));
   gtk_tree_store_set_value(tstore, &l, 0, &value);
   g_value_unset(&value);
 
   gtk_tree_store_set(tstore, &l,
 		     1, turns_to_tech,
 		     2, tech,
-		     3, colors_standard[bg],
+		     3, &get_color(tileset, bg)->color,
 		     -1);
 
   if (--levels <= 0)
@@ -256,7 +262,7 @@ static void help_tech_tree_activated_callback(GtkTreeView *view,
 
   gtk_tree_model_get_iter(GTK_TREE_MODEL(tstore), &it, path);
   gtk_tree_model_get(GTK_TREE_MODEL(tstore), &it, 2, &tech, -1);
-  select_help_item_string(get_tech_name(game.player_ptr, tech), HELP_TECH);
+  select_help_item_string(advance_name_for_player(game.player_ptr, tech), HELP_TECH);
 }
 
 /**************************************************************************
@@ -286,12 +292,12 @@ static void help_hyperlink_callback(GtkWidget *w)
   s=gtk_label_get_text(GTK_LABEL(w));
   type=GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(w), "page_type"));
 
-  /* May be able to skip, or may need to modify, advances[A_NONE].name
+  /* FIXME: May be able to skip, or may need to modify, advances[A_NONE]
      below, depending on which i18n is done elsewhere.
   */
-  if (strcmp(s, _("(Never)")) != 0
-      && strcmp(s, _("None")) != 0
-      && strcmp(s, advances[A_NONE].name) != 0)
+  if (strcmp(s, REQ_NEVER) != 0
+      && strcmp(s, REQ_NONE) != 0
+      && strcmp(s, advance_name_translation(A_NONE)) != 0)
     select_help_item_string(s, type);
 }
 
@@ -503,7 +509,7 @@ static void create_help_dialog(void)
   help_box = gtk_vbox_new(FALSE, 5);
   gtk_container_add(GTK_CONTAINER(help_frame), help_box);
 
-  unit_tile = gtk_pixcomm_new(UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT);
+  unit_tile = gtk_pixcomm_new(tileset_full_tile_width(tileset), tileset_full_tile_height(tileset));
   gtk_box_pack_start(GTK_BOX(help_box), unit_tile, FALSE, FALSE, 0);
 
   help_itable = gtk_table_new(1, 6, FALSE);
@@ -683,25 +689,32 @@ static void help_update_improvement(const struct help_item *pitem,
   
   create_help_page(HELP_IMPROVEMENT);
   
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
+  if (which<game.control.num_impr_types) {
+    struct impr_type *imp = improvement_by_number(which);
+    char *req = REQ_NONE;
+    char req_buf[512];
+
     sprintf(buf, "%d", impr_build_shield_cost(which));
     gtk_label_set_text(GTK_LABEL(help_ilabel[1]), buf);
     sprintf(buf, "%d", imp->upkeep);
     gtk_label_set_text(GTK_LABEL(help_ilabel[3]), buf);
-    if (imp->tech_req == A_LAST) {
-      gtk_label_set_text(GTK_LABEL(help_ilabel[5]), _("(Never)"));
-    } else {
-      gtk_label_set_text(GTK_LABEL(help_ilabel[5]),
-			 get_tech_name(game.player_ptr, imp->tech_req));
-    }
+
+    /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+     * Currently it's limited to 1 req but this code is partially prepared
+     * to be extended.  Remember MAX_NUM_REQS is a compile-time
+     * definition. */
+    requirement_vector_iterate(&imp->reqs, preq) {
+      req = get_req_source_text(&preq->source, req_buf, sizeof(req_buf));
+      break;
+    } requirement_vector_iterate_end;
+    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), req);
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
     gtk_label_set_text(GTK_LABEL(help_ilabel[1]), "0");
     gtk_label_set_text(GTK_LABEL(help_ilabel[3]), "0");
-    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), _("(Never)"));
-/*    create_tech_tree(help_improvement_tree, 0, game.num_tech_types, 3);*/
+    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), REQ_NEVER);
+/*    create_tech_tree(help_improvement_tree, 0, game.control.num_tech_types, 3);*/
   }
   gtk_widget_show(help_itable);
 
@@ -720,30 +733,40 @@ static void help_update_wonder(const struct help_item *pitem,
 
   create_help_page(HELP_WONDER);
 
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
+  if (which<game.control.num_impr_types) {
+    struct impr_type *imp = improvement_by_number(which);
+    int i;
+    char req_buf[512];
+
     sprintf(buf, "%d", impr_build_shield_cost(which));
     gtk_label_set_text(GTK_LABEL(help_wlabel[1]), buf);
-    if (imp->tech_req == A_LAST) {
-      gtk_label_set_text(GTK_LABEL(help_wlabel[3]), _("(Never)"));
-    } else {
-      gtk_label_set_text(GTK_LABEL(help_wlabel[3]),
-			 get_tech_name(game.player_ptr, imp->tech_req));
-    }
+
+    /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+     * Currently it's limited to 1 req but this code is partially prepared
+     * to be extended.  Remember MAX_NUM_REQS is a compile-time
+     * definition. */
+    i = 0;
+    requirement_vector_iterate(&imp->reqs, preq) {
+      gtk_label_set_text(GTK_LABEL(help_wlabel[3 + i]),
+			 get_req_source_text(&preq->source,
+					     req_buf, sizeof(req_buf)));
+      i++;
+      break;
+    } requirement_vector_iterate_end;
     if (tech_exists(imp->obsolete_by)) {
       gtk_label_set_text(GTK_LABEL(help_wlabel[5]),
-			 get_tech_name(game.player_ptr, imp->obsolete_by));
+			 advance_name_for_player(game.player_ptr, imp->obsolete_by));
     } else {
-      gtk_label_set_text(GTK_LABEL(help_wlabel[5]), _("(Never)"));
+      gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_NEVER);
     }
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
     /* can't find wonder */
     gtk_label_set_text(GTK_LABEL(help_wlabel[1]), "0");
-    gtk_label_set_text(GTK_LABEL(help_wlabel[3]), _("(Never)"));
-    gtk_label_set_text(GTK_LABEL(help_wlabel[5]), _("None"));
-/*    create_tech_tree(help_improvement_tree, 0, game.num_tech_types, 3); */
+    gtk_label_set_text(GTK_LABEL(help_wlabel[3]), REQ_NEVER);
+    gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_NONE);
+/*    create_tech_tree(help_improvement_tree, 0, game.control.num_tech_types, 3); */
   }
   gtk_widget_show(help_wtable);
 
@@ -756,15 +779,14 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, int i)
+				  char *title, struct unit_type *utype)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
 
   create_help_page(HELP_UNIT);
 
-  if (i<game.num_unit_types) {
-    struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d", unit_build_shield_cost(i));
+  if (utype) {
+    sprintf(buf, "%d", unit_build_shield_cost(utype));
     gtk_label_set_text(GTK_LABEL(help_ulabel[0][1]), buf);
     sprintf(buf, "%d", utype->attack_strength);
     gtk_label_set_text(GTK_LABEL(help_ulabel[0][4]), buf);
@@ -776,24 +798,26 @@ static void help_update_unit_type(const struct help_item *pitem,
     gtk_label_set_text(GTK_LABEL(help_ulabel[2][1]), buf);
     sprintf(buf, "%d", utype->hp);
     gtk_label_set_text(GTK_LABEL(help_ulabel[2][4]), buf);
-    gtk_label_set_text(GTK_LABEL(help_ulabel[3][1]), helptext_unit_upkeep_str(i));
-    sprintf(buf, "%d", utype->vision_range);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[3][1]),
+		       helptext_unit_upkeep_str(utype));
+    sprintf(buf, "%d", (int)sqrt((double)utype->vision_radius_sq));
     gtk_label_set_text(GTK_LABEL(help_ulabel[3][4]), buf);
     if(utype->tech_requirement==A_LAST) {
-      gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), _("(Never)"));
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_NEVER);
     } else {
       gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]),
-			 get_tech_name(game.player_ptr,
+			 advance_name_for_player(game.player_ptr,
 				       utype->tech_requirement));
     }
 /*    create_tech_tree(help_improvement_tree, 0, utype->tech_requirement, 3);*/
-    if(utype->obsoleted_by==-1) {
-      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), _("None"));
+    if (utype->obsoleted_by == U_NOT_OBSOLETED) {
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_NONE);
     } else {
-      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), get_unit_type(utype->obsoleted_by)->name);
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]),
+			 utype_name_translation(utype->obsoleted_by));
     }
 
-    helptext_unit(buf, i, pitem->text);
+    helptext_unit(buf, sizeof(buf), utype, pitem->text);
 
     gtk_text_buffer_set_text(help_text, buf, -1);
     gtk_widget_show(help_text_sw);
@@ -804,7 +828,7 @@ static void help_update_unit_type(const struct help_item *pitem,
 
       store.type = CANVAS_PIXCOMM;
       store.v.pixcomm = GTK_PIXCOMM(unit_tile);
-      create_overlay_unit(&store, i);
+      create_overlay_unit(&store, utype);
     }
     gtk_pixcomm_thaw(GTK_PIXCOMM(unit_tile));
     gtk_widget_show(unit_tile);
@@ -819,9 +843,9 @@ static void help_update_unit_type(const struct help_item *pitem,
     gtk_label_set_text(GTK_LABEL(help_ulabel[3][1]), "0");
     gtk_label_set_text(GTK_LABEL(help_ulabel[3][4]), "0");
 
-    gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), _("(Never)"));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_NEVER);
 /*    create_tech_tree(help_improvement_tree, 0, A_LAST, 3);*/
-    gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), _("None"));
+    gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_NONE);
 
     gtk_text_buffer_set_text(help_text, buf, -1);
     gtk_widget_show(help_text_sw);
@@ -855,7 +879,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 {
   int j;
   GtkWidget *w, *hbox;
-  char *buf = &long_buffer[0];
+  char buf[4096];
 
   create_help_page(HELP_TECH);
 
@@ -873,10 +897,19 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_widget_show(help_tree_sw);
     gtk_widget_show(help_tree_buttons_hbox);
 
-    helptext_tech(buf, i, pitem->text);
+    helptext_tech(buf, sizeof(buf), i, pitem->text);
     len = strlen(buf);
     my_chomp(buf, len);
 
+    if (get_tech_sprite(tileset, i)) {
+      GdkPixbuf *pixbuf = sprite_get_pixbuf(get_tech_sprite(tileset, i));
+
+      w = gtk_image_new_from_pixbuf(pixbuf);
+      gtk_widget_set_name(w, "help tech image");
+      gtk_box_pack_start(GTK_BOX(help_vbox), w, FALSE, FALSE, 0);
+      gtk_widget_show(w);
+    }
+
     w = gtk_text_view_new();
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), FALSE);
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
@@ -896,47 +929,55 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_widget_show(w);
 
     impr_type_iterate(j) {
-      if(i==improvement_types[j].tech_req) {
-        hbox = gtk_hbox_new(FALSE, 0);
-        gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
-        w = gtk_label_new(_("Allows"));
-        gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(improvement_types[j].name,
-			  is_wonder(j)?HELP_WONDER:HELP_IMPROVEMENT);
-        gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        gtk_widget_show_all(hbox);
-      }
-      if(i==improvement_types[j].obsolete_by) {
+      /* FIXME: need a more general mechanism for this, since this
+       * helptext needs to be shown in all possible req source types. */
+      requirement_vector_iterate(&improvement_by_number(j)->reqs, preq) {
+	if (preq->source.type == REQ_TECH
+	    && preq->source.value.tech == i) {
+	  hbox = gtk_hbox_new(FALSE, 0);
+	  gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
+	  w = gtk_label_new(_("Allows"));
+	  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	  w = help_slink_new(improvement_name_translation(j),
+	      is_great_wonder(j) ? HELP_WONDER
+	      : HELP_IMPROVEMENT);
+	  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	  gtk_widget_show_all(hbox);
+	}
+      } requirement_vector_iterate_end;
+      if (i == improvement_by_number(j)->obsolete_by) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
         w = gtk_label_new(_("Obsoletes"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(improvement_types[j].name,
-			  is_wonder(j)?HELP_WONDER:HELP_IMPROVEMENT);
+        w = help_slink_new(improvement_name_translation(j),
+			  is_great_wonder(j) ? HELP_WONDER : HELP_IMPROVEMENT);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         gtk_widget_show_all(hbox);
       }
     } impr_type_iterate_end;
 
-    unit_type_iterate(j) {
-      if(i!=get_unit_type(j)->tech_requirement) continue;
+    unit_type_iterate(punittype) {
+      if (i != punittype->tech_requirement) {
+	continue;
+      }
       hbox = gtk_hbox_new(FALSE, 0);
       gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
       w = gtk_label_new(_("Allows"));
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-      w = help_slink_new(get_unit_type(j)->name, HELP_UNIT);
+      w = help_slink_new(utype_name_translation(punittype), HELP_UNIT);
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       gtk_widget_show_all(hbox);
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.num_tech_types; j++) {
+    for (j = 0; j < game.control.num_tech_types; j++) {
       if(i==advances[j].req[0]) {
 	if(advances[j].req[1]==A_NONE) {
           hbox = gtk_hbox_new(FALSE, 0);
           gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
           w = gtk_label_new(_("Allows"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(get_tech_name(game.player_ptr, j), HELP_TECH);
+          w = help_slink_new(advance_name_for_player(game.player_ptr, j), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           gtk_widget_show_all(hbox);
 	}
@@ -945,11 +986,11 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
           gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
           w = gtk_label_new(_("Allows"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(get_tech_name(game.player_ptr, j), HELP_TECH);
+          w = help_slink_new(advance_name_for_player(game.player_ptr, j), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = gtk_label_new(_("with"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(get_tech_name(game.player_ptr,
+          w = help_slink_new(advance_name_for_player(game.player_ptr,
 					   advances[j].req[1]), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = gtk_label_new(Q_("?techhelp:"));
@@ -962,11 +1003,11 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
         w = gtk_label_new(_("Allows"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(get_tech_name(game.player_ptr, j), HELP_TECH);
+        w = help_slink_new(advance_name_for_player(game.player_ptr, j), HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = gtk_label_new(_("with"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(get_tech_name(game.player_ptr,
+        w = help_slink_new(advance_name_for_player(game.player_ptr,
 					 advances[j].req[0]), HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = gtk_label_new(Q_("?techhelp:"));
@@ -982,105 +1023,90 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, int i)
+				char *title, struct terrain *pterrain)
 {
-  char *buf = &long_buffer[0];
-  struct tile_type *ptype = get_tile_type(i);
+  char buf[4096];
 
   create_help_page(HELP_TERRAIN);
 
-  if (i < T_COUNT) {
+  if (pterrain) {
     sprintf(buf, "%d/%d.%d",
-	    ptype->movement_cost,
-	    (int)(ptype->defense_bonus/10),
-	    ptype->defense_bonus%10);
+	    pterrain->movement_cost,
+	    (int)((pterrain->defense_bonus + 100) / 100),
+	    (pterrain->defense_bonus + 100) % 100 / 10);
     gtk_label_set_text(GTK_LABEL(help_tlabel[0][1]), buf);
 
     sprintf(buf, "%d/%d/%d",
-	    ptype->food,
-	    ptype->shield,
-	    ptype->trade);
+	    pterrain->output[O_FOOD],
+	    pterrain->output[O_SHIELD],
+	    pterrain->output[O_TRADE]);
     gtk_label_set_text(GTK_LABEL(help_tlabel[0][4]), buf);
 
-    if (*(ptype->special_1_name)) {
-      sprintf(buf, _("%s F/R/T:"),
-	       ptype->special_1_name);
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][0]), buf);
-      sprintf(buf, "%d/%d/%d",
-	      ptype->food_special_1,
-	      ptype->shield_special_1,
-	      ptype->trade_special_1);
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][1]), buf);
-    } else {
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][0]), "");
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][1]), "");
-    }
+    buf[0] = '\0';
+    if (*(pterrain->resources)) {
+      struct resource **r;
 
-    if (*(ptype->special_2_name)) {
-      sprintf(buf, _("%s F/R/T:"),
-	      ptype->special_2_name);
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][3]), buf);
-      sprintf(buf, "%d/%d/%d",
-	      ptype->food_special_2,
-	      ptype->shield_special_2,
-	      ptype->trade_special_2);
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][4]), buf);
+      for (r = pterrain->resources; *r; r++) {
+	sprintf (buf + strlen (buf), " %s,", resource_name_translation(*r));
+      }
+      buf[strlen (buf) - 1] = '.';
     } else {
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][3]), "");
-      gtk_label_set_text(GTK_LABEL(help_tlabel[1][4]), "");
+      /* TRANS: "Resources: (none)" */
+      sprintf (buf + strlen (buf), _("(none)"));
     }
+    gtk_label_set_text(GTK_LABEL(help_tlabel[1][1]), buf);
 
-    if (ptype->road_trade_incr > 0) {
+    if (pterrain->road_trade_incr > 0) {
       sprintf(buf, _("+%d Trade / %d"),
-	      ptype->road_trade_incr,
-	      ptype->road_time);
-    } else if (ptype->road_time > 0) {
+	      pterrain->road_trade_incr,
+	      pterrain->road_time);
+    } else if (pterrain->road_time > 0) {
       sprintf(buf, _("no extra / %d"),
-	      ptype->road_time);
+	      pterrain->road_time);
     } else {
       strcpy(buf, _("n/a"));
     }
     gtk_label_set_text(GTK_LABEL(help_tlabel[2][1]), buf);
 
     strcpy(buf, _("n/a"));
-    if (ptype->irrigation_result == i) {
-      if (ptype->irrigation_food_incr > 0) {
+    if (pterrain->irrigation_result == pterrain) {
+      if (pterrain->irrigation_food_incr > 0) {
 	sprintf(buf, _("+%d Food / %d"),
-		ptype->irrigation_food_incr,
-		ptype->irrigation_time);
+		pterrain->irrigation_food_incr,
+		pterrain->irrigation_time);
       }
-    } else if (ptype->irrigation_result != T_NONE) {
+    } else if (pterrain->irrigation_result != T_NONE) {
       sprintf(buf, "%s / %d",
-	      get_tile_type(ptype->irrigation_result)->terrain_name,
-	      ptype->irrigation_time);
+	      terrain_name_translation(pterrain->irrigation_result),
+	      pterrain->irrigation_time);
     }
     gtk_label_set_text(GTK_LABEL(help_tlabel[2][4]), buf);
 
     strcpy(buf, _("n/a"));
-    if (ptype->mining_result == i) {
-      if (ptype->mining_shield_incr > 0) {
+    if (pterrain->mining_result == pterrain) {
+      if (pterrain->mining_shield_incr > 0) {
 	sprintf(buf, _("+%d Res. / %d"),
-		ptype->mining_shield_incr,
-		ptype->mining_time);
+		pterrain->mining_shield_incr,
+		pterrain->mining_time);
       }
-    } else if (ptype->mining_result != T_NONE) {
+    } else if (pterrain->mining_result != T_NONE) {
       sprintf(buf, "%s / %d",
-	      get_tile_type(ptype->mining_result)->terrain_name,
-	      ptype->mining_time);
+	      terrain_name_translation(pterrain->mining_result),
+	      pterrain->mining_time);
     }
     gtk_label_set_text(GTK_LABEL(help_tlabel[3][1]), buf);
 
-    if (ptype->transform_result != T_NONE) {
+    if (pterrain->transform_result != T_NONE) {
       sprintf(buf, "%s / %d",
-	      get_tile_type(ptype->transform_result)->terrain_name,
-	       ptype->transform_time);
+	      terrain_name_translation(pterrain->transform_result),
+	      pterrain->transform_time);
     } else {
       strcpy(buf, "n/a");
     }
     gtk_label_set_text(GTK_LABEL(help_tlabel[3][4]), buf);
   }
 
-  helptext_terrain(buf, i, pitem->text);
+  helptext_terrain(buf, sizeof(buf), pterrain, pitem->text);
   
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
@@ -1094,12 +1120,12 @@ static void help_update_terrain(const struct help_item *pitem,
 static void help_update_government(const struct help_item *pitem,
 				   char *title, struct government *gov)
 {
-  char *buf = &long_buffer[0];
+  char buf[4096];
 
   if (!gov) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, gov-governments, pitem->text);
+    helptext_government(buf, sizeof(buf), gov, pitem->text);
   }
   create_help_page(HELP_TEXT);
   gtk_text_buffer_set_text(help_text, buf, -1);
@@ -1125,26 +1151,30 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && is_wonder(i)) i = B_LAST;
+    i = find_improvement_by_translated_name(top);
+    if (i != B_LAST && is_great_wonder(i)) {
+      i = B_LAST;
+    }
     help_update_improvement(pitem, top, i);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && !is_wonder(i)) i = B_LAST;
+    i = find_improvement_by_translated_name(top);
+    if (i != B_LAST && !is_great_wonder(i)) {
+      i = B_LAST;
+    }
     help_update_wonder(pitem, top, i);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_name(top));
+    help_update_unit_type(pitem, top, find_unit_type_by_translated_name(top));
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_tech_by_name(top));
+    help_update_tech(pitem, top, find_advance_by_translated_name(top));
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, get_terrain_by_name(top));
+    help_update_terrain(pitem, top, find_terrain_by_translated_name(top));
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_name(top));
+    help_update_government(pitem, top, find_government_by_translated_name(top));
     break;
   case HELP_TEXT:
   default:
