@@ -46,6 +46,7 @@
 #include "gui_stuff.h"
 #include "mapctrl.h"
 #include "options.h"
+#include "overview_common.h"
 #include "tilespec.h"
 #include "text.h"
 #include "wldlg.h"
@@ -53,23 +54,8 @@
 #include "citydlg.h" /* For reset_city_dialogs() */
 #include "mapview.h"
 
-#define map_canvas_store (mapview_canvas.store->v.pixmap)
-
-static void pixmap_put_overlay_tile(GdkDrawable *pixmap,
-				    int canvas_x, int canvas_y,
-				    struct Sprite *ssprite);
-
-static void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
-					 int canvas_x, int canvas_y,
-					 struct Sprite *ssprite,
-					 bool fog);
-
-/* the intro picture is held in this pixmap, which is scaled to
-   the screen size */
-static SPRITE *scaled_intro_sprite = NULL;
-
 static GtkObject *map_hadj, *map_vadj;
-
+static int cursor_timer_id = 0, cursor_type = -1, cursor_frame = 0;
 
 /**************************************************************************
   If do_restore is FALSE it will invert the turn done button style. If
@@ -111,44 +97,47 @@ void update_timeout_label(void)
 **************************************************************************/
 void update_info_label( void )
 {
-  int  d;
-  int  sol, flake;
   GtkWidget *label;
 
   label = gtk_frame_get_label_widget(GTK_FRAME(main_frame_civ_name));
-  gtk_label_set_text(GTK_LABEL(label),
-		     get_nation_name(game.player_ptr->nation));
+  if (game.player_ptr) {
+    gtk_label_set_text(GTK_LABEL(label),
+		       nation_adjective_for_player(game.player_ptr));
+  } else {
+    gtk_label_set_text(GTK_LABEL(label), "-");
+  }
 
   gtk_label_set_text(GTK_LABEL(main_label_info), get_info_label_text());
 
-  sol = client_warming_sprite();
-  flake = client_cooling_sprite();
   set_indicator_icons(client_research_sprite(),
-		      sol,
-		      flake,
-		      game.player_ptr->government);
+		      client_warming_sprite(),
+		      client_cooling_sprite(),
+		      client_government_sprite());
 
-  d=0;
-  for (; d < game.player_ptr->economic.luxury /10; d++) {
-    struct Sprite *sprite = sprites.tax_luxury;
+  if (game.player_ptr) {
+    int d = 0;
 
-    gtk_image_set_from_pixmap(GTK_IMAGE(econ_label[d]),
-			      sprite->pixmap, sprite->mask);
-  }
+    for (; d < game.player_ptr->economic.luxury /10; d++) {
+      struct sprite *sprite = get_tax_sprite(tileset, O_LUXURY);
+
+      gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]),
+				sprite_get_pixbuf(sprite));
+    }
  
-  for (; d < (game.player_ptr->economic.science
-	     + game.player_ptr->economic.luxury) / 10; d++) {
-    struct Sprite *sprite = sprites.tax_science;
+    for (; d < (game.player_ptr->economic.science
+		+ game.player_ptr->economic.luxury) / 10; d++) {
+      struct sprite *sprite = get_tax_sprite(tileset, O_SCIENCE);
 
-    gtk_image_set_from_pixmap(GTK_IMAGE(econ_label[d]),
-			      sprite->pixmap, sprite->mask);
-  }
+      gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]),
+				sprite_get_pixbuf(sprite));
+    }
  
-  for (; d < 10; d++) {
-    struct Sprite *sprite = sprites.tax_gold;
+    for (; d < 10; d++) {
+      struct sprite *sprite = get_tax_sprite(tileset, O_GOLD);
 
-    gtk_image_set_from_pixmap(GTK_IMAGE(econ_label[d]),
-			      sprite->pixmap, sprite->mask);
+      gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]),
+				sprite_get_pixbuf(sprite));
+    }
   }
  
   update_timeout_label();
@@ -167,6 +156,43 @@ void update_info_label( void )
 		       "");
 }
 
+/**************************************************************************
+  This function is used to animate the mouse cursor. 
+**************************************************************************/
+static gint anim_cursor_cb(gpointer data)
+{
+  if (!cursor_timer_id) {
+    return FALSE;
+  }
+
+  cursor_frame++;
+  if (cursor_frame == NUM_CURSOR_FRAMES) {
+    cursor_frame = 0;
+  }
+
+  if (cursor_type == CURSOR_DEFAULT) {
+    gdk_window_set_cursor(root_window, NULL);
+    cursor_timer_id = 0;
+    return FALSE; 
+  }
+
+  gdk_window_set_cursor(root_window,
+                fc_cursors[cursor_type][cursor_frame]);
+  control_mouse_cursor(NULL);
+  return TRUE;
+}
+
+/**************************************************************************
+  This function will change the current mouse cursor.
+**************************************************************************/
+static void modify_mouse_cursor(enum cursor_type new_cursor_type)
+{
+  cursor_type = new_cursor_type;
+  if (!cursor_timer_id) {
+    cursor_timer_id = gtk_timeout_add(CURSOR_INTERVAL, anim_cursor_cb, NULL);
+  }
+}
+
 /**************************************************************************
   Update the information label which gives info on the current unit and the
   square under the current unit, for specified unit.  Note that in practice
@@ -177,118 +203,118 @@ void update_info_label( void )
   Also calls update_unit_pix_label() to update the icons for units on this
   square.
 **************************************************************************/
-void update_unit_info_label(struct unit *punit)
+void update_unit_info_label(struct unit_list *punits)
 {
+  enum cursor_type mouse_cursor_type = CURSOR_DEFAULT;
   GtkWidget *label;
 
   label = gtk_frame_get_label_widget(GTK_FRAME(unit_info_frame));
   gtk_label_set_text(GTK_LABEL(label),
-		     get_unit_info_label_text1(punit));
+		     get_unit_info_label_text1(punits));
 
   gtk_label_set_text(GTK_LABEL(unit_info_label),
-		     get_unit_info_label_text2(punit));
+		     get_unit_info_label_text2(punits));
 
-  if(punit) {
-    if (hover_unit != punit->id)
-      set_hover_state(NULL, HOVER_NONE, ACTIVITY_LAST);
-
-    switch (hover_state) {
-    case HOVER_NONE:
-      gdk_window_set_cursor (root_window, NULL);
-      break;
-    case HOVER_PATROL:
-      gdk_window_set_cursor (root_window, patrol_cursor);
-      break;
-    case HOVER_GOTO:
-    case HOVER_CONNECT:
-      gdk_window_set_cursor (root_window, goto_cursor);
-      break;
-    case HOVER_NUKE:
-      gdk_window_set_cursor (root_window, nuke_cursor);
-      break;
-    case HOVER_PARADROP:
-      gdk_window_set_cursor (root_window, drop_cursor);
-      break;
+  if (action_state == CURSOR_ACTION_WAIT) {
+    mouse_cursor_type = CURSOR_WAIT;
+  } else {
+  switch (hover_state) {
+  case HOVER_NONE:
+    if (action_state == CURSOR_ACTION_SELECT) {
+      mouse_cursor_type = CURSOR_SELECT;
+    } else if (action_state == CURSOR_ACTION_PARATROOPER) {
+      mouse_cursor_type = CURSOR_PARADROP;
+    } else if (action_state == CURSOR_ACTION_NUKE) {
+      mouse_cursor_type = CURSOR_NUKE;
+    } else {
+      mouse_cursor_type = CURSOR_DEFAULT;
     }
-  } else {
-    gdk_window_set_cursor(root_window, NULL);
+    break;
+  case HOVER_PATROL:
+    if (action_state == CURSOR_ACTION_INVALID) {
+      mouse_cursor_type = CURSOR_INVALID;
+    } else {
+      mouse_cursor_type = CURSOR_PATROL;
+    }
+    break;
+  case HOVER_GOTO:
+    if (action_state == CURSOR_ACTION_GOTO) {
+      mouse_cursor_type = CURSOR_GOTO;
+    } else if (action_state == CURSOR_ACTION_DEFAULT) {
+      mouse_cursor_type = CURSOR_DEFAULT;
+    } else if (action_state == CURSOR_ACTION_ATTACK) {
+      mouse_cursor_type = CURSOR_ATTACK;
+    } else {
+      mouse_cursor_type = CURSOR_INVALID;
+    }
+    break;
+  case HOVER_CONNECT:
+    if (action_state == CURSOR_ACTION_INVALID) {
+      mouse_cursor_type = CURSOR_INVALID;
+    } else {
+      mouse_cursor_type = CURSOR_GOTO;
+    }
+    break;
+  case HOVER_NUKE:
+    mouse_cursor_type = CURSOR_NUKE;
+    break;
+  case HOVER_PARADROP:
+    mouse_cursor_type = CURSOR_PARADROP;
+    break;
   }
-  update_unit_pix_label(punit);
-}
-
-
-/**************************************************************************
-...
-**************************************************************************/
-GdkPixmap *get_thumb_pixmap(int onoff)
-{
-  return sprites.treaty_thumb[BOOL_VAL(onoff)]->pixmap;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void set_indicator_icons(int bulb, int sol, int flake, int gov)
-{
-  struct Sprite *gov_sprite;
-
-  bulb = CLIP(0, bulb, NUM_TILES_PROGRESS-1);
-  sol = CLIP(0, sol, NUM_TILES_PROGRESS-1);
-  flake = CLIP(0, flake, NUM_TILES_PROGRESS-1);
-
-  gtk_image_set_from_pixmap(GTK_IMAGE(bulb_label),
-			    sprites.bulb[bulb]->pixmap, NULL);
-  gtk_image_set_from_pixmap(GTK_IMAGE(sun_label),
-			    sprites.warming[sol]->pixmap, NULL);
-  gtk_image_set_from_pixmap(GTK_IMAGE(flake_label),
-			    sprites.cooling[flake]->pixmap, NULL);
-
-  if (game.government_count==0) {
-    /* HACK: the UNHAPPY citizen is used for the government
-     * when we don't know any better. */
-    struct citizen_type c = {.type = CITIZEN_UNHAPPY};
-
-    gov_sprite = get_citizen_sprite(c, 0, NULL);
-  } else {
-    gov_sprite = get_government(gov)->sprite;
   }
-  gtk_image_set_from_pixmap(GTK_IMAGE(government_label),
-			    gov_sprite->pixmap, NULL);
+
+  modify_mouse_cursor(mouse_cursor_type);
+
+  update_unit_pix_label(punits);
+}
+
+
+/**************************************************************************
+...
+**************************************************************************/
+GdkPixbuf *get_thumb_pixbuf(int onoff)
+{
+  return sprite_get_pixbuf(get_treaty_thumb_sprite(tileset, BOOL_VAL(onoff)));
+}
+
+/****************************************************************************
+  Set information for the indicator icons typically shown in the main
+  client window.  The parameters tell which sprite to use for the
+  indicator.
+****************************************************************************/
+void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
+			 struct sprite *flake, struct sprite *gov)
+{
+  gtk_image_set_from_pixbuf(GTK_IMAGE(bulb_label),
+			    sprite_get_pixbuf(bulb));
+  gtk_image_set_from_pixbuf(GTK_IMAGE(sun_label),
+			    sprite_get_pixbuf(sol));
+  gtk_image_set_from_pixbuf(GTK_IMAGE(flake_label),
+			    sprite_get_pixbuf(flake));
+  gtk_image_set_from_pixbuf(GTK_IMAGE(government_label),
+			    sprite_get_pixbuf(gov));
+}
+
+/****************************************************************************
+  Return the dimensions of the area (container widget; maximum size) for
+  the overview.
+****************************************************************************/
+void get_overview_area_dimensions(int *width, int *height)
+{
+  gdk_drawable_get_size(overview_canvas->window, width, height);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void map_size_changed(void)
+void overview_size_changed(void)
 {
   gtk_widget_set_size_request(overview_canvas,
 			      overview.width, overview.height);
   update_map_canvas_scrollbars_size();
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-struct canvas *canvas_create(int width, int height)
-{
-  struct canvas *result = fc_malloc(sizeof(*result));
-
-  result->type = CANVAS_PIXMAP;
-  result->v.pixmap = gdk_pixmap_new(root_window, width, height, -1);
-  return result;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void canvas_free(struct canvas *store)
-{
-  if (store->type == CANVAS_PIXMAP) {
-    g_object_unref(store->v.pixmap);
-  }
-  free(store);
-}
-
 /****************************************************************************
   Return a canvas that is the overview window.
 ****************************************************************************/
@@ -344,45 +370,21 @@ gboolean map_canvas_expose(GtkWidget *w, GdkEventExpose *ev, gpointer data)
   static bool cleared = FALSE;
 
   if (!can_client_change_view()) {
-    if (map_configure || !scaled_intro_sprite) {
-
-      if (!intro_gfx_sprite) {
-        load_intro_gfx();
-      }
-
-      if (scaled_intro_sprite) {
-        free_sprite(scaled_intro_sprite);
-      }
-
-      scaled_intro_sprite = sprite_scale(intro_gfx_sprite,
-					 w->allocation.width,
-					 w->allocation.height);
-    }
-
-    if (scaled_intro_sprite) {
-      gdk_draw_drawable(map_canvas->window, civ_gc,
-			scaled_intro_sprite->pixmap,
-			ev->area.x, ev->area.y, ev->area.x, ev->area.y,
-			ev->area.width, ev->area.height);
-      gtk_widget_queue_draw(overview_canvas);
-      cleared = FALSE;
-    } else {
-      if (!cleared) {
-	gtk_widget_queue_draw(w);
-	cleared = TRUE;
-      }
+    if (!cleared) {
+      gtk_widget_queue_draw(w);
+      cleared = TRUE;
     }
     map_center = TRUE;
   }
   else
   {
-    if (scaled_intro_sprite) {
-      free_sprite(scaled_intro_sprite);
-      scaled_intro_sprite = NULL;
-    }
-
     if (map_exists()) { /* do we have a map at all */
-      gdk_draw_drawable(map_canvas->window, civ_gc, map_canvas_store,
+      /* First we mark the area to be updated as dirty.  Then we unqueue
+       * any pending updates, to make sure only the most up-to-date data
+       * is written (otherwise drawing bugs happen when old data is copied
+       * to screen).  Then we draw all changed areas to the screen. */
+      unqueue_mapview_updates(FALSE);
+      gdk_draw_drawable(map_canvas->window, civ_gc, mapview.store->v.pixmap,
 			ev->area.x, ev->area.y, ev->area.x, ev->area.y,
 			ev->area.width, ev->area.height);
       cleared = FALSE;
@@ -411,7 +413,7 @@ gboolean map_canvas_expose(GtkWidget *w, GdkEventExpose *ev, gpointer data)
 void flush_mapcanvas(int canvas_x, int canvas_y,
 		     int pixel_width, int pixel_height)
 {
-  gdk_draw_drawable(map_canvas->window, civ_gc, map_canvas_store,
+  gdk_draw_drawable(map_canvas->window, civ_gc, mapview.store->v.pixmap,
 		    canvas_x, canvas_y, canvas_x, canvas_y,
 		    pixel_width, pixel_height);
 }
@@ -511,119 +513,6 @@ void update_city_descriptions(void)
   update_map_canvas_visible();
 }
 
-/**************************************************************************
-  If necessary, clear the city descriptions out of the buffer.
-**************************************************************************/
-void prepare_show_city_descriptions(void)
-{
-  /* Nothing to do */
-}
-
-/****************************************************************************
-  Draw a description for the given city.  This description may include the
-  name, turns-to-grow, production, and city turns-to-build (depending on
-  client options).
-
-  (canvas_x, canvas_y) gives the location on the given canvas at which to
-  draw the description.  This is the location of the city itself so the
-  text must be drawn underneath it.  pcity gives the city to be drawn,
-  while (*width, *height) should be set by show_ctiy_desc to contain the
-  width and height of the text block (centered directly underneath the
-  city's tile).
-****************************************************************************/
-void show_city_desc(struct canvas *pcanvas, int canvas_x, int canvas_y,
-		    struct city *pcity, int *width, int *height)
-{
-  if (pcanvas->type == CANVAS_PIXMAP) {
-  static char buffer[512], buffer2[32];
-  PangoRectangle rect, rect2;
-  enum color_std color;
-  int extra_width = 0;
-  static PangoLayout *layout;
-
-  if (!layout) {
-    layout = pango_layout_new(gdk_pango_context_get());
-  }
-
-  *width = *height = 0;
-
-  canvas_x += NORMAL_TILE_WIDTH / 2;
-  canvas_y += NORMAL_TILE_HEIGHT;
-
-  if (draw_city_names) {
-    get_city_mapview_name_and_growth(pcity, buffer, sizeof(buffer),
-				     buffer2, sizeof(buffer2), &color);
-
-    pango_layout_set_font_description(layout, main_font);
-    if (buffer2[0] != '\0') {
-      /* HACK: put a character's worth of space between the two strings. */
-      pango_layout_set_text(layout, "M", -1);
-      pango_layout_get_pixel_extents(layout, &rect, NULL);
-      extra_width = rect.width;
-    }
-    pango_layout_set_text(layout, buffer, -1);
-    pango_layout_get_pixel_extents(layout, &rect, NULL);
-    rect.width += extra_width;
-
-    if (draw_city_growth && pcity->owner == game.player_idx) {
-      /* We need to know the size of the growth text before
-	 drawing anything. */
-      pango_layout_set_font_description(layout, city_productions_font);
-      pango_layout_set_text(layout, buffer2, -1);
-      pango_layout_get_pixel_extents(layout, &rect2, NULL);
-
-      /* Now return the layout to its previous state. */
-      pango_layout_set_font_description(layout, main_font);
-      pango_layout_set_text(layout, buffer, -1);
-    } else {
-      rect2.width = 0;
-    }
-
-    gtk_draw_shadowed_string(pcanvas->v.pixmap,
-			     toplevel->style->black_gc,
-			     toplevel->style->white_gc,
-			     canvas_x - (rect.width + rect2.width) / 2,
-			     canvas_y + PANGO_ASCENT(rect), layout);
-
-    if (draw_city_growth && pcity->owner == game.player_idx) {
-      pango_layout_set_font_description(layout, city_productions_font);
-      pango_layout_set_text(layout, buffer2, -1);
-      gdk_gc_set_foreground(civ_gc, colors_standard[color]);
-      gtk_draw_shadowed_string(pcanvas->v.pixmap,
-			       toplevel->style->black_gc,
-			       civ_gc,
-			       canvas_x - (rect.width + rect2.width) / 2
-			       + rect.width,
-			       canvas_y + PANGO_ASCENT(rect)
-			       + rect.height / 2 - rect2.height / 2,
-			       layout);
-    }
-
-    canvas_y += rect.height + 3;
-
-    *width = rect.width + rect2.width;
-    *height += rect.height + 3;
-  }
-
-  if (draw_city_productions && (pcity->owner==game.player_idx)) {
-    get_city_mapview_production(pcity, buffer, sizeof(buffer));
-
-    pango_layout_set_font_description(layout, city_productions_font);
-    pango_layout_set_text(layout, buffer, -1);
-
-    pango_layout_get_pixel_extents(layout, &rect, NULL);
-    gtk_draw_shadowed_string(pcanvas->v.pixmap,
-			     toplevel->style->black_gc,
-			     toplevel->style->white_gc,
-			     canvas_x - rect.width / 2,
-			     canvas_y + PANGO_ASCENT(rect), layout);
-
-    *width = MAX(*width, rect.width);
-    *height += rect.height;
-  }
-  }
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -658,7 +547,7 @@ void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p)
 
   gtk_pixcomm_freeze(p);
 
-  put_unit_city_overlays(punit, &store, 0, NORMAL_TILE_HEIGHT);
+  put_unit_city_overlays(punit, &store, 0, tileset_tile_height(tileset));
 
   gtk_pixcomm_thaw(p);
 }
@@ -666,22 +555,30 @@ void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p)
 /**************************************************************************
 ...
 **************************************************************************/
-static void pixmap_put_overlay_tile(GdkDrawable *pixmap,
-				    int canvas_x, int canvas_y,
-				    struct Sprite *ssprite)
+void pixmap_put_overlay_tile(GdkDrawable *pixmap,
+			     int canvas_x, int canvas_y,
+			     struct sprite *ssprite)
 {
   if (!ssprite) {
     return;
   }
-      
-  gdk_gc_set_clip_origin(civ_gc, canvas_x, canvas_y);
-  gdk_gc_set_clip_mask(civ_gc, ssprite->mask);
 
-  gdk_draw_drawable(pixmap, civ_gc, ssprite->pixmap,
+  if (ssprite->pixmap) {
+    gdk_gc_set_clip_origin(civ_gc, canvas_x, canvas_y);
+    gdk_gc_set_clip_mask(civ_gc, ssprite->mask);
+
+    gdk_draw_drawable(pixmap, civ_gc, ssprite->pixmap,
+		      0, 0,
+		      canvas_x, canvas_y,
+		      ssprite->width, ssprite->height);
+    gdk_gc_set_clip_mask(civ_gc, NULL);
+  } else {
+    gdk_draw_pixbuf(pixmap, civ_gc, ssprite->pixbuf,
 		    0, 0,
-		    canvas_x, canvas_y,
-		    ssprite->width, ssprite->height);
-  gdk_gc_set_clip_mask(civ_gc, NULL);
+		  canvas_x, canvas_y,
+		  ssprite->width, ssprite->height,
+		  GDK_RGB_DITHER_NONE, 0, 0);
+  }
 }
 
 /**************************************************************************
@@ -689,225 +586,67 @@ static void pixmap_put_overlay_tile(GdkDrawable *pixmap,
 **************************************************************************/
 static void pixmap_put_sprite(GdkDrawable *pixmap,
 			      int pixmap_x, int pixmap_y,
-			      struct Sprite *ssprite,
+			      struct sprite *ssprite,
 			      int offset_x, int offset_y,
 			      int width, int height)
 {
-  if (ssprite->mask) {
-    gdk_gc_set_clip_origin(civ_gc, pixmap_x, pixmap_y);
-    gdk_gc_set_clip_mask(civ_gc, ssprite->mask);
-  }
+#ifdef DEBUG
+  static int sprites = 0, pixbufs = 0;
+#endif
 
-  gdk_draw_drawable(pixmap, civ_gc, ssprite->pixmap,
+  if (ssprite->pixmap) {
+    if (ssprite->mask) {
+      gdk_gc_set_clip_origin(civ_gc, pixmap_x, pixmap_y);
+      gdk_gc_set_clip_mask(civ_gc, ssprite->mask);
+    }
+
+    gdk_draw_drawable(pixmap, civ_gc, ssprite->pixmap,
+		      offset_x, offset_y,
+		      pixmap_x + offset_x, pixmap_y + offset_y,
+		      MIN(width, MAX(0, ssprite->width - offset_x)),
+		      MIN(height, MAX(0, ssprite->height - offset_y)));
+
+    gdk_gc_set_clip_mask(civ_gc, NULL);
+  } else {
+    gdk_draw_pixbuf(pixmap, civ_gc, ssprite->pixbuf,
 		    offset_x, offset_y,
 		    pixmap_x + offset_x, pixmap_y + offset_y,
 		    MIN(width, MAX(0, ssprite->width - offset_x)),
-		    MIN(height, MAX(0, ssprite->height - offset_y)));
-
-  gdk_gc_set_clip_mask(civ_gc, NULL);
-}
-
-/**************************************************************************
-  Draw some or all of a sprite onto the mapview or citydialog canvas.
-**************************************************************************/
-void canvas_put_sprite(struct canvas *pcanvas,
-		       int canvas_x, int canvas_y,
-		       struct Sprite *sprite,
-		       int offset_x, int offset_y, int width, int height)
-{
-  switch (pcanvas->type) {
-    case CANVAS_PIXMAP:
-      pixmap_put_sprite(pcanvas->v.pixmap, canvas_x, canvas_y,
-	  sprite, offset_x, offset_y, width, height);
-      break;
-    case CANVAS_PIXCOMM:
-      gtk_pixcomm_copyto(pcanvas->v.pixcomm, sprite, canvas_x, canvas_y);
-      break;
-    case CANVAS_PIXBUF:
-      {
-	GdkPixbuf *src, *dst;
-
-	/* FIXME: is this right??? */
-	if (canvas_x < 0) {
-	  offset_x -= canvas_x;
-	  canvas_x = 0;
-	}
-	if (canvas_y < 0) {
-	  offset_y -= canvas_y;
-	  canvas_y = 0;
-	}
-
-
-	src = sprite_get_pixbuf(sprite);
-	dst = pcanvas->v.pixbuf;
-	gdk_pixbuf_composite(src, dst, canvas_x, canvas_y,
-	    MIN(width,
-	      MIN(gdk_pixbuf_get_width(dst), gdk_pixbuf_get_width(src))),
-	    MIN(height,
-	      MIN(gdk_pixbuf_get_height(dst), gdk_pixbuf_get_height(src))),
-	    canvas_x - offset_x, canvas_y - offset_y,
-	    1.0, 1.0, GDK_INTERP_NEAREST, 255);
-      }
-      break;
-    default:
-      break;
-  } 
-}
-
-/**************************************************************************
-  Draw a full sprite onto the mapview or citydialog canvas.
-**************************************************************************/
-void canvas_put_sprite_full(struct canvas *pcanvas,
-			    int canvas_x, int canvas_y,
-			    struct Sprite *sprite)
-{
-  canvas_put_sprite(pcanvas, canvas_x, canvas_y, sprite,
-		    0, 0, sprite->width, sprite->height);
-}
-
-/****************************************************************************
-  Draw a full sprite onto the canvas.  If "fog" is specified draw it with
-  fog.
-****************************************************************************/
-void canvas_put_sprite_fogged(struct canvas *pcanvas,
-			      int canvas_x, int canvas_y,
-			      struct Sprite *psprite,
-			      bool fog, int fog_x, int fog_y)
-{
-  if (pcanvas->type == CANVAS_PIXMAP) {
-    pixmap_put_overlay_tile_draw(pcanvas->v.pixmap, canvas_x, canvas_y,
-				 psprite, fog);
+		    MIN(height, MAX(0, ssprite->height - offset_y)),
+		    GDK_RGB_DITHER_NONE, 0, 0);
+#ifdef DEBUG
+    pixbufs++;
+#endif
   }
-}
 
-/**************************************************************************
-  Draw a filled-in colored rectangle onto the mapview or citydialog canvas.
-**************************************************************************/
-void canvas_put_rectangle(struct canvas *pcanvas,
-			  enum color_std color,
-			  int canvas_x, int canvas_y, int width, int height)
-{
-  GdkColor *col = colors_standard[color];
-
-  switch (pcanvas->type) {
-    case CANVAS_PIXMAP:
-      gdk_gc_set_foreground(fill_bg_gc, col);
-      gdk_draw_rectangle(pcanvas->v.pixmap, fill_bg_gc, TRUE,
-	  canvas_x, canvas_y, width, height);
-      break;
-    case CANVAS_PIXCOMM:
-      gtk_pixcomm_fill(pcanvas->v.pixcomm, col);
-      break;
-    case CANVAS_PIXBUF:
-      gdk_pixbuf_fill(pcanvas->v.pixbuf,
-	  ((guint32)(col->red & 0xff00) << 16)
-	  | ((col->green & 0xff00) << 8) | (col->blue & 0xff00) | 0xff);
-      break;
-    default:
-      break;
-  }
-}
-
-/****************************************************************************
-  Fill the area covered by the sprite with the given color.
-****************************************************************************/
-void canvas_fill_sprite_area(struct canvas *pcanvas,
-			     struct Sprite *psprite, enum color_std color,
-			     int canvas_x, int canvas_y)
-{
-  if (pcanvas->type == CANVAS_PIXMAP) {
-    gdk_gc_set_clip_origin(fill_bg_gc, canvas_x, canvas_y);
-    gdk_gc_set_clip_mask(fill_bg_gc, psprite->mask);
-    gdk_gc_set_foreground(fill_bg_gc, colors_standard[color]);
-
-    gdk_draw_rectangle(pcanvas->v.pixmap, fill_bg_gc, TRUE,
-		       canvas_x, canvas_y, psprite->width, psprite->height);
-
-    gdk_gc_set_clip_mask(fill_bg_gc, NULL);
-  }
-}
-
-/****************************************************************************
-  Fill the area covered by the sprite with the given color.
-****************************************************************************/
-void canvas_fog_sprite_area(struct canvas *pcanvas, struct Sprite *psprite,
-			    int canvas_x, int canvas_y)
-{
-  if (pcanvas->type == CANVAS_PIXMAP) {
-    gdk_gc_set_clip_origin(fill_tile_gc, canvas_x, canvas_y);
-    gdk_gc_set_clip_mask(fill_tile_gc, psprite->mask);
-    gdk_gc_set_foreground(fill_tile_gc, colors_standard[COLOR_STD_BLACK]);
-    gdk_gc_set_stipple(fill_tile_gc, black50);
-    gdk_gc_set_ts_origin(fill_tile_gc, canvas_x, canvas_y);
-
-    gdk_draw_rectangle(pcanvas->v.pixmap, fill_tile_gc, TRUE,
-		       canvas_x, canvas_y, psprite->width, psprite->height);
-
-    gdk_gc_set_clip_mask(fill_tile_gc, NULL); 
-  }
-}
-
-/**************************************************************************
-  Draw a colored line onto the mapview or citydialog canvas.
-**************************************************************************/
-void canvas_put_line(struct canvas *pcanvas, enum color_std color,
-		     enum line_type ltype, int start_x, int start_y,
-		     int dx, int dy)
-{
-  if (pcanvas->type == CANVAS_PIXMAP) {
-    GdkGC *gc = NULL;
-
-    switch (ltype) {
-    case LINE_NORMAL:
-      gc = thin_line_gc;
-      break;
-    case LINE_BORDER:
-      gc = border_line_gc;
-      break;
-    case LINE_TILE_FRAME:
-      gc = thick_line_gc;
-      break;
-    case LINE_GOTO:
-      gc = thick_line_gc;
-      break;
-    }
-
-    gdk_gc_set_foreground(gc, colors_standard[color]);
-    gdk_draw_line(pcanvas->v.pixmap, gc,
-		  start_x, start_y, start_x + dx, start_y + dy);
-  }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void canvas_copy(struct canvas *dest, struct canvas *src,
-		 int src_x, int src_y, int dest_x, int dest_y,
-		 int width, int height)
-{
-  if (dest->type == src->type) {
-    if (src->type == CANVAS_PIXMAP) {
-      gdk_draw_drawable(dest->v.pixmap, fill_bg_gc, src->v.pixmap,
-			src_x, src_y, dest_x, dest_y, width, height);
-    }
+#ifdef DEBUG
+  sprites++;
+  if (sprites % 1000 == 0) {
+    freelog(LOG_DEBUG, "%5d / %5d pixbufs = %d%%",
+	    pixbufs, sprites, 100 * pixbufs / sprites);
   }
+#endif
 }
 
 /**************************************************************************
   Created a fogged version of the sprite.  This can fail on older systems
   in which case the callers needs a fallback.
 **************************************************************************/
-static void fog_sprite(struct Sprite *sprite)
+static void fog_sprite(struct sprite *sprite)
 {
   int x, y;
   GdkPixbuf *fogged;
   guchar *pixel;
   const int bright = 65; /* Brightness percentage */
 
-  fogged = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
-			  sprite->width, sprite->height);
-  gdk_pixbuf_get_from_drawable(fogged, sprite->pixmap, NULL,
-			       0, 0, 0, 0, sprite->width, sprite->height);
+  if (sprite->pixmap) {
+    fogged = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
+			    sprite->width, sprite->height);
+    gdk_pixbuf_get_from_drawable(fogged, sprite->pixmap, NULL,
+				 0, 0, 0, 0, sprite->width, sprite->height);
+  } else {
+    fogged = gdk_pixbuf_copy(sprite->pixbuf);
+  }
 
   /* Iterate over all pixels, reducing brightness by 50%. */
   for (x = 0; x < sprite->width; x++) {
@@ -922,26 +661,33 @@ static void fog_sprite(struct Sprite *sprite)
     }
   }
 
-  gdk_pixbuf_render_pixmap_and_mask(fogged, &sprite->fogged,
-				    NULL, 0);
-  g_object_unref(fogged);
+  if (sprite->pixmap) {
+    gdk_pixbuf_render_pixmap_and_mask(fogged, &sprite->pixmap_fogged,
+				      NULL, 0);
+    g_object_unref(fogged);
+  } else {
+    sprite->pixbuf_fogged = fogged;
+  }
 }
 
 /**************************************************************************
 Only used for isometric view.
 **************************************************************************/
-static void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
-					 int canvas_x, int canvas_y,
-					 struct Sprite *ssprite,
-					 bool fog)
+void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
+				  int canvas_x, int canvas_y,
+				  struct sprite *ssprite,
+				  bool fog)
 {
   if (!ssprite) {
     return;
   }
 
-  if (fog && better_fog && !ssprite->fogged) {
+  if (fog && better_fog
+      && ((ssprite->pixmap && !ssprite->pixmap_fogged)
+	  || (!ssprite->pixmap && !ssprite->pixbuf_fogged))) {
     fog_sprite(ssprite);
-    if (!ssprite->fogged) {
+    if ((ssprite->pixmap && !ssprite->pixmap_fogged)
+	|| (!ssprite->pixmap && !ssprite->pixbuf_fogged)) {
       freelog(LOG_NORMAL,
 	      _("Better fog will only work in truecolor.  Disabling it"));
       better_fog = FALSE;
@@ -949,16 +695,23 @@ static void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
   }
 
   if (fog && better_fog) {
-    gdk_gc_set_clip_origin(fill_tile_gc, canvas_x, canvas_y);
-    gdk_gc_set_clip_mask(fill_tile_gc, ssprite->mask);
-
-    gdk_draw_drawable(pixmap, fill_tile_gc,
-		      ssprite->fogged,
-		      0, 0,
-		      canvas_x, canvas_y,
-		      ssprite->width, ssprite->height);
-    gdk_gc_set_clip_mask(fill_tile_gc, NULL);
-
+    if (ssprite->pixmap) {
+      if (ssprite->mask) {
+	gdk_gc_set_clip_origin(civ_gc, canvas_x, canvas_y);
+	gdk_gc_set_clip_mask(civ_gc, ssprite->mask);
+      }
+      gdk_draw_drawable(pixmap, civ_gc,
+			ssprite->pixmap_fogged,
+			0, 0,
+			canvas_x, canvas_y,
+			ssprite->width, ssprite->height);
+      gdk_gc_set_clip_mask(civ_gc, NULL);
+    } else {
+      gdk_draw_pixbuf(pixmap, civ_gc, ssprite->pixbuf_fogged,
+		      0, 0, canvas_x, canvas_y, 
+		      ssprite->width, ssprite->height,
+		      GDK_RGB_DITHER_NONE, 0, 0);
+    }
     return;
   }
 
@@ -970,8 +723,9 @@ static void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
      faster to just draw every second pixel black in the first place. */
   if (fog) {
     gdk_gc_set_clip_origin(fill_tile_gc, canvas_x, canvas_y);
-    gdk_gc_set_clip_mask(fill_tile_gc, ssprite->mask);
-    gdk_gc_set_foreground(fill_tile_gc, colors_standard[COLOR_STD_BLACK]);
+    gdk_gc_set_clip_mask(fill_tile_gc, sprite_get_mask(ssprite));
+    gdk_gc_set_foreground(fill_tile_gc,
+			  &get_color(tileset, COLOR_MAPVIEW_UNKNOWN)->color);
     gdk_gc_set_ts_origin(fill_tile_gc, canvas_x, canvas_y);
     gdk_gc_set_stipple(fill_tile_gc, black50);
 
@@ -991,43 +745,7 @@ void put_cross_overlay_tile(struct tile *ptile)
   if (tile_to_canvas_pos(&canvas_x, &canvas_y, ptile)) {
     pixmap_put_overlay_tile(map_canvas->window,
 			    canvas_x, canvas_y,
-			    sprites.user.attention);
-  }
-}
-
-/****************************************************************************
-  Draw a single tile of the citymap onto the mapview.  The tile is drawn
-  as the given color with the given worker on it.  The exact method of
-  drawing is left up to the GUI.
-****************************************************************************/
-void put_city_worker(struct canvas *pcanvas,
-		     enum color_std color, enum city_tile_type worker,
-		     int canvas_x, int canvas_y)
-{
-  if (pcanvas->type == CANVAS_PIXMAP) {
-    if (worker == C_TILE_EMPTY) {
-      gdk_gc_set_stipple(fill_tile_gc, gray25);
-    } else if (worker == C_TILE_WORKER) {
-      gdk_gc_set_stipple(fill_tile_gc, gray50);
-    } else {
-      return;
-    }
-
-    gdk_gc_set_ts_origin(fill_tile_gc, canvas_x, canvas_y);
-    gdk_gc_set_foreground(fill_tile_gc, colors_standard[color]);
-
-    if (is_isometric) {
-      gdk_gc_set_clip_origin(fill_tile_gc, canvas_x, canvas_y);
-      gdk_gc_set_clip_mask(fill_tile_gc, sprites.black_tile->mask);
-    }
-
-    gdk_draw_rectangle(pcanvas->v.pixmap, fill_tile_gc, TRUE,
-		       canvas_x, canvas_y,
-		       NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-
-    if (is_isometric) {
-      gdk_gc_set_clip_mask(fill_tile_gc, NULL);
-    }
+			    get_attention_crosshair_sprite(tileset));
   }
 }
 
@@ -1098,7 +816,8 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
 {
   GdkPoint points[5];
 
-  gdk_gc_set_foreground(civ_gc, colors_standard[COLOR_STD_YELLOW]);
+  gdk_gc_set_foreground(civ_gc,
+			&get_color(tileset, COLOR_MAPVIEW_SELECTION)->color);
 
   /* gdk_draw_rectangle() must start top-left.. */
   points[0].x = canvas_x;
@@ -1116,8 +835,6 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
   points[4].x = canvas_x;
   points[4].y = canvas_y;
   gdk_draw_lines(map_canvas->window, civ_gc, points, ARRAY_SIZE(points));
-
-  rectangle_active = TRUE;
 }
 
 /**************************************************************************
@@ -1128,4 +845,10 @@ void tileset_changed(void)
   reset_city_dialogs();
   reset_unit_table();
   blank_max_unit_size();
+
+  /* keep the icon of the executable on Windows (see PR#36491) */
+#ifndef WIN32_NATIVE
+  gtk_window_set_icon(GTK_WINDOW(toplevel),
+		sprite_get_pixbuf(get_icon_sprite(tileset, ICON_FREECIV)));
+#endif
 }
