@@ -51,6 +51,7 @@
 #include "optiondlg.h"
 #include "options.h"
 #include "packhand_gen.h"
+#include "servers.h"
 #include "spaceshipdlg.h"
 #include "tilespec.h"
 
@@ -58,12 +59,14 @@
 #include "gui_main.h"
 
 static enum {
-  LOGIN_TYPE, 
-  NEW_PASSWORD_TYPE, 
+  LOGIN_TYPE,
+  NEW_PASSWORD_TYPE,
   VERIFY_PASSWORD_TYPE,
   ENTER_PASSWORD_TYPE
 } dialog_config;
 
+extern void popup_settable_options_dialog(void);
+
 static HWND connect_dlg;
 static HWND start_dlg;
 static HWND newgame_dlg;
@@ -77,7 +80,6 @@ static HWND networkdlg_name;
 static HWND server_listview;
 static HWND lan_listview;
 
-static int autoconnect_timer_id;
 struct t_server_button {
   HWND button;
   char *button_string;
@@ -89,6 +91,7 @@ enum new_game_dlg_ids {
   ID_NEWGAMEDLG_AISKILL,
   ID_NEWGAMEDLG_AIFILL,
   ID_NEWGAMEDLG_OPTIONS,
+  ID_NEWGAMEDLG_NATIONS,
   ID_OK=IDOK,
   ID_CANCEL=IDCANCEL
 };
@@ -101,7 +104,7 @@ static int get_lanservers(HWND list);
 
 static int num_lanservers_timer = 0;
 
-extern void socket_timer(void);
+struct server_scan *lan = NULL, *meta = NULL;
 
 /*************************************************************************
  configure the dialog depending on what type of authentication request the
@@ -179,12 +182,21 @@ void handle_game_load(struct packet_game_load *packet)
   ShowWindow(start_dlg, SW_HIDE);
   ShowWindow(players_dlg, SW_SHOWNORMAL);
 
-  game.nplayers = packet->nplayers;
+  game.info.nplayers = packet->nplayers;
   ListView_DeleteAllItems(players_listview);
 
   for (i = 0; i < packet->nplayers; i++) {
+    const char *nation_name;
+    struct nation_type *pnation = nation_by_number(packet->nations[i]);
+
+    if (pnation == NO_NATION_SELECTED) {
+      nation_name = "";
+    } else {
+      nation_name = nation_adjective_translation(pnation);
+    }
+
     row[0] = packet->name[i];
-    row[1] = packet->nation_name[i];
+    row[1] = (char *)nation_name;
     row[2] = packet->is_alive[i] ? _("Alive") : _("Dead");
     row[3] = packet->is_ai[i] ? _("AI") : _("Human");
     fcwin_listview_add_row(players_listview, 0, 4, row);
@@ -192,14 +204,7 @@ void handle_game_load(struct packet_game_load *packet)
 
   /* if nplayers is zero, we suppose it's a scenario */
   if (packet->load_successful && packet->nplayers == 0) {
-    char message[MAX_LEN_MSG];
-
-    my_snprintf(message, sizeof(message), "/create %s", user_name);
-    send_chat(message);
-    my_snprintf(message, sizeof(message), "/ai %s", user_name);
-    send_chat(message);
-    my_snprintf(message, sizeof(message), "/take %s", user_name);
-    send_chat(message);
+    send_chat("/take -");
 
     /* create a false entry */
     row[0] = user_name;
@@ -340,7 +345,7 @@ static LONG CALLBACK connectdlg_proc(HWND hWnd, UINT message, WPARAM wParam,
   Callback function for network subwindow messages
 **************************************************************************/
 static LONG CALLBACK networkdlg_proc(HWND hWnd, UINT message, WPARAM wParam,
-				     LPARAM lParam)  
+				     LPARAM lParam)
 {
   LPNMHDR nmhdr;
   switch(message)
@@ -392,7 +397,7 @@ static LONG CALLBACK networkdlg_proc(HWND hWnd, UINT message, WPARAM wParam,
     default:
       return DefWindowProc(hWnd, message, wParam, lParam); 
     }
-  return FALSE;  
+  return FALSE;
 }
 
 /**************************************************************************
@@ -476,7 +481,7 @@ static LONG CALLBACK playersdlg_proc(HWND hWnd, UINT message, WPARAM wParam,
 	  }
 	}
 
-	sz_strlcpy(player_name, name);
+	sz_strlcpy(leader_name, name);
 
 	if (nmlv->hdr.code == NM_DBLCLK) {
 	  really_close_connection_dialog();
@@ -494,18 +499,48 @@ static LONG CALLBACK playersdlg_proc(HWND hWnd, UINT message, WPARAM wParam,
   return FALSE;  
 }
 
+/**************************************************************************
+  Callback function for when there's an error in the server scan.
+**************************************************************************/
+static void server_scan_error(struct server_scan *scan,
+			      const char *message)
+{
+  append_output_window(message);
+  freelog(LOG_NORMAL, "%s", message);
+  switch (server_scan_get_type(scan)) {
+  case SERVER_SCAN_LOCAL:
+    server_scan_finish(lan);
+    lan = NULL;
+    break;
+  case SERVER_SCAN_GLOBAL:
+    server_scan_finish(meta);
+    meta = NULL;
+    break;
+  case SERVER_SCAN_LAST:
+    break;
+  }
+}
+
 /**************************************************************************
  This function updates the list of LAN servers every 100 ms for 5 seconds. 
 **************************************************************************/
 static int get_lanservers(HWND list)
 {
-  struct server_list *server_list = get_lan_server_list();
+  struct server_list *server_list = NULL;
+
+  if (lan) {
+    server_list = server_scan_get_servers(lan);
+    if (!server_list) {
+      return -1;
+    }
+  }
+
   char *row[6];
 
   if (server_list != NULL) {
     ListView_DeleteAllItems(list);
 
-    server_list_iterate(*server_list, pserver) {
+    server_list_iterate(server_list, pserver) {
 
       row[0] = pserver->host;
       row[1] = pserver->port;
@@ -528,7 +563,7 @@ static int get_lanservers(HWND list)
 
   num_lanservers_timer++;
   if (num_lanservers_timer == 50) {
-    finish_lanserver_scan();
+    server_scan_finish(lan);
     num_lanservers_timer = 0;
     return 0;
   }
@@ -538,13 +573,23 @@ static int get_lanservers(HWND list)
 /**************************************************************************
 
  *************************************************************************/
-static int get_meta_list(HWND list, char *errbuf, int n_errbuf)
+static int get_meta_list(HWND list)
 {
   int i;
   char *row[6];
   char  buf[6][64];
 
-  struct server_list *server_list = create_server_list(errbuf, n_errbuf);
+  struct server_list *server_list = NULL;
+
+  if (meta) {
+    for (i = 0; i < 100; i++) {
+      server_list = server_scan_get_servers(meta);
+      if (server_list) {
+        break;
+      }
+      Sleep(100);
+    }
+  }
 
   if (!server_list) {
     return -1;
@@ -555,7 +600,7 @@ static int get_meta_list(HWND list, char *errbuf, int n_errbuf)
   for (i = 0; i < 6; i++)
     row[i] = buf[i];
 
-  server_list_iterate(*server_list, pserver) {
+  server_list_iterate(server_list, pserver) {
     sz_strlcpy(buf[0], pserver->host);
     sz_strlcpy(buf[1], pserver->port);
     sz_strlcpy(buf[2], pserver->version);
@@ -564,7 +609,7 @@ static int get_meta_list(HWND list, char *errbuf, int n_errbuf)
     sz_strlcpy(buf[5], pserver->message);
     fcwin_listview_add_row(list, 0, 6, row);
   } server_list_iterate_end;
-  
+
   ListView_SetColumnWidth(list, 0, LVSCW_AUTOSIZE);
   ListView_SetColumnWidth(list, 1, LVSCW_AUTOSIZE);
   ListView_SetColumnWidth(list, 2, LVSCW_AUTOSIZE);
@@ -572,8 +617,6 @@ static int get_meta_list(HWND list, char *errbuf, int n_errbuf)
   ListView_SetColumnWidth(list, 4, LVSCW_AUTOSIZE);
   ListView_SetColumnWidth(list, 5, LVSCW_AUTOSIZE);
 
-  delete_server_list(server_list);
-
   return 0;
 }
 
@@ -621,14 +664,18 @@ static LONG CALLBACK tabs_page_proc(HWND dlg, UINT message, WPARAM wParam,
       break;
     case WM_COMMAND:
       if (LOWORD(wParam)==IDOK) {
-	char errbuf[128];
 	if (TabCtrl_GetCurSel(tab_ctrl) == 2) {
+          if (!lan) {
+            lan = server_scan_begin(SERVER_SCAN_LOCAL, server_scan_error);
+          }
 	  get_lanservers(lan_listview);
 	} else {
-	  if (get_meta_list(server_listview, errbuf, sizeof(errbuf)) == -1) {
-	    append_output_window(errbuf);
-	  }
-	}
+          if (!meta) {
+            meta = server_scan_begin(SERVER_SCAN_GLOBAL, server_scan_error);
+          }
+          Sleep(500);
+	  get_meta_list(server_listview);
+      	}
       }
       break;
     case WM_NOTIFY:
@@ -649,87 +696,6 @@ static LONG CALLBACK tabs_page_proc(HWND dlg, UINT message, WPARAM wParam,
   return 0;
 }
 
-/**************************************************************************
-  Make an attempt to autoconnect to the server.
-  (server_autoconnect() gets GTK to call this function every so often.)
-**************************************************************************/
-static int try_to_autoconnect()
-{
-  char errbuf[512];
-  static int count = 0;
-
-  count++;
-
-  if (count >= MAX_AUTOCONNECT_ATTEMPTS) {
-    freelog(LOG_FATAL,
-            _("Failed to contact server \"%s\" at port "
-              "%d as \"%s\" after %d attempts"),
-            server_host, server_port, user_name, count);
-    exit(EXIT_FAILURE);
-  }
-
-  switch (try_to_connect(user_name, errbuf, sizeof(errbuf))) {
-  case 0:                       /* Success! */
-    return FALSE;               /* Do not call this
-                                   function again */
-#if 0
-  case ECONNREFUSED:            /* Server not available (yet) */
-    return TRUE;                /* Keep calling this function */
-#endif
-  default:                      /* All other errors are fatal */
-    freelog(LOG_FATAL,
-            _("Error contacting server \"%s\" at port %d "
-              "as \"%s\":\n %s\n"),
-            server_host, server_port, user_name, errbuf);
-    exit(EXIT_FAILURE);     
-  }
-}
-
-/**************************************************************************
-
-**************************************************************************/
-static void CALLBACK autoconnect_timer(HWND hwnd,UINT uMsg,
-				       UINT idEvent,DWORD  dwTime)  
-{
-  printf("Timer\n");
-  if (!try_to_autoconnect())
-    KillTimer(NULL, autoconnect_timer_id);
-}
-
-/**************************************************************************
-  Start trying to autoconnect to civserver.  Calls
-  get_server_address(), then arranges for try_to_autoconnect(), which
-  calls try_to_connect(), to be called roughly every
-  AUTOCONNECT_INTERVAL milliseconds, until success, fatal error or
-  user intervention.  
-**************************************************************************/
-void server_autoconnect()
-{
-  char buf[512];
-
-  my_snprintf(buf, sizeof(buf),
-              _("Auto-connecting to server \"%s\" at port %d "
-                "as \"%s\" every %d.%d second(s) for %d times"),
-              server_host, server_port, user_name,
-              AUTOCONNECT_INTERVAL / 1000,AUTOCONNECT_INTERVAL % 1000, 
-              MAX_AUTOCONNECT_ATTEMPTS);
-  append_output_window(buf);
-  if (get_server_address(server_host, server_port, buf, sizeof(buf)) < 0) {
-    freelog(LOG_FATAL,
-            _("Error contacting server \"%s\" at port %d "
-              "as \"%s\":\n %s\n"),
-            server_host, server_port, user_name, buf);
-    exit(EXIT_FAILURE);
-  }
-  printf("server_autoconnect\n");
-  if (try_to_autoconnect()) {
-    printf("T2\n");
-    autoconnect_timer_id = SetTimer(root_window, 3, AUTOCONNECT_INTERVAL,
-				    autoconnect_timer);
-  }
-
-}
-
 /**************************************************************************
   Handle the saving and loading functions.
 **************************************************************************/
@@ -806,9 +772,6 @@ void handle_save_load(const char *title, bool is_save)
 static void load_game_callback()
 {
   if (is_server_running() || client_start_server()) {
-    while(!can_client_access_hack()) {
-      socket_timer();
-    }
     handle_save_load(_("Load Game"), FALSE);
   }
 }
@@ -843,6 +806,8 @@ static void set_new_game_params(HWND win)
   my_snprintf(aifill_str, sizeof(aifill_str), "/set aifill %d", aifill);
   send_chat(aifill_str);
 
+  really_close_connection_dialog();
+
   send_chat("/start");
 }
 
@@ -865,7 +830,10 @@ static LONG CALLBACK new_game_proc(HWND win, UINT message,
       switch((enum new_game_dlg_ids)LOWORD(wParam))
 	{
 	case ID_NEWGAMEDLG_OPTIONS:
-	  send_report_request(REPORT_SERVER_OPTIONS2);
+	  popup_settable_options_dialog();
+	  break;
+        case ID_NEWGAMEDLG_NATIONS:
+	  popup_races_dialog(game.player_ptr);
 	  break;
 	case ID_CANCEL:
 	  client_kill_server(TRUE);
@@ -918,6 +886,15 @@ static void cdlg_minsize(POINT *size, void *data)
 **************************************************************************/
 static void cdlg_del(void *data)
 {
+  if (lan) {
+    server_scan_finish(lan);
+    lan = NULL;
+  }
+  if (meta) {
+    server_scan_finish(meta);
+    meta = NULL;
+  }
+
   DestroyWindow(network_dlg);
   DestroyWindow(start_dlg);
   DestroyWindow(players_dlg);
@@ -1056,7 +1033,9 @@ void gui_server_connect()
 
   fcwin_box_add_button(hbox, _("Game Options"), ID_NEWGAMEDLG_OPTIONS, 0, 
 		       TRUE, TRUE, 5);
-  fcwin_box_add_button(hbox, _("OK"), ID_OK, 0, TRUE, TRUE, 5);
+  fcwin_box_add_button(hbox, _("Pick Nation"), ID_NEWGAMEDLG_NATIONS, 0, 
+		       TRUE, TRUE, 5);
+  fcwin_box_add_button(hbox, _("Start Game"), ID_OK, 0, TRUE, TRUE, 5);
   fcwin_box_add_button(hbox, _("Cancel"), ID_CANCEL, 0, TRUE, TRUE, 5);
 
   fcwin_box_add_box(newgame_vbox, hbox, TRUE, FALSE, 5);
