@@ -18,7 +18,7 @@
 HWND popup_message_dialog(HWND parent, char *dialogname,
 			  const char *text, ...);
 void destroy_message_dialog(HWND dlg);
-void popup_revolution_dialog(int government);
+void popup_revolution_dialog(struct government *gov);
 void message_dialog_button_set_sensitive(HWND dlg,int id,int state);
 BOOL unitselect_init(HINSTANCE hInstance);
 #endif  /* FC__DIALOGS_H */
