@@ -31,14 +31,15 @@
 #include "shared.h"
 #include "support.h"
 
+#include "canvas.h"
 #include "chatline.h"
 #include "climisc.h"
 #include "clinet.h"
 #include "diptreaty.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
-#include "graphics.h"
 #include "mapview.h"
+#include "sprite.h"
 #include "tilespec.h"
 
 #include "diplodlg.h"
@@ -79,8 +80,8 @@ struct Diplomacy_dialog {
   HWND gold1_label;
   POINT thumb0_pos;
   POINT thumb1_pos;
-  struct Sprite *thumb0;
-  struct Sprite *thumb1;
+  struct sprite *thumb0;
+  struct sprite *thumb1;
   HMENU menu_shown;
 };
 
@@ -92,7 +93,7 @@ struct Diplomacy_dialog {
     TYPED_LIST_ITERATE(struct Diplomacy_dialog, dialoglist, pdialog)
 #define dialog_list_iterate_end  LIST_ITERATE_END
 
-static struct dialog_list dialog_list;
+static struct dialog_list *dialog_list;
 static bool dialog_list_list_has_been_initialised = FALSE;
 
 static struct Diplomacy_dialog *create_diplomacy_dialog(int other_player_id);
@@ -114,8 +115,8 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
     ListBox_AddString(pdialog->list,buf);
   } clause_list_iterate_end;
   
-  pdialog->thumb0=sprites.treaty_thumb[BOOL_VAL(pdialog->treaty.accept0)];
-  pdialog->thumb1=sprites.treaty_thumb[BOOL_VAL(pdialog->treaty.accept1)];
+  pdialog->thumb0 = get_treaty_thumb_sprite(tileset, BOOL_VAL(pdialog->treaty.accept0));
+  pdialog->thumb1 = get_treaty_thumb_sprite(tileset, BOOL_VAL(pdialog->treaty.accept1));
   hdc=GetDC(pdialog->mainwin);
   draw_sprite(pdialog->thumb0,hdc,pdialog->thumb0_pos.x,pdialog->thumb0_pos.y);
   draw_sprite(pdialog->thumb1,hdc,pdialog->thumb1_pos.x,pdialog->thumb1_pos.y);
@@ -128,7 +129,7 @@ static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 static void diplomacy_dialog_close(struct Diplomacy_dialog *pdialog)
 {
   dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
-					    pdialog->treaty.plr1->player_no);
+					    player_number(pdialog->treaty.plr1));
   DestroyWindow(pdialog->mainwin);
   
 }
@@ -171,12 +172,12 @@ static void popup_tech_menu(struct Diplomacy_dialog *pdialog,int plr)
   plr0=plr?pdialog->treaty.plr1:pdialog->treaty.plr0;
   plr1=plr?pdialog->treaty.plr0:pdialog->treaty.plr1;
   menu=CreatePopupMenu();
-  for(i=1, flag=0; i<game.num_tech_types; i++) {
+  for(i=1, flag=0; i<game.control.num_tech_types; i++) {
     if (get_invention(plr0, i) == TECH_KNOWN
         && (get_invention(plr1, i) == TECH_UNKNOWN  
             || get_invention(plr1, i) == TECH_REACHABLE)
         && tech_is_available(plr1, i)) {
-      AppendMenu(menu,MF_STRING,ID_ADVANCES_BASE+i,advances[i].name);
+      AppendMenu(menu,MF_STRING,ID_ADVANCES_BASE+i,advance_name_translation(i));
       iteminfo.dwItemData = plr0->player_no * MAX_NUM_ITEMS * MAX_NUM_ITEMS +
 			    plr1->player_no * MAX_NUM_ITEMS + i;
       iteminfo.fMask = MIIM_DATA;
@@ -206,7 +207,7 @@ static void popup_cities_menu(struct Diplomacy_dialog *pdialog,int plr)
   menu=CreatePopupMenu();
   plr0=plr?pdialog->treaty.plr1:pdialog->treaty.plr0;
   plr1=plr?pdialog->treaty.plr0:pdialog->treaty.plr1;
-  n=city_list_size(&plr0->cities);
+  n=city_list_size(plr0->cities);
   if (n>0) {
     city_list_ptrs = fc_malloc(sizeof(struct city*)*n);
   } else {
@@ -223,9 +224,9 @@ static void popup_cities_menu(struct Diplomacy_dialog *pdialog,int plr)
   qsort(city_list_ptrs, i, sizeof(struct city*), city_name_compare);
   
   for(j=0; j<i; j++) {
-    AppendMenu(menu,MF_STRING,ID_CITIES_BASE+j,city_list_ptrs[j]->name);
+    AppendMenu(menu,MF_STRING,ID_CITIES_BASE+j,city_name(city_list_ptrs[j]));
     iteminfo.dwItemData=city_list_ptrs[j]->id*1024 + 
-      plr0->player_no*32 + plr1->player_no;
+      player_number(plr0)*32 + player_number(plr1);
     iteminfo.fMask = MIIM_DATA;
     iteminfo.cbSize = sizeof(MENUITEMINFO);
     SetMenuItemInfo(menu, ID_CITIES_BASE+j, FALSE, &iteminfo);
@@ -255,11 +256,11 @@ static void handle_gold_entry(struct Diplomacy_dialog *pdialog,int plr)
     if (sscanf(buf, "%d", &amount) == 1 && amount >= 0
 	&& amount <= pgiver->economic.gold) {
        dsend_packet_diplomacy_create_clause_req(&aconnection, 
-					pdialog->treaty.plr1->player_no,
-					pgiver->player_no,
+					player_number(pdialog->treaty.plr1),
+					player_number(pgiver),
 					CLAUSE_GOLD, amount);
      } else {
-       append_output_window(_("Game: Invalid amount of gold specified."));
+       append_output_window(_("Invalid amount of gold specified."));
      }
   }
 }
@@ -273,8 +274,8 @@ static void handle_vision_button(struct Diplomacy_dialog *pdialog,int plr)
   pgiver=plr?pdialog->treaty.plr1:pdialog->treaty.plr0;
 
   dsend_packet_diplomacy_create_clause_req(&aconnection, 
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_VISION,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_VISION,
 					   0);
 }
 
@@ -287,8 +288,8 @@ static void handle_embassy_button(struct Diplomacy_dialog *pdialog, int plr)
   pgiver = plr ? pdialog->treaty.plr1 : pdialog->treaty.plr0;
 
   dsend_packet_diplomacy_create_clause_req(&aconnection, 
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_EMBASSY,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_EMBASSY,
 					   0);
 }
 
@@ -326,8 +327,8 @@ static void give_sea_map(struct Diplomacy_dialog *pdialog, int plr)
   struct player *pgiver;
   pgiver=plr?pdialog->treaty.plr1:pdialog->treaty.plr0;
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_SEAMAP,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_SEAMAP,
 					   0);
 }
 
@@ -339,8 +340,8 @@ static void give_map(struct Diplomacy_dialog *pdialog, int plr)
   struct player *pgiver;
   pgiver=plr?pdialog->treaty.plr1:pdialog->treaty.plr0;
   dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_MAP,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_MAP,
 					   0);
 }
 
@@ -356,7 +357,7 @@ static void handle_cities_menu(struct Diplomacy_dialog *pdialog,int choice)
   choice -= plrno0 * 32;
   plrno1 = choice;
   dsend_packet_diplomacy_create_clause_req(&aconnection, 
-					   pdialog->treaty.plr1->player_no,
+					   player_number(pdialog->treaty.plr1),
 					   plrno0, CLAUSE_CITY, value);
 }
 
@@ -372,7 +373,7 @@ static void handle_advances_menu(struct Diplomacy_dialog *pdialog,int choice)
   int value = choice % MAX_NUM_ITEMS;
 
   dsend_packet_diplomacy_create_clause_req(&aconnection, 
-					   pdialog->treaty.plr1->player_no,
+					   player_number(pdialog->treaty.plr1),
 					   plrno0, CLAUSE_ADVANCE, value);
 }
 
@@ -383,8 +384,8 @@ static void diplomacy_dialog_add_pact_clause(struct Diplomacy_dialog *pdialog,
 					     int type)
 {
   dsend_packet_diplomacy_create_clause_req(&aconnection, 
-					   pdialog->treaty.plr1->player_no,
-					   pdialog->treaty.plr0->player_no,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pdialog->treaty.plr0),
 					   type, 0);
 }
 
@@ -401,8 +402,8 @@ static void handle_erase_button(struct Diplomacy_dialog *pdialog)
   clause_list_iterate(pdialog->treaty.clauses, pclause) {
     if(i == row) {
       dsend_packet_diplomacy_remove_clause_req(&aconnection,
-					       pdialog->treaty.plr1->player_no,
-					       pclause->from->player_no,
+					       player_number(pdialog->treaty.plr1),
+					       player_number(pclause->from),
 					       pclause->type, pclause->value);
       return;
     }
@@ -416,7 +417,7 @@ static void handle_erase_button(struct Diplomacy_dialog *pdialog)
 static void handle_accept_button(struct Diplomacy_dialog *pdialog)
 {
   dsend_packet_diplomacy_accept_treaty_req(&aconnection,
-					   pdialog->treaty.plr1->player_no);
+					   player_number(pdialog->treaty.plr1));
 }
 
 /****************************************************************
@@ -448,7 +449,7 @@ static LONG CALLBACK diplomacy_proc(HWND dlg,UINT message,WPARAM wParam,LPARAM l
   case WM_DESTROY:
     if (pdialog->menu_shown)
       DestroyMenu(pdialog->menu_shown);
-    dialog_list_unlink(&dialog_list, pdialog);
+    dialog_list_unlink(dialog_list, pdialog);
     free(pdialog);
     break;
   case WM_COMMAND:
@@ -560,8 +561,8 @@ static LONG CALLBACK diplomacy_proc(HWND dlg,UINT message,WPARAM wParam,LPARAM l
 *****************************************************************/
 static void thumb_minsize(POINT *minsize, void *data)
 {
-  minsize->x=sprites.treaty_thumb[0]->width;
-  minsize->y=sprites.treaty_thumb[0]->height;
+  minsize->x = get_treaty_thumb_sprite(tileset, FALSE)->width;
+  minsize->y = get_treaty_thumb_sprite(tileset, FALSE)->height;
 }
 
 /****************************************************************
@@ -580,7 +581,7 @@ static void thumb_setsize(RECT *rc, void *data)
 *****************************************************************/
 static struct Diplomacy_dialog *create_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = get_player(other_player_id);
+  struct player *plr0 = game.player_ptr, *plr1 = player_by_number(other_player_id);
 
   char buf[512];
   struct fcwin_box *vbox;
@@ -589,7 +590,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(int other_player_id)
   struct Diplomacy_dialog *pdialog;
 
   pdialog=fc_malloc(sizeof(struct Diplomacy_dialog));  
-  dialog_list_insert(&dialog_list, pdialog);
+  dialog_list_prepend(dialog_list, pdialog);
   pdialog->menu_shown=NULL;
   init_treaty(&pdialog->treaty, plr0, plr1);
 
@@ -602,7 +603,8 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(int other_player_id)
 						pdialog);
   vbox=fcwin_vbox_new(pdialog->mainwin,FALSE);
   my_snprintf(buf, sizeof(buf),
-              _("The %s offerings"), get_nation_name(plr0->nation));
+              _("The %s offerings"),
+              nation_adjective_for_player(plr0));
   fcwin_box_add_static(vbox,buf,0,SS_LEFT,FALSE,FALSE,5);
   fcwin_box_add_button(vbox,_("Maps"),ID_MAP0,0,FALSE,FALSE,5);
   fcwin_box_add_button(vbox,_("Advances"),ID_TECH0,0,FALSE,FALSE,5);
@@ -624,33 +626,37 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(int other_player_id)
 	      _("This Eternal Treaty\n"
 		"marks the results of the diplomatic work between\n"
 		"The %s %s %s\nand\nThe %s %s %s"),
-	      get_nation_name(plr0->nation),
-	      get_ruler_title(plr0->government, plr0->is_male, plr0->nation),
-	      plr0->name,
-	      get_nation_name(plr1->nation),
-	      get_ruler_title(plr1->government, plr1->is_male, plr1->nation),
-	      plr1->name);
+	      nation_adjective_for_player(plr0),
+	      ruler_title_translation(plr0),
+	      player_name(plr0),
+	      nation_adjective_for_player(plr1),
+	      ruler_title_translation(plr1),
+	      player_name(plr1));
   fcwin_box_add_static(vbox,buf,0,SS_CENTER,FALSE,FALSE,5);
   pdialog->list=fcwin_box_add_list(vbox,6,ID_LIST,WS_VSCROLL,TRUE,TRUE,5);
   hbox2=fcwin_hbox_new(pdialog->mainwin,FALSE);
-  my_snprintf(buf, sizeof(buf), _("%s view:"), get_nation_name(plr0->nation));
+
+  my_snprintf(buf, sizeof(buf), _("%s view:"),
+              nation_adjective_for_player(plr0));
   fcwin_box_add_static(hbox2,buf,0,SS_LEFT,FALSE,FALSE,5);
   fcwin_box_add_generic(hbox2,thumb_minsize,thumb_setsize,NULL,
 			&pdialog->thumb0_pos,FALSE,FALSE,5);
   
-  my_snprintf(buf, sizeof(buf), _("%s view:"), get_nation_name(plr1->nation));
+  my_snprintf(buf, sizeof(buf), _("%s view:"),
+              nation_adjective_for_player(plr1));
   fcwin_box_add_static(hbox2,buf,0,SS_LEFT,FALSE,FALSE,5);
   fcwin_box_add_generic(hbox2,thumb_minsize,thumb_setsize,NULL,
 			&pdialog->thumb1_pos,FALSE,FALSE,5);
-  pdialog->thumb0=sprites.treaty_thumb[0];
-  pdialog->thumb1=sprites.treaty_thumb[0];
+  pdialog->thumb0 = get_treaty_thumb_sprite(tileset, FALSE);
+  pdialog->thumb1 = get_treaty_thumb_sprite(tileset, FALSE);
   fcwin_box_add_box(vbox,hbox2,FALSE,FALSE,5);
 
   fcwin_box_add_box(hbox,vbox,TRUE,TRUE,5);
   
   vbox=fcwin_vbox_new(pdialog->mainwin,FALSE);
   my_snprintf(buf, sizeof(buf),
-              _("The %s offerings"), get_nation_name(plr1->nation));
+              _("The %s offerings"),
+              nation_adjective_for_player(plr1));
   fcwin_box_add_static(vbox,buf,0,SS_LEFT,FALSE,FALSE,5);
   fcwin_box_add_button(vbox,_("Maps"),ID_MAP1,0,FALSE,FALSE,5);
   fcwin_box_add_button(vbox,_("Advances"),ID_TECH1,0,FALSE,FALSE,5);
@@ -686,10 +692,10 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(int other_player_id)
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = get_player(other_player_id);
+  struct player *plr0 = game.player_ptr, *plr1 = player_by_number(other_player_id);
 
   if(!dialog_list_list_has_been_initialised) {
-    dialog_list_init(&dialog_list);
+    dialog_list = dialog_list_new();
     dialog_list_list_has_been_initialised = TRUE;
   }
 
@@ -760,7 +766,7 @@ void handle_diplomacy_create_clause(int counterpart, int giver,
     return;
   }
 
-  add_clause(&pdialog->treaty, get_player(giver), type, value);
+  add_clause(&pdialog->treaty, player_by_number(giver), type, value);
   update_diplomacy_dialog(pdialog);
 }
 
@@ -791,7 +797,7 @@ void handle_diplomacy_remove_clause(int counterpart, int giver,
     return;
   }
 
-  remove_clause(&pdialog->treaty, get_player(giver), type, value);
+  remove_clause(&pdialog->treaty, player_by_number(giver), type, value);
   update_diplomacy_dialog(pdialog);
 }
 
@@ -804,7 +810,7 @@ void close_all_diplomacy_dialogs(void)
     return;
   }
 
-  while (dialog_list_size(&dialog_list) > 0) {
-    close_diplomacy_dialog(dialog_list_get(&dialog_list, 0));
+  while (dialog_list_size(dialog_list) > 0) {
+    close_diplomacy_dialog(dialog_list_get(dialog_list, 0));
   }
 }
