@@ -28,8 +28,10 @@
 #include "government.h"
 #include "map.h"
 #include "mem.h"
+#include "movement.h"
 #include "support.h"
 #include "unit.h"
+#include "unitlist.h"
  
 #include "chatline.h"
 #include "cityrep.h"
@@ -67,8 +69,7 @@ enum MenuID {
   IDM_GAME_LOCAL_OPTIONS,
   IDM_GAME_MESSAGE_OPTIONS,
   IDM_GAME_SAVE_SETTINGS,
-  IDM_GAME_SERVER_OPTIONS1,
-  IDM_GAME_SERVER_OPTIONS2,
+  IDM_GAME_SERVER_OPTIONS,
   IDM_GAME_SAVE_GAME,
   IDM_GAME_SAVE_QUICK,
   IDM_GAME_EXPORT_LOG, 
@@ -304,14 +305,13 @@ static struct my_menu main_menu[] = {
   {N_("_Message Options"),			IDM_GAME_MESSAGE_OPTIONS},
   {N_("Sa_ve Settings"),			IDM_GAME_SAVE_SETTINGS},
   {"", IDM_SEPARATOR},
-  {N_("_Initial Server Options"),		IDM_GAME_SERVER_OPTIONS1},
-  {N_("Server O_ptions"),			IDM_GAME_SERVER_OPTIONS2},
+  {N_("Server O_ptions"),			IDM_GAME_SERVER_OPTIONS},
   {"", IDM_SEPARATOR},
   {N_("_Save Game"),				IDM_GAME_SAVE_QUICK},
   {N_("Save Game _As..."),			IDM_GAME_SAVE_GAME},
   {"", IDM_SEPARATOR},
-  {N_("E_xport Log"),				IDM_GAME_EXPORT_LOG},
-  {N_("Clear _Log"),				IDM_GAME_CLEAR_OUTPUT},
+  {N_("_Write Log"),				IDM_GAME_EXPORT_LOG},
+  {N_("_Clear Log"),				IDM_GAME_CLEAR_OUTPUT},
   {"", IDM_SEPARATOR},
   {N_("_Disconnect"),				IDM_GAME_DISCONNECT},
   {N_("_Quit")			"\tCtl+Q",	IDM_GAME_QUIT},
@@ -319,10 +319,10 @@ static struct my_menu main_menu[] = {
 
 
   {N_("Gov_ernment"),				IDM_SUBMENU},
-  {N_("Tax Rates")		"\tShift+T",	IDM_GOVERNMENT_TAX_RATE},
+  {N_("_Tax Rates")		"\tShift+T",	IDM_GOVERNMENT_TAX_RATE},
   { "", IDM_SEPARATOR},
   {N_("_Find City")		"\tCtl+F",	IDM_GOVERNMENT_FIND_CITY},
-  {N_("Work_lists")		"\tCtl+W",	IDM_GOVERNMENT_WORKLISTS},
+  {N_("_Worklists")		"\tCtl+W",	IDM_GOVERNMENT_WORKLISTS},
   { "", IDM_SEPARATOR},
   {N_("_Change Government"),			IDM_SUBMENU},
   {N_("_Revolution"),				IDM_GOVERNMENT_REVOLUTION},
@@ -398,15 +398,17 @@ static struct my_menu main_menu[] = {
   {N_("_Reports"),				IDM_SUBMENU},
   {N_("_Cities")		"\tF1",		IDM_REPORTS_CITIES},
   {N_("_Units")			"\tF2",		IDM_REPORTS_UNITS},
-  {N_("_Players")		"\tF3",		IDM_REPORTS_PLAYERS},
+  /* TRANS: Nations report action */
+  {N_("_Nations")		"\tF3",		IDM_REPORTS_PLAYERS},
   {N_("_Economy")		"\tF5",		IDM_REPORTS_ECONOMY},
-  {N_("_Science")		"\tF6",		IDM_REPORTS_SCIENCE},
+  /* TRANS: Research report action */
+  {N_("_Research")		"\tF6",		IDM_REPORTS_SCIENCE},
   {"", IDM_SEPARATOR},
   {N_("_Wonders of the World")	"\tF7",		IDM_REPORTS_WONDERS},
   {N_("_Top Five Cities")	"\tF8",		IDM_REPORTS_TOP_CITIES},
   {N_("_Messages")		"\tF10",	IDM_REPORTS_MESSAGES},
   {N_("_Demographics")		"\tF11",	IDM_REPORTS_DEMOGRAPHICS},
-  {N_("S_paceship")		"\tF12",	IDM_REPORTS_SPACESHIP},
+  {N_("_Spaceship")		"\tF12",	IDM_REPORTS_SPACESHIP},
   {NULL, 0},
 
 
@@ -572,12 +574,9 @@ void handle_menu(int code)
     case IDM_GAME_SAVE_SETTINGS:
       save_options();
       break;
-    case IDM_GAME_SERVER_OPTIONS1:
-      send_report_request(REPORT_SERVER_OPTIONS1);
+    case IDM_GAME_SERVER_OPTIONS:
+      popup_settable_options_dialog();
       break;   
-    case IDM_GAME_SERVER_OPTIONS2:
-      send_report_request(REPORT_SERVER_OPTIONS2);
-      break;     
     case IDM_GAME_SAVE_GAME:
       handle_save_load(_("Select Location to Save"), TRUE);
       break;
@@ -594,7 +593,7 @@ void handle_menu(int code)
       disconnect_from_server();
       break;
     case IDM_GAME_QUIT:
-      exit(EXIT_SUCCESS);
+      client_exit();
       break;
 
 
@@ -608,7 +607,7 @@ void handle_menu(int code)
       popup_worklists_report(game.player_ptr);
       break;
     case IDM_GOVERNMENT_REVOLUTION:
-      popup_revolution_dialog(-1);
+      popup_revolution_dialog(NULL);
       break;
 
 
@@ -689,25 +688,28 @@ void handle_menu(int code)
 
 
     case IDM_ORDERS_BUILD_CITY:
-      if(get_unit_in_focus()) {
-	struct unit *punit = get_unit_in_focus();
-	/* Enable the button for adding to a city in all cases, so we
-	   get an eventual error message from the server if we try. */
-	if (can_unit_add_or_build_city(punit)) {
-	  key_unit_build_city();
-	} else {
-	  key_unit_build_wonder();
-	}
-      }
+      unit_list_iterate(get_units_in_focus(), punit) {
+        /* FIXME: this can provide different actions for different units...
+         * not good! */
+        /* Enable the button for adding to a city in all cases, so we
+         * get an eventual error message from the server if we try. */
+        if (can_unit_add_or_build_city(punit)) {
+          key_unit_build_city();
+        } else {
+          key_unit_build_wonder();
+        }
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_ROAD:
-      if (get_unit_in_focus()) {
-	if (unit_can_est_traderoute_here(get_unit_in_focus())) {
-	  key_unit_traderoute();
-	} else {
-	  key_unit_road();
-	}
-      }
+      unit_list_iterate(get_units_in_focus(), punit) {
+        /* FIXME: this can provide different actions for different units...
+         * not good! */
+        if (unit_can_est_traderoute_here(punit)) {
+          key_unit_traderoute();
+        } else {
+          key_unit_road();
+        }
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_IRRIGATE:
       key_unit_irrigate();
@@ -719,22 +721,29 @@ void handle_menu(int code)
       key_unit_transform();
       break;
     case IDM_ORDERS_FORTRESS:
-      if (get_unit_in_focus()) {
-	if (can_unit_do_activity(get_unit_in_focus(), ACTIVITY_FORTRESS)) {
-	  key_unit_fortress();
-	} else {
-	  key_unit_fortify();
-	}
-      }
+      unit_list_iterate(get_units_in_focus(), punit) {
+        /* FIXME: this can provide different actions for different units...
+         * not good! */
+        if (can_unit_do_activity(punit, ACTIVITY_FORTRESS)) {
+          key_unit_fortress();
+        } else {
+          key_unit_fortify();
+        }
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_AIRBASE:
       key_unit_airbase();
       break;
     case IDM_ORDERS_POLLUTION:
-      if (can_unit_paradrop(get_unit_in_focus()))
-	key_unit_paradrop();
-      else
-	key_unit_pollution();
+      unit_list_iterate(get_units_in_focus(), punit) {
+        /* FIXME: this can provide different actions for different units...
+         * not good! */
+        if (can_unit_paradrop(punit)) {
+          key_unit_paradrop();
+        } else {
+          key_unit_pollution();
+        }
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_FALLOUT:
       key_unit_fallout();
@@ -749,26 +758,22 @@ void handle_menu(int code)
       key_unit_homecity();
       break;
     case IDM_ORDERS_LOAD:
-      request_unit_load(get_unit_in_focus(), NULL);
+      unit_list_iterate(get_units_in_focus(), punit) {
+        request_unit_load(punit, NULL);
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_UNLOAD:
-      if (get_unit_in_focus()) {
-	struct unit *punit = get_unit_in_focus();
-	if (can_unit_unload(punit, find_unit_by_id(punit->transported_by))
-	 && can_unit_exist_at_tile(punit, punit->tile)) {
-	  request_unit_unload(punit);
-	} else if (get_transporter_occupancy(punit) > 0) {
-	  key_unit_unload_all();
-	}
-      }
+      unit_list_iterate(get_units_in_focus(), punit) {
+        request_unit_unload(punit);
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_WAKEUP_OTHERS:
       key_unit_wakeup_others();
       break;
     case IDM_ORDERS_AUTO_SETTLER:
-      if (get_unit_in_focus()) {
-	request_unit_auto(get_unit_in_focus());
-      }
+      unit_list_iterate(get_units_in_focus(), punit) {
+        request_unit_autosettlers(punit);
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_AUTO_EXPLORE:
       key_unit_auto_explore();
@@ -789,14 +794,14 @@ void handle_menu(int code)
       key_unit_goto();
       break;
     case IDM_ORDERS_GOTO_CITY:
-      if(get_unit_in_focus()) {
-	popup_goto_dialog();
+      if (get_num_units_in_focus() > 0) {
+        popup_goto_dialog();
       }
       break;
     case IDM_ORDERS_RETURN:
-      if (get_unit_in_focus()) {
-	request_unit_return(get_unit_in_focus());
-      }
+      unit_list_iterate(get_units_in_focus(), punit) {
+        request_unit_return(punit);
+      } unit_list_iterate_end;
       break;
     case IDM_ORDERS_DISBAND:
       key_unit_disband();
@@ -817,28 +822,28 @@ void handle_menu(int code)
 
 
     case IDM_REPORTS_CITIES:
-      popup_city_report_dialog(0);
+      popup_city_report_dialog(TRUE);
       break;
     case IDM_REPORTS_UNITS:
-      popup_activeunits_report_dialog(0);
+      popup_activeunits_report_dialog(TRUE);
       break;
     case IDM_REPORTS_PLAYERS:
-      popup_players_dialog();
+      popup_players_dialog(TRUE);
       break;
     case IDM_REPORTS_ECONOMY:
-      popup_economy_report_dialog(0);
+      popup_economy_report_dialog(TRUE);
       break;  
     case IDM_REPORTS_SCIENCE:
-      popup_science_dialog(0);
+      popup_science_dialog(TRUE);
       break;
     case IDM_REPORTS_WONDERS:
       send_report_request(REPORT_WONDERS_OF_THE_WORLD);
-      break;          
+      break;
     case IDM_REPORTS_TOP_CITIES:
       send_report_request(REPORT_TOP_5_CITIES);
-      break;       
+      break;
     case IDM_REPORTS_MESSAGES:
-      popup_meswin_dialog();
+      popup_meswin_dialog(TRUE);
       break;
     case IDM_REPORTS_DEMOGRAPHICS:
       send_report_request(REPORT_DEMOGRAPHIC);
@@ -922,7 +927,8 @@ void handle_menu(int code)
     default:
       if ((enum MenuID)code >= IDM_GOVERNMENT_CHANGE_FIRST
 	   && (enum MenuID)code <= IDM_GOVERNMENT_CHANGE_LAST) {
-	popup_revolution_dialog(code - IDM_GOVERNMENT_CHANGE_FIRST);
+	struct government *gov = government_by_number(code - IDM_GOVERNMENT_CHANGE_FIRST);
+	popup_revolution_dialog(gov);
       }
       break;
     }
@@ -979,6 +985,21 @@ HMENU create_mainmenu(void)
   return my_create_menu(&items);
 }
 
+/****************************************************************************
+  Return the text for the tile, changed by the activity.
+
+  Should only be called for irrigation, mining, or transformation, and
+  only when the activity changes the base terrain type.
+****************************************************************************/
+static const char *get_tile_change_menu_text(struct tile *ptile,
+					     enum unit_activity activity)
+{
+  struct tile newtile = *ptile;
+
+  tile_apply_activity(&newtile, activity);
+  return tile_get_info_text(ptile);
+}
+
 /**************************************************************************
   Update the status and names of all menu items.
 **************************************************************************/
@@ -991,14 +1012,12 @@ update_menus(void)
 
   my_enable_menu(menu, IDM_GAME_SAVE_GAME,
 		 can_client_access_hack()
-		 && get_client_state() >= CLIENT_GAME_RUNNING_STATE);
+		 && C_S_RUNNING <= client_state());
   my_enable_menu(menu, IDM_GAME_SAVE_QUICK,
 		 can_client_access_hack()
-		 && get_client_state() >= CLIENT_GAME_RUNNING_STATE);
-  my_enable_menu(menu, IDM_GAME_SERVER_OPTIONS2,
-		 aconnection.established);
-  my_enable_menu(menu, IDM_GAME_SERVER_OPTIONS1,
-		 get_client_state() >= CLIENT_GAME_RUNNING_STATE);
+		 && C_S_RUNNING <= client_state());
+  my_enable_menu(menu, IDM_GAME_SERVER_OPTIONS,
+		 C_S_RUNNING <= client_state());
   my_enable_menu(menu, IDM_GAME_DISCONNECT,
 		 aconnection.established);
 
@@ -1013,7 +1032,6 @@ update_menus(void)
   } else {
     struct unit *punit;
     HMENU govts;
-    int i;
 
     /* remove previous government entries. */
     for (id = IDM_GOVERNMENT_CHANGE_FIRST; id <= IDM_GOVERNMENT_CHANGE_LAST;
@@ -1025,15 +1043,18 @@ update_menus(void)
     govts = GetSubMenu(GetSubMenu(menu, 1), 5); 
 
     /* add new government entries. */
-    for (i = 0; i < game.government_count; i++) {
-      if (i != game.government_when_anarchy) {
-	AppendMenu(govts, MF_STRING, IDM_GOVERNMENT_CHANGE_FIRST + i,
-		   governments[i].name);
-	my_enable_menu(menu, IDM_GOVERNMENT_CHANGE_FIRST + i, 
-		       can_change_to_government(game.player_ptr, i)
+    id = IDM_GOVERNMENT_CHANGE_FIRST;
+
+    government_iterate(g) {
+      if (g != game.government_when_anarchy) {
+	AppendMenu(govts, MF_STRING, id + g->index,
+		   government_name_translation(g));
+	my_enable_menu(menu, id + g->index,
+		       can_change_to_government(game.player_ptr, g)
 		       && can_client_issue_orders());
       }
-    }
+    } government_iterate_end;
+
     my_enable_menu(menu, IDM_GOVERNMENT_FIND_CITY, TRUE);
 
     /* Enables IDM_VIEW_MENU */
@@ -1047,17 +1068,17 @@ update_menus(void)
     }
 
     if (can_client_issue_orders()) {
-      my_enable_menu(menu, IDM_GOVERNMENT_TAX_RATE, game.rgame.changable_tax);
+      my_enable_menu(menu, IDM_GOVERNMENT_TAX_RATE, game.info.changable_tax);
       my_enable_menu(menu, IDM_GOVERNMENT_WORKLISTS, TRUE);
       my_enable_menu(menu, IDM_GOVERNMENT_REVOLUTION, TRUE);
     }
 
 
-    my_enable_menu(menu, IDM_REPORTS_SPACESHIP, 
+    my_enable_menu(menu, IDM_REPORTS_SPACESHIP,
 		   (game.player_ptr->spaceship.state!=SSHIP_NONE));
 
     my_check_menu(menu, IDM_VIEW_MAP_GRID, draw_map_grid);
-    my_enable_menu(menu, IDM_VIEW_NATIONAL_BORDERS, game.borders > 0);
+    my_enable_menu(menu, IDM_VIEW_NATIONAL_BORDERS, game.info.borders > 0);
     my_check_menu(menu, IDM_VIEW_NATIONAL_BORDERS, draw_borders);
     my_check_menu(menu, IDM_VIEW_CITY_NAMES, draw_city_names);
     my_enable_menu(menu, IDM_VIEW_CITY_GROWTH, draw_city_names);
@@ -1075,7 +1096,7 @@ update_menus(void)
     my_check_menu(menu, IDM_VIEW_CITIES, draw_cities);
     my_check_menu(menu, IDM_VIEW_UNITS, draw_units);
     my_check_menu(menu, IDM_VIEW_FOCUS_UNIT, draw_focus_unit);
-    my_enable_menu(menu, IDM_VIEW_FOCUS_UNIT, !draw_units); 
+    my_enable_menu(menu, IDM_VIEW_FOCUS_UNIT, !draw_units);
     my_check_menu(menu, IDM_VIEW_FOG_OF_WAR, draw_fog_of_war);
 
     /* Remaining part of this function: Update Orders menu */
@@ -1084,19 +1105,18 @@ update_menus(void)
       return;
     }
 
-    if ((punit = get_unit_in_focus())) {
+    if (NULL != (punit = head_of_units_in_focus())) {
       const char *irrfmt = _("Change to %s");
       const char *minfmt = _("Change to %s");
       const char *transfmt = _("Transform to %s");
       char irrtext[128], mintext[128], transtext[128];
       char *roadtext;
-      Terrain_type_id  ttype;
-      struct tile_type *tinfo;
+      struct terrain *pterrain;
 
       sz_strlcpy(irrtext, N_("Build Irrigation") "\tI");
       sz_strlcpy(mintext, N_("Build Mine") "\tM");
       sz_strlcpy(transtext, N_("Transform Terrain") "\tO");
-      
+
       /* Since the entire menu is disabled by default, enable the
 	 items with no checks. */
       my_enable_menu(menu, IDM_ORDERS_PATROL, TRUE);
@@ -1135,21 +1155,21 @@ update_menus(void)
       my_enable_menu(menu, IDM_ORDERS_PILLAGE,
 		     can_unit_do_activity(punit, ACTIVITY_PILLAGE));
       my_enable_menu(menu, IDM_ORDERS_DISBAND,
-		     !unit_flag(punit, F_UNDISBANDABLE));
+		     !unit_has_type_flag(punit, F_UNDISBANDABLE));
       my_enable_menu(menu, IDM_ORDERS_HOMECITY,
 		     can_unit_change_homecity(punit));
       my_enable_menu(menu, IDM_ORDERS_LOAD,
 	can_unit_load(punit, find_transporter_for_unit(punit,
 						       punit->tile)));
       my_enable_menu(menu, IDM_ORDERS_UNLOAD,
-	(can_unit_unload(punit, find_unit_by_id(punit->transported_by))
+	(can_unit_unload(punit, game_find_unit_by_number(punit->transported_by))
 	 && can_unit_exist_at_tile(punit, punit->tile)) 
 	|| get_transporter_occupancy(punit) > 0);
       my_enable_menu(menu, IDM_ORDERS_WAKEUP_OTHERS,
 		     is_unit_activity_on_tile(ACTIVITY_SENTRY,
 					      punit->tile));
       my_enable_menu(menu, IDM_ORDERS_AUTO_SETTLER,
-		     can_unit_do_auto(punit));
+		     can_unit_do_autosettlers(punit));
       my_enable_menu(menu, IDM_ORDERS_AUTO_EXPLORE,
 		     can_unit_do_activity(punit, ACTIVITY_EXPLORE));
       my_enable_menu(menu, IDM_ORDERS_CONNECT_ROAD,
@@ -1165,12 +1185,12 @@ update_menus(void)
 		     && diplomat_can_do_action(punit, DIPLOMAT_ANY_ACTION,
 					       punit->tile));
       my_enable_menu(menu, IDM_ORDERS_NUKE,
-		     unit_flag(punit, F_NUCLEAR));
-      if (unit_flag(punit, F_HELP_WONDER)) {
+		     unit_has_type_flag(punit, F_NUCLEAR));
+      if (unit_has_type_flag(punit, F_HELP_WONDER)) {
 	my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Help Build Wonder")
 		       "\tB");
-      } else if (unit_flag(punit, F_CITIES)) {
-	if (map_get_city(punit->tile)) {
+      } else if (unit_has_type_flag(punit, F_CITIES)) {
+	if (tile_get_city(punit->tile)) {
 	  my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Add to City")
 			 "\tB");
 	} else {
@@ -1181,10 +1201,10 @@ update_menus(void)
 	my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Build City") "\tB");
       }
  
-      if (unit_flag(punit, F_TRADE_ROUTE)) {
+      if (unit_has_type_flag(punit, F_TRADE_ROUTE)) {
 	my_rename_menu(menu, IDM_ORDERS_ROAD, N_("Make Trade Route") "\tR");
-      } else if (unit_flag(punit, F_SETTLERS)) {
-	if (map_has_special(punit->tile, S_ROAD)) {
+      } else if (unit_has_type_flag(punit, F_SETTLERS)) {
+	if (tile_has_special(punit->tile, S_ROAD)) {
 	  roadtext = N_("Build Railroad") "\tR";
 	  road_activity = ACTIVITY_RAILROAD;  
 	} else {
@@ -1196,29 +1216,30 @@ update_menus(void)
 	my_rename_menu(menu, IDM_ORDERS_ROAD, N_("Build Road") "\tR");
       }
 
-      ttype = punit->tile->terrain;
-      tinfo = get_tile_type(ttype);
-      if (tinfo->irrigation_result != T_NONE
-	  && tinfo->irrigation_result != ttype) {
+      pterrain = punit->tile->terrain;
+      if (pterrain->irrigation_result != T_NONE
+	  && pterrain->irrigation_result != pterrain) {
 	my_snprintf(irrtext, sizeof(irrtext), irrfmt,
-		    (get_tile_type(tinfo->irrigation_result))->terrain_name);
-	sz_strlcat(irrtext, "\tI");
-      } else if (map_has_special(punit->tile, S_IRRIGATION)
+		    get_tile_change_menu_text(punit->tile,
+					      ACTIVITY_IRRIGATE));
+ 	sz_strlcat(irrtext, "\tI");
+      } else if (tile_has_special(punit->tile, S_IRRIGATION)
 		 && player_knows_techs_with_flag(game.player_ptr,
 						 TF_FARMLAND)) {
 	sz_strlcpy(irrtext, N_("Build Farmland") "\tI");
       }
-      if (tinfo->mining_result != T_NONE
-	  && tinfo->mining_result != ttype) {
+      if (pterrain->mining_result != T_NONE
+	  && pterrain->mining_result != pterrain) {
 	my_snprintf(mintext, sizeof(mintext), minfmt,
-		    (get_tile_type(tinfo->mining_result))->terrain_name);
-	sz_strlcat(mintext, "\tM");
+		    get_tile_change_menu_text(punit->tile, ACTIVITY_MINE));
+ 	sz_strlcat(mintext, "\tM");
       }
-      if (tinfo->transform_result != T_NONE
-	  && tinfo->transform_result != ttype) {
+      if (pterrain->transform_result != T_NONE
+	  && pterrain->transform_result != pterrain) {
 	my_snprintf(transtext, sizeof(transtext), transfmt,
-		    (get_tile_type(tinfo->transform_result))->terrain_name);
-	sz_strlcat(transtext, "\tO");
+		    get_tile_change_menu_text(punit->tile,
+					      ACTIVITY_TRANSFORM));
+ 	sz_strlcat(transtext, "\tO");
       }
 
       my_rename_menu(menu, IDM_ORDERS_IRRIGATE, irrtext);
@@ -1232,14 +1253,14 @@ update_menus(void)
 		       "\tF");
       }
 
-      if (unit_flag(punit, F_PARATROOPERS)) {
+      if (unit_has_type_flag(punit, F_PARATROOPERS)) {
 	my_rename_menu(menu, IDM_ORDERS_POLLUTION, N_("Paradrop") "\tP");
       } else {
 	my_rename_menu(menu, IDM_ORDERS_POLLUTION, N_("Clean Pollution")
 		       "\tP");
       }
 
-      if (!unit_flag(punit, F_SETTLERS)) {
+      if (!unit_has_type_flag(punit, F_SETTLERS)) {
 	my_rename_menu(menu, IDM_ORDERS_AUTO_SETTLER, N_("Auto Attack")
 		       "\tA");
       } else {
