@@ -55,11 +55,12 @@ static Widget players_meet_command;
 static Widget players_war_command;
 static Widget players_vision_command;
 static Widget players_sship_command;
+static bool players_dialog_shell_is_raised;
 
 static int list_index_to_player_index[MAX_NUM_PLAYERS];
 
 
-static void create_players_dialog(void);
+static void create_players_dialog(bool raise);
 static void players_close_callback(Widget w, XtPointer client_data, 
 				   XtPointer call_data);
 static void players_meet_callback(Widget w, XtPointer client_data, 
@@ -81,10 +82,16 @@ static void players_sship_callback(Widget w, XtPointer client_data,
 /****************************************************************
 popup the dialog somewhat inside the main-window 
 *****************************************************************/
-void popup_players_dialog(void)
+void popup_players_dialog(bool raise)
 {
+  players_dialog_shell_is_raised = raise;
+
   if(!players_dialog_shell)
-    create_players_dialog();
+    create_players_dialog(raise);
+
+  if (raise) {
+    XtSetSensitive(main_form, FALSE);
+  }
 
   xaw_set_relative_position(toplevel, players_dialog_shell, 5, 25);
   XtPopup(players_dialog_shell, XtGrabNone);
@@ -96,6 +103,9 @@ void popup_players_dialog(void)
 void popdown_players_dialog(void)
 {
   if (players_dialog_shell) {
+    if (players_dialog_shell_is_raised) {
+      XtSetSensitive(main_form, TRUE);
+    }
     XtDestroyWidget(players_dialog_shell);
     players_dialog_shell = 0;
   }
@@ -104,11 +114,12 @@ void popdown_players_dialog(void)
 /****************************************************************
 ...
 *****************************************************************/
-void create_players_dialog(void)
+void create_players_dialog(bool raise)
 {
   players_dialog_shell =
     I_IN(I_T(XtCreatePopupShell("playerspopup", 
-				topLevelShellWidgetClass,
+				raise ? transientShellWidgetClass
+				: topLevelShellWidgetClass,
 				toplevel, NULL, 0)));
 
   players_form = XtVaCreateManagedWidget("playersform", 
@@ -192,38 +203,38 @@ void create_players_dialog(void)
 
 
 /**************************************************************************
-...
+  Updates the player list dialog.
 **************************************************************************/
 void update_players_dialog(void)
 {
    if(players_dialog_shell && !is_plrdlg_frozen()) {
-    int i,j;
+    int j = 0;
     Dimension width;
     static char *namelist_ptrs[MAX_NUM_PLAYERS];
     static char namelist_text[MAX_NUM_PLAYERS][256];
     const struct player_diplstate *pds;
 
-    for(i=0,j=0; i<game.nplayers; i++) {
-      char idlebuf[32], statebuf[32], namebuf[32], dsbuf[32], repbuf[32];
+    players_iterate(pplayer) {
+      char idlebuf[32], statebuf[32], namebuf[32], dsbuf[32];
       
       /* skip barbarians */
-      if(is_barbarian(&game.players[i]))
+      if(is_barbarian(pplayer))
         continue;
 
       /* text for idleness */
-      if(game.players[i].nturns_idle>3) {
+      if(pplayer->nturns_idle>3) {
 	my_snprintf(idlebuf, sizeof(idlebuf),
 		    PL_("(idle %d turn)", "(idle %d turns)",
-			game.players[i].nturns_idle - 1),
-		    game.players[i].nturns_idle - 1);
+			pplayer->nturns_idle - 1),
+		    pplayer->nturns_idle - 1);
       } else {
 	idlebuf[0]='\0';
       }
 
       /* text for state */
-      if(game.players[i].is_alive) {
-	if(game.players[i].is_connected) {
-	  if(game.players[i].turn_done)
+      if(pplayer->is_alive) {
+	if(pplayer->is_connected) {
+	  if(pplayer->phase_done)
 	    sz_strlcpy(statebuf, _("done"));
 	  else
 	    sz_strlcpy(statebuf, _("moving"));
@@ -235,17 +246,17 @@ void update_players_dialog(void)
 	sz_strlcpy(statebuf, _("R.I.P"));
 
       /* text for name, plus AI marker */       
-      if(game.players[i].ai.control)
-	my_snprintf(namebuf, sizeof(namebuf), "*%-15s",game.players[i].name);
+      if(pplayer->ai.control)
+	my_snprintf(namebuf, sizeof(namebuf), "*%-15s",player_name(pplayer));
       else
-        my_snprintf(namebuf, sizeof(namebuf), "%-16s",game.players[i].name);
+        my_snprintf(namebuf, sizeof(namebuf), "%-16s",player_name(pplayer));
       namebuf[16] = '\0';
 
       /* text for diplstate type and turns -- not applicable if this is me */
-      if (i == game.player_idx) {
+      if ((player_number(pplayer) == game.info.player_idx) || !game.player_ptr) {
 	strcpy(dsbuf, "-");
       } else {
-	pds = pplayer_get_diplstate(game.player_ptr, get_player(i));
+	pds = pplayer_get_diplstate(game.player_ptr, pplayer);
 	if (pds->type == DS_CEASEFIRE) {
 	  my_snprintf(dsbuf, sizeof(dsbuf), "%s (%d)",
 		      diplstate_text(pds->type), pds->turns_left);
@@ -255,27 +266,22 @@ void update_players_dialog(void)
 	}
       }
 
-      /* text for reputation */
-      my_snprintf(repbuf, sizeof(repbuf),
-		  reputation_text(game.players[i].reputation));
-
       /* assemble the whole lot */
       my_snprintf(namelist_text[j], sizeof(namelist_text[j]),
-	      "%-16s %-12s %-8s %-15s %-8s %-13s %-6s   %-15s%s", 
+	      "%-16s %-12s %-8s %-15s %-8s %-6s   %-15s%s", 
 	      namebuf,
-	      get_nation_name(game.players[i].nation), 
-	      get_embassy_status(game.player_ptr, &game.players[i]),
+	      nation_adjective_for_player(pplayer), 
+	      get_embassy_status(game.player_ptr, pplayer),
 	      dsbuf,
-	      get_vision_status(game.player_ptr, &game.players[i]),
-	      repbuf,
+	      get_vision_status(game.player_ptr, pplayer),
 	      statebuf,
-	      player_addr_hack(&game.players[i]),  /* Fixme for multi-conn */
+	      player_addr_hack(pplayer),  /* Fixme for multi-conn */
 	      idlebuf);
 
       namelist_ptrs[j]=namelist_text[j];
-      list_index_to_player_index[j] = i;
+      list_index_to_player_index[j] = player_number(pplayer);
       j++;
-    }
+    } players_iterate_end;
     
     XawListChange(players_list, namelist_ptrs, j, 0, True);
 
@@ -293,7 +299,7 @@ void players_list_callback(Widget w, XtPointer client_data,
 {
   XawListReturnStruct *ret;
 
-  ret=XawListShowCurrent(players_list);
+  ret = XawListShowCurrent(players_list);
 
   XtSetSensitive(players_meet_command, FALSE);
   XtSetSensitive(players_int_command, FALSE);
@@ -352,7 +358,7 @@ void players_meet_callback(Widget w, XtPointer client_data,
       dsend_packet_diplomacy_init_meeting_req(&aconnection, player_index);
     }
     else {
-      append_output_window(_("Game: You need an embassy to establish"
+      append_output_window(_("You need an embassy to establish"
 			     " a diplomatic meeting."));
     }
   }
