@@ -23,37 +23,51 @@
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
+#include <X11/Xaw/AsciiText.h>  
+#include <X11/Xaw/Command.h>
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Label.h>
-#include <X11/Xaw/SimpleMenu.h> 
-#include <X11/Xaw/Command.h>
 #include <X11/Xaw/List.h>
 #include <X11/Xaw/MenuButton.h>
+#include <X11/Xaw/SimpleMenu.h> 
 #include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/Toggle.h>
+#include <X11/Xaw/Viewport.h>
 
-#include "city.h"
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
-#include "packets.h"
 #include "shared.h"
 #include "support.h"
 
-#include "cityrep.h"
+#include "city.h"
+#include "game.h"
+#include "government.h"
+#include "packets.h"
+#include "unitlist.h"
+
+#include "chatline_common.h" /* for send_chat() */
 #include "climisc.h"
 #include "clinet.h"
+#include "text.h"
+
+#include "cityrep.h"
 #include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "optiondlg.h"
-#include "text.h"
 
-#include "repodlgs.h"
 #include "repodlgs_common.h"
+#include "repodlgs.h"
 
 /******************************************************************/
+static Widget science_dialog_shell;
+static Widget science_label;
+static Widget science_current_label, science_goal_label;
+static Widget science_change_menu_button, science_goal_menu_button;
+static Widget science_list, science_help_toggle;
+static Widget science_help_note;
+static Widget popupmenu, goalmenu;
+static int science_dialog_shell_is_modal;
 
 void create_science_dialog(bool make_modal);
 void science_close_callback(Widget w, XtPointer client_data, 
@@ -67,16 +81,13 @@ void science_goal_callback(Widget w, XtPointer client_data,
 
 
 /******************************************************************/
-static Widget science_dialog_shell;
-static Widget science_label;
-static Widget science_current_label, science_goal_label;
-static Widget science_change_menu_button, science_goal_menu_button;
-static Widget science_list, science_help_toggle;
-static Widget science_help_note;
-static int science_dialog_shell_is_modal;
-static Widget popupmenu, goalmenu;
+static Widget economy_dialog_shell;
+static Widget economy_label, economy_label2;
+static Widget economy_list, economy_list_label;
+static Widget sellall_command, sellobsolete_command;
+static int economy_dialog_shell_is_modal;
+static int economy_improvement_type[B_LAST];
 
-/******************************************************************/
 void create_economy_report_dialog(bool make_modal);
 void economy_close_callback(Widget w, XtPointer client_data, 
 			 XtPointer call_data);
@@ -84,15 +95,15 @@ void economy_selloff_callback(Widget w, XtPointer client_data,
                             XtPointer call_data);
 void economy_list_callback(Widget w, XtPointer client_data,
                          XtPointer call_data);
-static int economy_improvement_type[B_LAST];
-
-static Widget economy_dialog_shell;
-static Widget economy_label, economy_label2;
-static Widget economy_list, economy_list_label;
-static Widget sellall_command, sellobsolete_command;
-static int economy_dialog_shell_is_modal;
 
 /******************************************************************/
+static Widget activeunits_dialog_shell;
+static Widget activeunits_label, activeunits_label2;
+static Widget activeunits_list, activeunits_list_label;
+static Widget upgrade_command;
+static int activeunits_dialog_shell_is_modal;
+static int activeunits_type[U_LAST];
+
 void create_activeunits_report_dialog(bool make_modal);
 void activeunits_close_callback(Widget w, XtPointer client_data, 
 			 XtPointer call_data);
@@ -102,15 +113,24 @@ void activeunits_refresh_callback(Widget w, XtPointer client_data,
 			 XtPointer call_data);
 void activeunits_list_callback(Widget w, XtPointer client_data, 
                            XtPointer call_data);
-static int activeunits_type[U_LAST];
-
-static Widget activeunits_dialog_shell;
-static Widget activeunits_label, activeunits_label2;
-static Widget activeunits_list, activeunits_list_label;
-static Widget upgrade_command;
-
-static int activeunits_dialog_shell_is_modal;
 /******************************************************************/
+static Widget settable_options_dialog_shell;
+static Widget settable_options_form;
+static Widget settable_options_label;
+static Widget settable_options_viewport;
+static Widget settable_options_scrollform;
+static Widget *settable_options_widgets = NULL;
+static Widget settable_options_ok_command;
+static Widget settable_options_cancel_command;
+
+void create_settable_options_dialog(void);
+void update_settable_options_dialog(void);
+void settable_options_toggle_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data);
+void settable_options_ok_callback(Widget w, XtPointer client_data,
+				  XtPointer call_data);
+void settable_options_cancel_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data);
 
 /******************************************************************
 ...
@@ -151,8 +171,7 @@ static const char *get_report_title_plus(const char *report_name,
 ************************ ***************************************/
 void popup_science_dialog(bool make_modal)
 {
-
-  if(!science_dialog_shell) {
+  if (!science_dialog_shell && game.player_ptr) {
     Position x, y;
     Dimension width, height;
     
@@ -171,7 +190,6 @@ void popup_science_dialog(bool make_modal)
     
     XtPopup(science_dialog_shell, XtGrabNone);
   }
-
 }
 
 /****************************************************************
@@ -203,38 +221,38 @@ void create_science_dialog(bool make_modal)
   char current_text[512];
   char goal_text[512];
   const char *report_title;
+  struct player_research* research = get_player_research(game.player_ptr);
   
-  if (game.player_ptr->research.researching == A_UNSET) {
+  if (research->researching == A_UNSET) {
     my_snprintf(current_text, sizeof(current_text),
-		_("Researching %s: %d/%d"),
-		advances[A_NONE].name,
-		game.player_ptr->research.bulbs_researched,
-		total_bulbs_required(game.player_ptr));
+		_("Researching %s: %d/%s"),
+		advance_name_translation(A_NONE),
+		research->bulbs_researched,
+		_("unknown"));
   } else {
     my_snprintf(current_text, sizeof(current_text),
 		_("Researching %s: %d/%d"),
-		get_tech_name(game.player_ptr,
-			      game.player_ptr->research.researching),
-		game.player_ptr->research.bulbs_researched,
+		advance_name_researching(game.player_ptr),
+		research->bulbs_researched,
 		total_bulbs_required(game.player_ptr));
   }
 
-  if (game.player_ptr->ai.tech_goal == A_UNSET) {
+  if (research->tech_goal == A_UNSET) {
     my_snprintf(goal_text, sizeof(goal_text),
 		_("Goal: %s (%d steps)"),
-		advances[A_NONE].name,
+		advance_name_translation(A_NONE),
 		0);
   } else {
     my_snprintf(goal_text, sizeof(goal_text),
 		_("Goal: %s (%d steps)"),
-		advances[game.player_ptr->ai.tech_goal].name,
+		advance_name_translation(research->tech_goal),
 		num_unknown_techs_for_goal(game.player_ptr,
-					   game.player_ptr->ai.tech_goal));
+					   research->tech_goal));
   }
   
-  for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
+  for(i=A_FIRST, j=0; i<game.control.num_tech_types; i++)
     if(get_invention(game.player_ptr, i)==TECH_KNOWN) {
-      tech_list_names_ptrs[j]=advances[i].name;
+      tech_list_names_ptrs[j]=advance_name_translation(i);
       j++;
     }
   tech_list_names_ptrs[j]=0;
@@ -250,9 +268,10 @@ void create_science_dialog(bool make_modal)
   science_form = XtVaCreateManagedWidget("scienceform", 
 					 formWidgetClass,
 					 science_dialog_shell,
-					 NULL);   
+					 NULL);
   my_snprintf(rate_text, sizeof(rate_text), "\ntext not set yet");
-  report_title=get_report_title_plus(_("Science"), rate_text);
+  /* TRANS: Research report title */
+  report_title=get_report_title_plus(_("Research"), rate_text);
   science_label = XtVaCreateManagedWidget("sciencelabel", 
 					  labelWidgetClass, 
 					  science_form,
@@ -317,10 +336,10 @@ void create_science_dialog(bool make_modal)
 				NULL);
 
   
-  for(i=A_FIRST, flag=0; i<game.num_tech_types; i++)
+  for(i=A_FIRST, flag=0; i<game.control.num_tech_types; i++)
     if(get_invention(game.player_ptr, i)==TECH_REACHABLE) {
       Widget entry=
-      XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
+      XtVaCreateManagedWidget(advance_name_translation(i), smeBSBObjectClass, 
 			      popupmenu, NULL);
       XtAddCallback(entry, XtNcallback, science_change_callback, 
 		    (XtPointer) i); 
@@ -330,14 +349,14 @@ void create_science_dialog(bool make_modal)
   if(!flag)
     XtSetSensitive(science_change_menu_button, FALSE);
   
- for(i=A_FIRST, flag=0; i<game.num_tech_types; i++)
+ for(i=A_FIRST, flag=0; i<game.control.num_tech_types; i++)
     if (tech_is_available(game.player_ptr, i)
         && get_invention(game.player_ptr, i) != TECH_KNOWN
         && advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST
         && (num_unknown_techs_for_goal(game.player_ptr, i) < 11
-	    || i == game.player_ptr->ai.tech_goal)) {
+	    || i == research->tech_goal)) {
       Widget entry=
-      XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
+      XtVaCreateManagedWidget(advance_name_translation(i), smeBSBObjectClass, 
 			      goalmenu, NULL);
       XtAddCallback(entry, XtNcallback, science_goal_callback, 
 		    (XtPointer) i); 
@@ -385,7 +404,7 @@ void science_change_callback(Widget w, XtPointer client_data,
 
   XtVaGetValues(science_help_toggle, XtNstate, &b, NULL);
   if (b == TRUE) {
-    popup_help_dialog_typed(advances[to].name, HELP_TECH);
+    popup_help_dialog_typed(advance_name_translation(to), HELP_TECH);
   } else {
     dsend_packet_player_research(&aconnection, to);
   }
@@ -402,7 +421,7 @@ void science_goal_callback(Widget w, XtPointer client_data,
 
   XtVaGetValues(science_help_toggle, XtNstate, &b, NULL);
   if (b == TRUE) {
-    popup_help_dialog_typed(advances[to].name, HELP_TECH);
+    popup_help_dialog_typed(advance_name_translation(to), HELP_TECH);
   } else {
     dsend_packet_player_tech_goal(&aconnection, to);
   }
@@ -460,46 +479,47 @@ void science_dialog_update(void)
     int j, flag;
     size_t i;
     const char *report_title;
+    struct player_research* research = get_player_research(game.player_ptr);
     
-    report_title = get_report_title_plus(_("Science"), science_dialog_text());
+    /* TRANS: Research report title */
+    report_title = get_report_title_plus(_("Research"), science_dialog_text());
     xaw_set_label(science_label, report_title);
     free((void *) report_title);
 
-    if (game.player_ptr->research.researching == A_UNSET) {
+    if (research->researching == A_UNSET) {
       my_snprintf(text, sizeof(text),
-		  _("Researching %s: %d/%d"),
-		  advances[A_NONE].name,
-		  game.player_ptr->research.bulbs_researched,
-		  total_bulbs_required(game.player_ptr));
+		  _("Researching %s: %d/%s"),
+		  advance_name_translation(A_NONE),
+		  research->bulbs_researched,
+		  _("unknown"));
     } else {
       my_snprintf(text, sizeof(text),
 		  _("Researching %s: %d/%d"),
-		  get_tech_name(game.player_ptr,
-				game.player_ptr->research.researching),
-		  game.player_ptr->research.bulbs_researched,
+		  advance_name_researching(game.player_ptr),
+		  research->bulbs_researched,
 		  total_bulbs_required(game.player_ptr));
     }
 
     xaw_set_label(science_current_label, text);
 
-    if (game.player_ptr->ai.tech_goal == A_UNSET) {
+    if (research->tech_goal == A_UNSET) {
       my_snprintf(text, sizeof(text),
 		  _("Goal: %s (%d steps)"),
-		  advances[A_NONE].name,
+		  advance_name_translation(A_NONE),
 		  0);
     } else {
       my_snprintf(text, sizeof(text),
 		  _("Goal: %s (%d steps)"),
-		  advances[game.player_ptr->ai.tech_goal].name,
+		  advance_name_translation(research->tech_goal),
 		  num_unknown_techs_for_goal(game.player_ptr,
-					    game.player_ptr->ai.tech_goal));
+					     research->tech_goal));
     }
 
     xaw_set_label(science_goal_label, text);
 
-    for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
+    for(i=A_FIRST, j=0; i<game.control.num_tech_types; i++)
       if(get_invention(game.player_ptr, i)==TECH_KNOWN) {
-	tech_list_names_ptrs[j]=advances[i].name;
+	tech_list_names_ptrs[j]=advance_name_translation(i);
 	j++;
       }
     tech_list_names_ptrs[j]=0;
@@ -514,10 +534,10 @@ void science_dialog_update(void)
 				   science_change_menu_button, 
 				   NULL);
     
-      for(i=A_FIRST, flag=0; i<game.num_tech_types; i++)
+      for(i=A_FIRST, flag=0; i<game.control.num_tech_types; i++)
       if(get_invention(game.player_ptr, i)==TECH_REACHABLE) {
 	Widget entry=
-	  XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
+	  XtVaCreateManagedWidget(advance_name_translation(i), smeBSBObjectClass, 
 				  popupmenu, NULL);
 	XtAddCallback(entry, XtNcallback, science_change_callback, 
 		      (XtPointer) i); 
@@ -534,14 +554,14 @@ void science_dialog_update(void)
 				  science_goal_menu_button, 
 				  NULL);
     
-    for(i=A_FIRST, flag=0; i<game.num_tech_types; i++)
+    for(i=A_FIRST, flag=0; i<game.control.num_tech_types; i++)
       if (tech_is_available(game.player_ptr, i)
 	  && get_invention(game.player_ptr, i) != TECH_KNOWN
 	  && advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST
 	  && (num_unknown_techs_for_goal(game.player_ptr, i) < 11
-	      || i == game.player_ptr->ai.tech_goal)) {
+	      || i == research->tech_goal)) {
 	Widget entry=
-	  XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
+	  XtVaCreateManagedWidget(advance_name_translation(i), smeBSBObjectClass, 
 				  goalmenu, NULL);
 	XtAddCallback(entry, XtNcallback, science_goal_callback, 
 		      (XtPointer) i); 
@@ -688,7 +708,7 @@ void economy_list_callback(Widget w, XtPointer client_data,
   if (ret->list_index != XAW_LIST_NONE) {
     /* The user has selected an improvement type. */
     int i = economy_improvement_type[ret->list_index];
-    bool is_sellable = (i >= 0 && i < game.num_impr_types && !is_wonder(i));
+    bool is_sellable = (improvement_exists(i) && can_sell_building(i));
 
     XtSetSensitive(sellobsolete_command, is_sellable
 		   && improvement_obsolete(game.player_ptr, i));
@@ -760,7 +780,8 @@ void economy_report_dialog_update(void)
       struct improvement_entry *p = &entries[i];
 
       my_snprintf(economy_list_names[i], sizeof(economy_list_names[i]),
-		  "%-20s%5d%5d%6d", get_improvement_name(p->type),
+		  "%-20s%5d%5d%6d",
+		  improvement_name_translation(p->type),
 		  p->count, p->cost, p->total_cost);
       economy_list_names_ptrs[i] = economy_list_names[i];
       economy_improvement_type[i] = p->type;
@@ -934,8 +955,8 @@ void activeunits_list_callback(Widget w, XtPointer client_data,
 
   may_upgrade =
     ((inx != XAW_LIST_NONE) &&
-     (unit_type_exists (activeunits_type[inx])) &&
-     (can_upgrade_unittype (game.player_ptr, activeunits_type[inx]) != -1));
+     (can_upgrade_unittype(game.player_ptr,
+       utype_by_number(activeunits_type[inx])) != NULL));
 
   XtSetSensitive (upgrade_command, may_upgrade);
 }
@@ -966,27 +987,27 @@ void activeunits_upgrade_callback(Widget w, XtPointer client_data,
 			 XtPointer call_data)
 {
   char buf[512];
-  int ut1,ut2;
+  struct unit_type *punittype1, *punittype2;
 
   XawListReturnStruct *ret;
   ret=XawListShowCurrent(activeunits_list);
 
   if(ret->list_index!=XAW_LIST_NONE) {
-    ut1 = activeunits_type[ret->list_index];
-    if (!(unit_type_exists (ut1))) {
-      return;
-    }
+    punittype1 = utype_by_number(activeunits_type[ret->list_index]);
+    CHECK_UNIT_TYPE(punittype1);
+
     /* puts(unit_types[ut1].name); */
 
-    ut2 = can_upgrade_unittype(game.player_ptr,
-			       activeunits_type[ret->list_index]);
+    punittype2 = can_upgrade_unittype(game.player_ptr,
+		   utype_by_number(activeunits_type[ret->list_index]));
 
     my_snprintf(buf, sizeof(buf),
-	    _("Upgrade as many %s to %s as possible for %d gold each?\n"
-	      "Treasury contains %d gold."),
-	    unit_types[ut1].name, unit_types[ut2].name,
-	    unit_upgrade_price(game.player_ptr, ut1, ut2),
-	    game.player_ptr->economic.gold);
+		_("Upgrade as many %s to %s as possible for %d gold each?\n"
+		  "Treasury contains %d gold."),
+		utype_name_translation(punittype1),
+		utype_name_translation(punittype2),
+		unit_upgrade_price(game.player_ptr, punittype1, punittype2),
+		game.player_ptr->economic.gold);
 
     popup_message_dialog(toplevel, "upgradedialog", buf,
 			 upgrade_callback_yes,
@@ -1043,6 +1064,7 @@ void activeunits_report_dialog_update(void)
     struct repoinfo unittotals;
     const char *report_title;
     char activeunits_total[100];
+    Unit_type_id i;
     
     report_title = get_centered_report_title(_("Units"));
     xaw_set_label(activeunits_label, report_title);
@@ -1050,31 +1072,33 @@ void activeunits_report_dialog_update(void)
 
     memset(unitarray, '\0', sizeof(unitarray));
     unit_list_iterate(game.player_ptr->units, punit) {
-      (unitarray[punit->type].active_count)++;
+      (unitarray[unit_type(punit)->index].active_count)++;
       if (punit->homecity) {
-	unitarray[punit->type].upkeep_shield += punit->upkeep;
-	unitarray[punit->type].upkeep_food += punit->upkeep_food;
+	unitarray[unit_type(punit)->index].upkeep_shield += punit->upkeep[O_SHIELD];
+	unitarray[unit_type(punit)->index].upkeep_food += punit->upkeep[O_FOOD];
+	/* TODO: gold upkeep */
       }
     }
     unit_list_iterate_end;
     city_list_iterate(game.player_ptr->cities,pcity) {
-      if (pcity->is_building_unit &&
-	  (unit_type_exists (pcity->currently_building)))
-	(unitarray[pcity->currently_building].building_count)++;
+      if (pcity->production.is_unit) {
+	(unitarray[pcity->production.value].building_count)++;
+      }
     }
     city_list_iterate_end;
 
     k = 0;
     memset(&unittotals, '\0', sizeof(unittotals));
-    unit_type_iterate(i) {
+    unit_type_iterate(punittype) {
+      i = punittype->index;
       if ((unitarray[i].active_count > 0) || (unitarray[i].building_count > 0)) {
 	my_snprintf
 	  (
 	   activeunits_list_names[k],
 	   sizeof(activeunits_list_names[k]),
 	   "%-27s%c%9d%9d%9d%9d",
-	   unit_name(i),
-	   can_upgrade_unittype(game.player_ptr, i) != -1 ? '*': '-',
+	   utype_name_translation(punittype),
+	   can_upgrade_unittype(game.player_ptr, punittype) != NULL ? '*': '-',
 	   unitarray[i].building_count,
 	   unitarray[i].active_count,
 	   unitarray[i].upkeep_shield,
@@ -1130,8 +1154,8 @@ void popup_endgame_report_dialog(struct packet_endgame_report *packet)
                      "%2d: The %s ruler %s scored %d points\n",
                      packet->score[i]),
                  i + 1,
-                 get_nation_name(get_player(packet->id[i])->nation),
-                 get_player(packet->id[i])->name,
+                 nation_adjective_for_player(player_by_number(packet->id[i])),
+                 player_name(player_by_number(packet->id[i])),
                  packet->score[i]);
   }
   popup_notify_dialog(_("Final Report:"),
@@ -1139,10 +1163,336 @@ void popup_endgame_report_dialog(struct packet_endgame_report *packet)
                       buffer);
 }
 
-/*************************************************************************
-  Server options dialog
-*************************************************************************/
+/****************************************************************************
+			SERVER OPTIONS DIALOG
+****************************************************************************/
+
+/****************************************************************************
+  Show a dialog with the server options.
+****************************************************************************/
 void popup_settable_options_dialog(void)
 {
-  /* PORT ME */
+  if (!settable_options_dialog_shell) {
+    create_settable_options_dialog();
+  }
+
+  update_settable_options_dialog();
+
+  XtPopup(settable_options_dialog_shell, XtGrabNone);
+}
+
+/****************************************************************************
+  Popdown server options dialog.
+****************************************************************************/
+void popdown_settable_options_dialog(void)
+{
+  if (settable_options_dialog_shell) {
+    XtDestroyWidget(settable_options_dialog_shell);
+    settable_options_dialog_shell = NULL;
+  }
+}
+
+/****************************************************************************
+  Server options dialog.
+****************************************************************************/
+void create_settable_options_dialog(void)
+{
+  Widget prev_widget, longest_label = NULL;
+  size_t longest_len = 0;
+  Dimension width;
+  int i;
+
+  settable_options_dialog_shell =
+    I_T(XtCreatePopupShell("settableoptionspopup", transientShellWidgetClass,
+			   toplevel, NULL, 0));
+  settable_options_form =
+    XtVaCreateManagedWidget("settableoptionsform", formWidgetClass,
+			    settable_options_dialog_shell, NULL);
+
+  settable_options_label =
+    I_L(XtVaCreateManagedWidget("settableoptionslabel", labelWidgetClass,
+				settable_options_form, NULL));
+
+  settable_options_viewport =
+    XtVaCreateManagedWidget("settableoptionsviewport", viewportWidgetClass,
+			    settable_options_form, NULL);
+  settable_options_scrollform =
+    XtVaCreateManagedWidget("settableoptionsscrollform", formWidgetClass, 
+			    settable_options_viewport, NULL);
+
+  if (!settable_options_widgets) {
+    settable_options_widgets = fc_calloc(num_settable_options, sizeof(Widget));
+  }
+
+  prev_widget = NULL; /* init the prev-Widget */
+
+  for (i = 0; i < num_settable_options; i++) {
+    char buf[256];
+    size_t len;
+    struct options_settable *o = &settable_options[i];
+
+    my_snprintf(buf, sizeof(buf), "%s: %s", o->name,
+		_(o->short_help));
+    len = strlen(buf);
+
+    /* 
+     * Remember widget so we can reset the vertical position.
+     */
+    settable_options_widgets[i] = prev_widget;
+
+    if (prev_widget) {
+      prev_widget =
+	XtVaCreateManagedWidget("label", labelWidgetClass,
+				settable_options_scrollform,
+				XtNlabel, buf,
+				XtNfromVert, prev_widget,
+				NULL);
+    } else {
+      prev_widget =
+	XtVaCreateManagedWidget("label", labelWidgetClass,
+				settable_options_scrollform,
+				XtNlabel, buf,
+				NULL);
+    }
+
+    if (len > longest_len) {
+      longest_len = len;
+      longest_label = prev_widget;
+    }
+
+    /* 
+     * The addition of a scrollbar screws things up. There must be a
+     * better way to do this.
+     */
+    XtVaGetValues(prev_widget, XtNwidth, &width, NULL);
+    XtVaSetValues(prev_widget, XtNwidth, width + 15, NULL);
+  }
+
+  XtVaGetValues(longest_label, XtNwidth, &width, NULL);
+  XtVaSetValues(settable_options_label, XtNwidth, width + 15, NULL);
+
+  for (i = 0; i < num_settable_options; i++) {
+    struct options_settable *o = &settable_options[i];
+
+    if (setting_class_is_changeable(o->sclass)
+	&& o->is_visible) {
+      switch (o->stype) {
+      case SSET_BOOL:
+	if (settable_options_widgets[i]) {
+	  prev_widget =
+	    XtVaCreateManagedWidget("toggle", toggleWidgetClass,
+				    settable_options_scrollform,
+				    XtNfromHoriz, settable_options_label,
+				    XtNfromVert, settable_options_widgets[i],
+				    NULL);
+	} else {
+	  prev_widget =
+	    XtVaCreateManagedWidget("toggle", toggleWidgetClass,
+				    settable_options_scrollform,
+				    XtNfromHoriz, settable_options_label,
+				    NULL);
+	}
+	XtAddCallback(prev_widget, XtNcallback,
+		      settable_options_toggle_callback, NULL);
+	break;
+      case SSET_INT:
+      case SSET_STRING:
+	if (settable_options_widgets[i]) {
+	  prev_widget =
+	    XtVaCreateManagedWidget("input", asciiTextWidgetClass,
+				    settable_options_scrollform,
+				    XtNfromHoriz, settable_options_label,
+				    XtNfromVert, settable_options_widgets[i],
+				    NULL);
+	} else {
+	  prev_widget =
+	    XtVaCreateManagedWidget("input", asciiTextWidgetClass,
+				    settable_options_scrollform,
+				    XtNfromHoriz, settable_options_label,
+				    NULL);
+	}
+	break;
+      }
+
+    } else {
+      if (settable_options_widgets[i]) {
+	prev_widget =
+	  XtVaCreateManagedWidget("rlabel", labelWidgetClass,
+				  settable_options_scrollform,
+				  XtNfromHoriz, settable_options_label,
+				  XtNfromVert, settable_options_widgets[i],
+				  NULL);
+      } else {
+	prev_widget =
+	  XtVaCreateManagedWidget("rlabel", labelWidgetClass,
+				  settable_options_scrollform,
+				  XtNfromHoriz, settable_options_label,
+				  NULL);
+      }
+    }
+    /* store the final widget */
+    settable_options_widgets[i] = prev_widget;
+  }
+
+  settable_options_ok_command =
+    I_L(XtVaCreateManagedWidget("settableoptionsokcommand",
+				commandWidgetClass,
+				settable_options_form,
+				NULL));
+
+  settable_options_cancel_command =
+    I_L(XtVaCreateManagedWidget("settableoptionscancelcommand",
+				commandWidgetClass,
+				settable_options_form,
+				NULL));
+
+  XtAddCallback(settable_options_ok_command, XtNcallback,
+		settable_options_ok_callback, NULL);
+  XtAddCallback(settable_options_cancel_command, XtNcallback,
+		settable_options_cancel_callback, NULL);
+
+  XtRealizeWidget(settable_options_dialog_shell);
+
+  XSetWMProtocols(display, XtWindow(settable_options_dialog_shell),
+		  &wm_delete_window, 1);
+  XtOverrideTranslations(settable_options_dialog_shell,
+    XtParseTranslationTable("<Message>WM_PROTOCOLS: msg-close-settable-options()"));
+
+  xaw_horiz_center(settable_options_label);
+}
+
+/****************************************************************************
+  Update server options dialog.
+****************************************************************************/
+void update_settable_options_dialog(void)
+{
+  if (settable_options_dialog_shell) {
+    char buf[256];
+    int i;
+
+    for (i = 0; i < num_settable_options; i++) {
+      struct options_settable *o = &settable_options[i];
+
+      if (setting_class_is_changeable(o->sclass)
+	  && o->is_visible) {
+	switch (o->stype) {
+	case SSET_BOOL:
+	  XtVaSetValues(settable_options_widgets[i],
+			XtNstate, o->val ? True : False,
+			XtNlabel,
+			o->val ? _("Yes") : _("No"), NULL);
+	  break;
+	case SSET_INT:
+	  my_snprintf(buf, sizeof(buf), "%d", o->val);
+	  XtVaSetValues(settable_options_widgets[i], XtNstring, buf, NULL);
+	  break;
+	case SSET_STRING:
+	  my_snprintf(buf, sizeof(buf), "%s", o->strval);
+	  XtVaSetValues(settable_options_widgets[i], XtNstring, buf, NULL);
+	  break;
+	}
+      } else {
+	if (o->is_visible) {
+	  switch (o->stype) {
+	  case SSET_BOOL:
+	    my_snprintf(buf, sizeof(buf), "%s",
+	      o->val != 0 ? _("true") : _("false"));
+	    break;
+	  case SSET_INT:
+	    my_snprintf(buf, sizeof(buf), "%d", o->val);
+	    break;
+	  case SSET_STRING:
+	    my_snprintf(buf, sizeof(buf), "%s", o->strval);
+	    break;
+	  }
+	} else {
+	  my_snprintf(buf, sizeof(buf), _("(hidden)"));
+	}
+	XtVaSetValues(settable_options_widgets[i], XtNlabel, buf, NULL);
+      }
+    }
+  }
+}
+
+/****************************************************************************
+  Changes the label of the toggle widget to Yes/No depending on the state of
+  the toggle.
+****************************************************************************/
+void settable_options_toggle_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data)
+{
+  Boolean b;
+
+  XtVaGetValues(w, XtNstate, &b, NULL);
+  XtVaSetValues(w, XtNlabel, b ? _("Yes") : _("No"), NULL);
+}
+
+/****************************************************************************
+  OK button callback.
+****************************************************************************/
+void settable_options_ok_callback(Widget w, XtPointer client_data,
+				  XtPointer call_data)
+{
+  if (settable_options_dialog_shell) {
+    Boolean b, old_b;
+    int val, i;
+    XtPointer dp;
+
+    for (i = 0; i < num_settable_options; i++) {
+      struct options_settable *o = &settable_options[i];
+
+      if (setting_class_is_changeable(o->sclass)
+	  && o->is_visible) {
+	char buffer[MAX_LEN_MSG];
+
+	switch (o->stype) {
+	case SSET_BOOL:
+	  old_b = o->val ? True: False;
+	  XtVaGetValues(settable_options_widgets[i], XtNstate, &b, NULL);
+	  if (b != old_b) {
+	    my_snprintf(buffer, MAX_LEN_MSG, "/set %s %s",
+			o->name, b ? "1" : "0");
+	    send_chat(buffer);
+	  }
+	  break;
+	case SSET_INT:
+	  XtVaGetValues(settable_options_widgets[i], XtNstring, &dp, NULL);
+	  sscanf(dp, "%d", &val);
+	  if (val != o->val) {
+	    my_snprintf(buffer, MAX_LEN_MSG, "/set %s %d",
+			o->name, val);
+	    send_chat(buffer);
+	  }
+	  break;
+	case SSET_STRING:
+	  XtVaGetValues(settable_options_widgets[i], XtNstring, &dp, NULL);
+	  if (strcmp(o->strval, dp)) {
+	    my_snprintf(buffer, MAX_LEN_MSG, "/set %s %s",
+			o->name, (char *)dp);
+	    send_chat(buffer);
+	  }
+	  break;
+	}
+      }
+    }
+
+    popdown_settable_options_dialog();
+  }
+}
+
+/****************************************************************************
+  Cancel button callback.
+****************************************************************************/
+void settable_options_cancel_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data)
+{
+  popdown_settable_options_dialog();
+}
+
+/****************************************************************************
+  Callback for overrided wm_delete_window of server options dialog.
+****************************************************************************/
+void settable_options_msg_close(Widget w)
+{
+  popdown_settable_options_dialog();
 }
