@@ -15,6 +15,8 @@
 
 #include "shared.h"		/* bool type */
 
+#include "sprite_g.h"
+
 bool isometric_view_supported(void);
 bool overhead_view_supported(void);
 
@@ -23,16 +25,4 @@ void load_cursors(void);
 
 void free_intro_radar_sprites(void);
 
-struct Sprite;			/* opaque type, real type is gui-dep */
-
-const char **gfx_fileextensions(void);
-
-struct Sprite *load_gfxfile(const char *filename);
-struct Sprite *crop_sprite(struct Sprite *source,
-			   int x, int y, int width, int height,
-			   struct Sprite *mask,
-			   int mask_offset_x, int mask_offset_y);
-void get_sprite_dimensions(struct Sprite *sprite, int *width, int *height);
-void free_sprite(struct Sprite *s);
-
 #endif  /* FC__GRAPHICS_G_H */
