@@ -18,11 +18,13 @@
 #include <assert.h>
 #include <stdlib.h>		/* qsort */
 
-#include "combat.h"
 #include "fcintl.h"
 #include "log.h"
 #include "support.h"
 
+#include "combat.h"
+#include "unitlist.h"
+
 #include "agents.h"
 #include "chatline_common.h"
 #include "cityrep_g.h"
@@ -36,6 +38,7 @@
 #include "mapctrl_g.h"
 #include "mapview_g.h"
 #include "options.h"
+#include "overview_common.h"
 #include "tilespec.h"
 
 #include "mapctrl_common.h"
@@ -48,14 +51,14 @@ static int rec_w, rec_h;                /* width, heigth in pixels */
 
 bool rbutton_down = FALSE;
 bool rectangle_active = FALSE;
+static bool rectangle_append;
 
 /* This changes the behaviour of left mouse
    button in Area Selection mode. */
 bool tiles_hilited_cities = FALSE;
 
 /* The mapcanvas clipboard */
-static int clipboard = -1;
-static bool clipboard_is_unit;
+struct city_production clipboard = {.value = -1};
 
 /* Goto with drag and drop. */
 bool keyboardless_goto_button_down = FALSE;
@@ -79,16 +82,18 @@ static void define_tiles_within_rectangle(void);
  anchor is not the drawing start point, but is used to calculate
  width, height. Also record the current mapview centering.
 **************************************************************************/
-void anchor_selection_rectangle(int canvas_x, int canvas_y)
+void anchor_selection_rectangle(int canvas_x, int canvas_y,
+				bool append)
 {
   struct tile *ptile = canvas_pos_to_nearest_tile(canvas_x, canvas_y);
 
   tile_to_canvas_pos(&rec_anchor_x, &rec_anchor_y, ptile);
-  rec_anchor_x += NORMAL_TILE_WIDTH / 2;
-  rec_anchor_y += NORMAL_TILE_HEIGHT / 2;
+  rec_anchor_x += tileset_tile_width(tileset) / 2;
+  rec_anchor_y += tileset_tile_height(tileset) / 2;
   /* FIXME: This may be off-by-one. */
   rec_canvas_center_tile = get_center_tile_mapcanvas();
   rec_w = rec_h = 0;
+  rectangle_append = append;
 }
 
 /**************************************************************************
@@ -104,16 +109,16 @@ void anchor_selection_rectangle(int canvas_x, int canvas_y)
 **************************************************************************/
 static void define_tiles_within_rectangle(void)
 {
-  const int W = NORMAL_TILE_WIDTH,   half_W = W / 2;
-  const int H = NORMAL_TILE_HEIGHT,  half_H = H / 2;
+  const int W = tileset_tile_width(tileset),   half_W = W / 2;
+  const int H = tileset_tile_height(tileset),  half_H = H / 2;
   const int segments_x = abs(rec_w / half_W);
   const int segments_y = abs(rec_h / half_H);
 
   /* Iteration direction */
   const int inc_x = (rec_w > 0 ? half_W : -half_W);
   const int inc_y = (rec_h > 0 ? half_H : -half_H);
-
   int x, y, x2, y2, xx, yy;
+  int units = 0;
 
   y = rec_corner_y;
   for (yy = 0; yy <= segments_y; yy++, y += inc_y) {
@@ -143,10 +148,21 @@ static void define_tiles_within_rectangle(void)
 
       /*  Tile passed all tests; process it.
        */
-      if (ptile->city && ptile->city->owner == game.player_idx) {
-        ptile->client.hilite = HILITE_CITY;
+      if (ptile->city && city_owner(ptile->city) == game.player_ptr) {
+	/* FIXME: handle rectangle_append */
+        map_deco[ptile->index].hilite = HILITE_CITY;
         tiles_hilited_cities = TRUE;
       }
+      unit_list_iterate(ptile->units, punit) {
+	if (unit_owner(punit) == game.player_ptr) {
+	  if (units == 0 && !rectangle_append) {
+	    set_unit_focus(punit);
+	  } else {
+	    add_unit_focus(punit);
+	  }
+	  units++;
+	}
+      } unit_list_iterate_end;
     }
   }
 
@@ -161,8 +177,8 @@ static void define_tiles_within_rectangle(void)
 **************************************************************************/
 void update_selection_rectangle(int canvas_x, int canvas_y)
 {
-  const int W = NORMAL_TILE_WIDTH,    half_W = W / 2;
-  const int H = NORMAL_TILE_HEIGHT,   half_H = H / 2;
+  const int W = tileset_tile_width(tileset),    half_W = W / 2;
+  const int H = tileset_tile_height(tileset),   half_H = H / 2;
   static struct tile *rec_tile = NULL;
   int diff_x, diff_y;
   struct tile *center_tile;
@@ -199,7 +215,7 @@ void update_selection_rectangle(int canvas_x, int canvas_y)
    */
   if (diff_x != 0 || diff_y != 0) {
 
-    if (is_isometric) {
+    if (tileset_is_isometric(tileset)) {
       rec_w += (diff_x - diff_y) * half_W;
       rec_h += (diff_x + diff_y) * half_H;
 
@@ -227,17 +243,40 @@ void update_selection_rectangle(int canvas_x, int canvas_y)
   }
 
   /* It is currently drawn only to the screen, not backing store */
+  rectangle_active = TRUE;
   draw_selection_rectangle(canvas_x, canvas_y, rec_w, rec_h);
   rec_corner_x = canvas_x;
   rec_corner_y = canvas_y;
 }
 
+/**************************************************************************
+  Redraws the selection rectangle after a map flush.
+**************************************************************************/
+void redraw_selection_rectangle(void)
+{
+  if (rectangle_active) {
+    draw_selection_rectangle(rec_corner_x, rec_corner_y, rec_w, rec_h);
+  }
+}
+
+/**************************************************************************
+  Redraws the selection rectangle after a map flush.
+**************************************************************************/
+void cancel_selection_rectangle(void)
+{
+  if (rectangle_active) {
+    rectangle_active = FALSE;
+    rbutton_down = FALSE;
+    dirty_rect(rec_corner_x, rec_corner_y, rec_w, rec_h);
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
 bool is_city_hilited(struct city *pcity)
 {
-  return pcity->tile->client.hilite == HILITE_CITY;
+  return map_deco[pcity->tile->index].hilite == HILITE_CITY;
 }
 
 /**************************************************************************
@@ -249,7 +288,7 @@ void cancel_tile_hiliting(void)
     tiles_hilited_cities = FALSE;
 
     whole_map_iterate(ptile) {
-      ptile->client.hilite = HILITE_NONE;
+      map_deco[ptile->index].hilite = HILITE_NONE;
     } whole_map_iterate_end;
 
     update_map_canvas_visible();
@@ -279,14 +318,14 @@ void toggle_tile_hilite(struct tile *ptile)
 {
   struct city *pcity = ptile->city;
 
-  if (ptile->client.hilite == HILITE_CITY) {
-    ptile->client.hilite = HILITE_NONE;
+  if (map_deco[ptile->index].hilite == HILITE_CITY) {
+    map_deco[ptile->index].hilite = HILITE_NONE;
     if (pcity) {
       toggle_city_hilite(pcity, FALSE); /* cityrep.c */
     }
   }
-  else if (pcity && pcity->owner == game.player_idx) {
-    ptile->client.hilite = HILITE_CITY;
+  else if (pcity && city_owner(pcity) == game.player_ptr) {
+    map_deco[ptile->index].hilite = HILITE_CITY;
     tiles_hilited_cities = TRUE;
     toggle_city_hilite(pcity, TRUE);
   }
@@ -294,7 +333,7 @@ void toggle_tile_hilite(struct tile *ptile)
     return;
   }
 
-  refresh_tile_mapcanvas(ptile, TRUE);
+  refresh_tile_mapcanvas(ptile, FALSE, TRUE);
 }
 
 /**************************************************************************
@@ -322,36 +361,38 @@ void key_city_overlay(int canvas_x, int canvas_y)
 **************************************************************************/
 void clipboard_copy_production(struct tile *ptile)
 {
-  char msg[MAX_LEN_MSG];
   struct city *pcity = ptile->city;
 
+  if (!can_client_issue_orders()) {
+    return;
+  }
+
   if (pcity) {
-    if (pcity->owner != game.player_idx)  {
+    if (city_owner(pcity) != game.player_ptr)  {
       return;
     }
-    clipboard = pcity->currently_building;
-    clipboard_is_unit = pcity->is_building_unit;
+    clipboard = pcity->production;
   } else {
     struct unit *punit = find_visible_unit(ptile);
     if (!punit) {
       return;
     }
-    if (!can_player_build_unit_direct(game.player_ptr, punit->type))  {
-      my_snprintf(msg, sizeof(msg),
-      _("Game: You don't know how to build %s!"),
-        unit_types[punit->type].name);
-      append_output_window(msg);
+    if (!can_player_build_unit_direct(game.player_ptr, unit_type(punit)))  {
+      create_event(ptile, E_BAD_COMMAND,
+		   _("You don't know how to build %s!"),
+		   unit_name_translation(punit));
       return;
     }
-    clipboard_is_unit = TRUE;
-    clipboard = punit->type;
+    clipboard.is_unit = TRUE;
+    clipboard.value = unit_type(punit)->index;
   }
   upgrade_canvas_clipboard();
 
-  my_snprintf(msg, sizeof(msg), _("Game: Copy %s to clipboard."),
-    clipboard_is_unit ? unit_types[clipboard].name :
-    get_improvement_name(clipboard));
-  append_output_window(msg);
+  create_event(ptile, E_CITY_PRODUCTION_CHANGED, /* ? */
+	       _("Copy %s to clipboard."),
+	       clipboard.is_unit
+	       ? utype_name_translation(utype_by_number(clipboard.value))
+	       : improvement_name_translation(clipboard.value));
 }
 
 /**************************************************************************
@@ -363,13 +404,12 @@ void clipboard_paste_production(struct city *pcity)
   if (!can_client_issue_orders()) {
     return;
   }
-  if (clipboard == -1) {
-    append_output_window(
-    _("Game: Clipboard is empty."));
+  if (clipboard.value == -1) {
+    create_event(pcity->tile, E_BAD_COMMAND, _("Clipboard is empty."));
     return;
   }
   if (!tiles_hilited_cities) {
-    if (pcity && pcity->owner == game.player_idx) {
+    if (pcity && city_owner(pcity) == game.player_ptr) {
       clipboard_send_production_packet(pcity);
     }
     return;
@@ -390,15 +430,14 @@ void clipboard_paste_production(struct city *pcity)
 **************************************************************************/
 static void clipboard_send_production_packet(struct city *pcity)
 {
-  cid mycid = cid_encode(clipboard_is_unit, clipboard);
-
-  if (mycid == cid_encode_from_city(pcity)
-      || !city_can_build_impr_or_unit(pcity, mycid)) {
+  if ((clipboard.is_unit == pcity->production.is_unit
+       && clipboard.value == pcity->production.value)
+      || !city_can_build_impr_or_unit(pcity, clipboard)) {
     return;
   }
 
-  dsend_packet_city_change(&aconnection, pcity->id, clipboard,
-			   clipboard_is_unit);
+  dsend_packet_city_change(&aconnection, pcity->id, clipboard.value,
+			   clipboard.is_unit);
 }
 
 /**************************************************************************
@@ -407,10 +446,15 @@ static void clipboard_send_production_packet(struct city *pcity)
 **************************************************************************/
 void upgrade_canvas_clipboard(void)
 {
-  if (clipboard_is_unit)  {
-    int u = can_upgrade_unittype(game.player_ptr, clipboard);
-    if (u != -1)  {
-      clipboard = u;
+  if (!can_client_issue_orders()) {
+    return;
+  }
+  if (clipboard.is_unit)  {
+    struct unit_type *u
+      = can_upgrade_unittype(game.player_ptr, utype_by_number(clipboard.value));
+
+    if (u)  {
+      clipboard.value = u->index;
     }
   }
 }
@@ -423,12 +467,9 @@ void release_goto_button(int canvas_x, int canvas_y)
   struct tile *ptile = canvas_pos_to_tile(canvas_x, canvas_y);
 
   if (keyboardless_goto_active && hover_state == HOVER_GOTO && ptile) {
-    struct unit *punit =
-        player_find_unit_by_id(game.player_ptr, hover_unit);
-
     do_unit_goto(ptile);
-    set_hover_state(NULL, HOVER_NONE, ACTIVITY_LAST);
-    update_unit_info_label(punit);
+    set_hover_state(NULL, HOVER_NONE, ACTIVITY_LAST, ORDER_LAST);
+    update_unit_info_label(get_units_in_focus());
   }
   keyboardless_goto_active = FALSE;
   keyboardless_goto_button_down = FALSE;
@@ -443,11 +484,11 @@ void maybe_activate_keyboardless_goto(int canvas_x, int canvas_y)
 {
   struct tile *ptile = canvas_pos_to_tile(canvas_x, canvas_y);
 
-  if (ptile && get_unit_in_focus()
+  if (ptile && get_num_units_in_focus() > 0
       && !same_pos(keyboardless_goto_start_tile, ptile)
       && can_client_issue_orders()) {
     keyboardless_goto_active = TRUE;
-    request_unit_goto();
+    request_unit_goto(ORDER_LAST);
   }
 }
 
@@ -470,14 +511,14 @@ bool get_turn_done_button_state()
 **************************************************************************/
 void scroll_mapview(enum direction8 gui_dir)
 {
-  int gui_x = mapview_canvas.gui_x0, gui_y = mapview_canvas.gui_y0;
+  int gui_x = mapview.gui_x0, gui_y = mapview.gui_y0;
 
   if (!can_client_change_view()) {
     return;
   }
 
-  gui_x += DIR_DX[gui_dir] * mapview_canvas.width / 2;
-  gui_y += DIR_DY[gui_dir] * mapview_canvas.height / 2;
+  gui_x += DIR_DX[gui_dir] * mapview.width / 2;
+  gui_y += DIR_DY[gui_dir] * mapview.height / 2;
   set_mapview_origin(gui_x, gui_y);
 }
 
@@ -578,7 +619,7 @@ void update_turn_done_button_state()
   }
 
   new_state = (can_client_issue_orders()
-	       && !game.player_ptr->turn_done && !agents_busy()
+	       && !game.player_ptr->phase_done && !agents_busy()
 	       && !turn_done_sent);
   if (new_state == turn_done_state) {
     return;
@@ -592,7 +633,9 @@ void update_turn_done_button_state()
 
   if (turn_done_state) {
     if (waiting_for_end_turn
-	|| (game.player_ptr->ai.control && !ai_manual_turn_done)) {
+	|| (game.player_ptr
+	    && game.player_ptr->ai.control
+	    && !ai_manual_turn_done)) {
       send_turn_done();
     } else {
       update_turn_done_button(TRUE);
@@ -605,22 +648,18 @@ void update_turn_done_button_state()
 **************************************************************************/
 void update_line(int canvas_x, int canvas_y)
 {
-  if ((hover_state == HOVER_GOTO
-       || hover_state == HOVER_PATROL
-       || hover_state == HOVER_CONNECT)
-      && draw_goto_line) {
-    struct tile *ptile, *old_tile;
+  struct tile *ptile;
 
+  switch (hover_state) {
+  case HOVER_GOTO:
+  case HOVER_PATROL:
+  case HOVER_CONNECT:
     ptile = canvas_pos_to_tile(canvas_x, canvas_y);
-    if (!ptile) {
-      return;
-    }
 
-    old_tile = get_line_dest();
-    if (!same_pos(old_tile, ptile)) {
-      draw_line(ptile);
-    }
-  }
+    is_valid_goto_draw_line(ptile);
+  default:
+    break;
+  };
 }
 
 /****************************************************************************
@@ -628,45 +667,20 @@ void update_line(int canvas_x, int canvas_y)
 ****************************************************************************/
 void overview_update_line(int overview_x, int overview_y)
 {
-  if ((hover_state == HOVER_GOTO
-       || hover_state == HOVER_PATROL
-       || hover_state == HOVER_CONNECT)
-      && draw_goto_line) {
-    struct tile *ptile, *old_tile;
-    int x, y;
+  struct tile *ptile;
+  int x, y;
 
+  switch (hover_state) {
+  case HOVER_GOTO:
+  case HOVER_PATROL:
+  case HOVER_CONNECT:
     overview_to_map_pos(&x, &y, overview_x, overview_y);
     ptile = map_pos_to_tile(x, y);
 
-    old_tile = get_line_dest();
-    if (!same_pos(ptile, old_tile)) {
-      draw_line(ptile);
-    }
-  }
-}
-
-/**************************************************************************
-  Find the focus unit's chance of success at attacking/defending the
-  given tile.  Return FALSE if the values cannot be determined (e.g., no
-  units on the tile).
-**************************************************************************/
-bool get_chance_to_win(int *att_chance, int *def_chance, struct tile *ptile)
-{
-  struct unit *my_unit, *defender, *attacker;
-
-  if (!(my_unit = get_unit_in_focus())
-      || !(defender = get_defender(my_unit, ptile))
-      || !(attacker = get_attacker(my_unit, ptile))) {
-    return FALSE;
-  }
-
-  /* chance to win when active unit is attacking the selected unit */
-  *att_chance = unit_win_chance(my_unit, defender) * 100;
-
-  /* chance to win when selected unit is attacking the active unit */
-  *def_chance = (1.0 - unit_win_chance(attacker, my_unit)) * 100;
-
-  return TRUE;
+    is_valid_goto_draw_line(ptile);
+  default:
+    break;
+  };
 }
 
 /****************************************************************************
@@ -692,8 +706,8 @@ static int unit_list_compare(const void *a, const void *b)
   } else {
     /* If the transporters aren't the same, put in order by the
      * transporters. */
-    const struct unit *ptrans1 = find_unit_by_id(punit1->transported_by);
-    const struct unit *ptrans2 = find_unit_by_id(punit2->transported_by);
+    const struct unit *ptrans1 = game_find_unit_by_number(punit1->transported_by);
+    const struct unit *ptrans2 = game_find_unit_by_number(punit2->transported_by);
 
     if (!ptrans1) {
       ptrans1 = punit1;
@@ -709,7 +723,7 @@ static int unit_list_compare(const void *a, const void *b)
 /****************************************************************************
   Fill and sort the list of units on the tile.
 ****************************************************************************/
-void fill_tile_unit_list(struct tile *ptile, struct unit **unit_list)
+void fill_tile_unit_list(const struct tile *ptile, struct unit **unit_list)
 {
   int i = 0;
 
