@@ -18,8 +18,12 @@
 #include <assert.h>
 #include <string.h>
 
+#include "log.h"
 #include "mem.h"
 
+#include "game.h"
+#include "movement.h"
+
 #include "pf_tools.h"
 
 
@@ -36,10 +40,8 @@ static void pft_fill_unit_default_parameter(struct pf_parameter *parameter,
 static int seamove(const struct tile *ptile, enum direction8 dir,
                    const struct tile *ptile1, struct pf_parameter *param)
 {
-  /* MOVE_COST_FOR_VALID_SEA_STEP means ships can move between */
-  if (ptile->move_cost[dir] == MOVE_COST_FOR_VALID_SEA_STEP
-      || is_non_allied_unit_tile(ptile1, param->owner)
-      || is_non_allied_city_tile(ptile1, param->owner)) {
+  if (is_ocean(ptile1->terrain) || ptile1->city
+      || is_non_allied_unit_tile(ptile1, param->owner)) {
     return SINGLE_MOVE;
   } else {
     return PF_IMPOSSIBLE_MC;
@@ -65,7 +67,7 @@ static int seamove_no_bombard(const struct tile *ptile, enum direction8 dir,
 			      struct pf_parameter *param)
 {
   /* MOVE_COST_FOR_VALID_SEA_STEP means ships can move between */
-  if (ptile->move_cost[dir] == MOVE_COST_FOR_VALID_SEA_STEP
+  if (map_move_cost_ai(ptile, ptile1) == MOVE_COST_FOR_VALID_SEA_STEP
       && !is_non_allied_city_tile(ptile1, param->owner)) {
     return SINGLE_MOVE;
   } else {
@@ -75,7 +77,7 @@ static int seamove_no_bombard(const struct tile *ptile, enum direction8 dir,
 
 /************************************************************
   A cost function for a sea unit which allows going one step 
-  into the land (shore bombardment).
+  into the land (for shore bombardment and ferries).
   Things to remember: we should prevent going from land to
   anywhere, unless we are leaving a friendly city, in which
   case we can move into the ocean but not into the land.
@@ -122,7 +124,7 @@ static int normal_move_unit(const struct tile *ptile, enum direction8 dir,
 			    const struct tile *ptile1,
 			    struct pf_parameter *param)
 {
-  Terrain_type_id terrain1 = ptile1->terrain;
+  struct terrain *terrain1 = ptile1->terrain;
   int move_cost;
 
   if (is_ocean(terrain1)) {
@@ -137,10 +139,10 @@ static int normal_move_unit(const struct tile *ptile, enum direction8 dir,
             || is_non_allied_city_tile(ptile1, param->owner))) {
       move_cost = PF_IMPOSSIBLE_MC;
     } else {
-      move_cost = get_tile_type(terrain1)->movement_cost * SINGLE_MOVE;
+      move_cost = terrain1->movement_cost * SINGLE_MOVE;
     }
   } else {
-    move_cost = ptile->move_cost[dir];
+    move_cost = map_move_cost_ai(ptile, ptile1);
   }
 
   return move_cost;
@@ -169,8 +171,7 @@ static int land_attack_move(const struct tile *src_tile, enum direction8 dir,
     /* Sea-to-Land. */
     if (!is_non_allied_unit_tile(tgt_tile, param->owner)
         && !is_non_allied_city_tile(tgt_tile, param->owner)) {
-      move_cost 
-        = get_tile_type(tgt_tile->terrain)->movement_cost * SINGLE_MOVE;
+      move_cost = tgt_tile->terrain->movement_cost * SINGLE_MOVE;
     } else if (BV_ISSET(param->unit_flags, F_MARINES)) {
       /* Can attack!! */
       move_cost = SINGLE_MOVE;
@@ -189,7 +190,7 @@ static int land_attack_move(const struct tile *src_tile, enum direction8 dir,
       move_cost = SINGLE_MOVE;
     } else {
       /* Normal move */
-      move_cost = src_tile->move_cost[dir];
+      move_cost = map_move_cost_ai(src_tile, tgt_tile);
     }
   }
 
@@ -217,15 +218,15 @@ static int land_overlap_move(const struct tile *ptile, enum direction8 dir,
 			     const struct tile *ptile1,
 			     struct pf_parameter *param)
 {
-  Terrain_type_id terrain1 = ptile1->terrain;
+  struct terrain *terrain1 = ptile1->terrain;
   int move_cost;
 
   if (is_ocean(terrain1)) {
     move_cost = SINGLE_MOVE;
   } else if (is_ocean(ptile->terrain)) {
-    move_cost = get_tile_type(terrain1)->movement_cost * SINGLE_MOVE;
+    move_cost = terrain1->movement_cost * SINGLE_MOVE;
   } else {
-    move_cost = ptile->move_cost[dir];
+    move_cost = map_move_cost_ai(ptile, ptile1);
   }
 
   return move_cost;
@@ -240,14 +241,14 @@ static int reverse_move_unit(const struct tile *tile0, enum direction8 dir,
 			     const struct tile *ptile,
 			     struct pf_parameter *param)
 {
-  int terrain0 = map_get_terrain(tile0);
-  int terrain1 = ptile->terrain;
+  struct terrain *terrain0 = tile0->terrain;
+  struct terrain *terrain1 = ptile->terrain;
   int move_cost = PF_IMPOSSIBLE_MC;
 
   if (is_ocean(terrain1)) {
     if (ground_unit_transporter_capacity(ptile, param->owner) > 0) {
       /* Landing */
-      move_cost = get_tile_type(terrain0)->movement_cost * SINGLE_MOVE;
+      move_cost = terrain0->movement_cost * SINGLE_MOVE;
     } else {
       /* Nothing to land from */
       move_cost = PF_IMPOSSIBLE_MC;
@@ -287,7 +288,8 @@ static int igter_move_unit(const struct tile *ptile, enum direction8 dir,
       move_cost = MOVE_COST_ROAD;
     }
   } else {
-    move_cost = (ptile->move_cost[dir] != 0 ? MOVE_COST_ROAD : 0);
+    move_cost = (map_move_cost_ai(ptile, ptile1) != 0
+		 ? MOVE_COST_ROAD : 0);
   }
   return move_cost;
 }
@@ -304,14 +306,14 @@ static int reverse_igter_move_unit(const struct tile *tile0,
 {
   int move_cost;
 
-  if (is_ocean(map_get_terrain(ptile))) {
+  if (is_ocean(ptile->terrain)) {
     if (ground_unit_transporter_capacity(ptile, param->owner) > 0) {
       /* Landing */
       move_cost = MOVE_COST_ROAD;
     } else {
       move_cost = PF_IMPOSSIBLE_MC;
     }
-  } else if (is_ocean(map_get_terrain(tile0))) {
+  } else if (is_ocean(tile0->terrain)) {
     /* Boarding */
     move_cost = MOVE_COST_ROAD;
   } else {
@@ -323,6 +325,50 @@ static int reverse_igter_move_unit(const struct tile *tile0,
 #endif
 
 
+/****************************************************************************
+  A cost function for amphibious movement.
+****************************************************************************/
+static int amphibious_move(const struct tile *ptile, enum direction8 dir,
+			   const struct tile *ptile1,
+			   struct pf_parameter *param)
+{
+  const bool ocean = is_ocean(ptile->terrain);
+  const bool ocean1 = is_ocean(ptile1->terrain);
+  struct pft_amphibious *amphibious = param->data;
+  int cost, scale;
+
+  if (ocean && ocean1) {
+    /* Sea move */
+    cost = amphibious->sea.get_MC(ptile, dir, ptile1, &amphibious->sea);
+    scale = amphibious->sea_scale;
+  } else if (ocean && is_allied_city_tile(ptile1, param->owner)) {
+    /* Entering port (same as sea move) */
+    cost = amphibious->sea.get_MC(ptile, dir, ptile1, &amphibious->sea);
+    scale = amphibious->sea_scale;
+  } else if (ocean) {
+    /* Disembark; use land movement function to handle F_MARINES */
+    cost = amphibious->land.get_MC(ptile, dir, ptile1, &amphibious->land);
+    scale = amphibious->land_scale;
+  } else if (is_allied_city_tile(ptile, param->owner) && ocean1) {
+    /* Leaving port (same as sea move) */
+    cost = amphibious->sea.get_MC(ptile, dir, ptile1, &amphibious->sea);
+    scale = amphibious->sea_scale;
+  } else if (ocean1) {
+    /* Now we have disembarked, our ferry can not help us - we have to
+     * stay on the land. */
+    cost = PF_IMPOSSIBLE_MC;
+    scale = amphibious->land_scale;
+  } else {
+    /* land move */
+    cost = amphibious->land.get_MC(ptile, dir, ptile1, &amphibious->land);
+    scale = amphibious->land_scale;
+  }
+  if (cost != PF_IMPOSSIBLE_MC) {
+    cost *= scale;
+  }
+  return cost;
+}
+
 /* ===================== Extra Cost Callbacks ======================== */
 
 /*********************************************************************
@@ -333,7 +379,7 @@ static int afraid_of_dark_forest(const struct tile *ptile,
 				 enum known_type known,
 				 struct pf_parameter *param)
 {
-  if (map_get_terrain(ptile) == T_FOREST) {
+  if (ptile->terrain->index == T_FOREST) {
     /* Willing to spend extra 2 turns to go around a forest tile */
     return PF_TURN_FACTOR * 2;
   }
@@ -342,6 +388,40 @@ static int afraid_of_dark_forest(const struct tile *ptile,
 }
 #endif
 
+/****************************************************************************
+  Extra cost call back for amphibious movement
+****************************************************************************/
+static int amphibious_extra_cost(const struct tile *ptile,
+				 enum known_type known,
+				 struct pf_parameter *param)
+{
+  struct pft_amphibious *amphibious = param->data;
+  const bool ocean = is_ocean(ptile->terrain);
+  int cost, scale;
+
+  if (known == TILE_UNKNOWN) {
+    /* We can travel almost anywhere */
+    cost = SINGLE_MOVE;
+    scale = MAX(amphibious->sea_scale, amphibious->land_scale);
+  } else if (ocean && amphibious->sea.get_EC) {
+    /* Do the EC callback for sea moves. */
+    cost = amphibious->sea.get_EC(ptile, known, &amphibious->sea);
+    scale = amphibious->sea_scale;
+  } else if (!ocean && amphibious->land.get_EC) {
+    /* Do the EC callback for land moves. */
+    cost = amphibious->land.get_EC(ptile, known, &amphibious->land);
+    scale = amphibious->land_scale;
+  } else {
+    cost = 0;
+    scale = 1;
+  }
+
+  if (cost != PF_IMPOSSIBLE_MC) {
+    cost *= scale;
+  }
+  return cost;
+}
+
 
 /* ===================== Tile Behaviour Callbacks ==================== */
 
@@ -390,36 +470,172 @@ enum tile_behavior no_fights(const struct tile *ptile, enum known_type known,
   return TB_NORMAL;
 }
 
+/****************************************************************************
+  PF callback to prohibit attacking anyone, except at the destination.
+****************************************************************************/
+enum tile_behavior no_intermediate_fights(const struct tile *ptile,
+					  enum known_type known,
+					  struct pf_parameter *param)
+{
+  if (is_non_allied_unit_tile(ptile, param->owner)
+      || is_non_allied_city_tile(ptile, param->owner)) {
+    return TB_DONT_LEAVE;
+  }
+  return TB_NORMAL;
+}
+
+/*********************************************************************
+  A callback for amphibious movement
+*********************************************************************/
+static enum tile_behavior amphibious_behaviour(const struct tile *ptile,
+					       enum known_type known,
+					       struct pf_parameter *param)
+{
+  struct pft_amphibious *amphibious = param->data;
+  const bool ocean = is_ocean(ptile->terrain);
+
+  /* Simply a wrapper for the sea or land tile_behavior callbacks. */
+  if (ocean && amphibious->sea.get_TB) {
+    return amphibious->sea.get_TB(ptile, known, &amphibious->sea);
+  } else if (!ocean && amphibious->land.get_TB) {
+    return amphibious->land.get_TB(ptile, known, &amphibious->land);
+  }
+  return TB_NORMAL;
+}
 
 /* =====================  Postion Dangerous Callbacks ================ */
 
 /**********************************************************************
   An example of position-dangerous callback.  For triremes.
   FIXME: it cheats.
+  Allow one move onto land (for use for ferries and land
+  bombardment)
 ***********************************************************************/
 static bool trireme_is_pos_dangerous(const struct tile *ptile,
 				     enum known_type known,
 				     struct pf_parameter *param)
 {
+  /* Assume that unknown tiles are unsafe. */
+  if (known == TILE_UNKNOWN) {
+    return TRUE;
+  }
+
   /* We test TER_UNSAFE even though under the current ruleset there is no
    * way for a trireme to be on a TER_UNSAFE tile. */
   /* Unsafe or unsafe-ocean tiles without cities are dangerous. */
-  return ((terrain_has_flag(ptile->terrain, TER_UNSAFE) 
-	  || (is_ocean(ptile->terrain) && !is_safe_ocean(ptile)))
-	  && ptile->city == NULL);
+  /* Pretend all land tiles are safe. */
+  return (ptile->city == NULL
+	  && is_ocean(ptile->terrain)
+	  && (terrain_has_flag(ptile->terrain, TER_UNSAFE) 
+	      || (is_ocean(ptile->terrain) && !is_safe_ocean(ptile))));
+}
+
+/****************************************************************************
+  Refueling base for air units.
+****************************************************************************/
+static bool is_possible_base_fuel(const struct tile *ptile,
+                                  struct pf_parameter *param)
+{
+  /* All airbases are considered possible, simply attack enemies. */
+  return (is_allied_city_tile(ptile, param->owner)
+       || tile_has_special(ptile, S_AIRBASE));
+}
+
+/****************************************************************************
+  Position-dangerous callback for air units.
+****************************************************************************/
+static bool is_pos_dangerous_fuel(const struct tile *ptile,
+                                  enum known_type known,
+                                  struct pf_parameter *param)
+{
+  int moves = SINGLE_MOVE * real_map_distance(param->start_tile, ptile);
+  int have = get_moves_left_initially(param);
+  int left = have - moves;
+
+  if (left < 0) {
+    /* not enough fuel. */
+    return TRUE;
+  }
+
+  if (have >= moves * 2
+   && (is_possible_base_fuel(param->start_tile, param)
+    || !param->owner->ai.control)) {
+    /* has enough fuel for round trip. */
+    return FALSE;
+  }
+
+  if (TILE_UNKNOWN != known
+   && (is_possible_base_fuel(ptile, param)
+    || is_enemy_city_tile(ptile, param->owner))) {
+    /* allow attacks, even suicidal ones */
+    return FALSE;
+  }
+
+  if (TILE_KNOWN == known
+   && is_enemy_unit_tile(ptile, param->owner)) {
+    /* don't reveal unknown units */
+    return FALSE;
+  }
+
+  /* similar to find_nearest_airbase() */
+  iterate_outward(ptile, left / SINGLE_MOVE, atile) {
+    if (TILE_UNKNOWN != tile_get_known(atile, param->owner)
+     && is_possible_base_fuel(atile, param)) {
+      return FALSE;
+    }
+  } iterate_outward_end;
+  
+  /* Carriers are ignored since they are likely to move. */
+  return TRUE;
 }
 
 /**********************************************************************
-  Position-dangerous callback for all units other than triremes.
+  Position-dangerous callback for sea units other than triremes.
+  Allow one move onto land (for use for ferries and land
+  bombardment)
+***********************************************************************/
+static bool is_overlap_pos_dangerous(const struct tile *ptile,
+				     enum known_type known,
+				     struct pf_parameter *param)
+{
+  /* Unsafe tiles without cities are dangerous. */
+  /* Pretend all land tiles are safe. */
+  return (ptile->city == NULL
+	  && is_ocean(ptile->terrain)
+	  && terrain_has_flag(ptile->terrain, TER_UNSAFE));
+}
+
+/**********************************************************************
+  Position-dangerous callback for typical units.
 ***********************************************************************/
 static bool is_pos_dangerous(const struct tile *ptile, enum known_type known,
 			     struct pf_parameter *param)
 {
   /* Unsafe tiles without cities are dangerous. */
-  return (terrain_has_flag(ptile->terrain, TER_UNSAFE)
+  return (ptile->terrain != T_UNKNOWN
+	  && terrain_has_flag(ptile->terrain, TER_UNSAFE)
 	  && ptile->city == NULL);
 }
 
+/****************************************************************************
+  Position-dangerous callback for amphibious movement.
+****************************************************************************/
+static bool amphibious_is_pos_dangerous(const struct tile *ptile,
+					enum known_type known,
+					struct pf_parameter *param)
+{
+  struct pft_amphibious *amphibious = param->data;
+  const bool ocean = is_ocean(ptile->terrain);
+
+  /* Simply a wrapper for the sea or land danger callbacks. */
+  if (ocean && amphibious->sea.is_pos_dangerous) {
+    return amphibious->sea.is_pos_dangerous(ptile, known, param);
+  } else if (!ocean && amphibious->land.is_pos_dangerous) {
+    return amphibious->land.is_pos_dangerous(ptile, known, param);
+  }
+  return FALSE;
+}
+
 /* =====================  Tools for filling parameters =============== */
 
 /**********************************************************************
@@ -432,34 +648,54 @@ void pft_fill_unit_parameter(struct pf_parameter *parameter,
 
   switch (unit_type(punit)->move_type) {
   case LAND_MOVING:
-    if (unit_flag(punit, F_IGTER)) {
+    if (unit_has_type_flag(punit, F_IGTER)) {
       parameter->get_MC = igter_move_unit;
     } else {
       parameter->get_MC = normal_move_unit;
     }
     break;
   case SEA_MOVING:
-    if (unit_flag(punit, F_NO_LAND_ATTACK)) {
-      parameter->get_MC = seamove_no_bombard;
-    } else {
+    if (!unit_has_type_flag(punit, F_NO_LAND_ATTACK)) {
       parameter->get_MC = seamove;
+    } else {
+      parameter->get_MC = seamove_no_bombard;
     }
     break;
   case AIR_MOVING:
     parameter->get_MC = single_airmove;
+    if (unit_type(punit)->fuel > 0) {
+      parameter->is_pos_dangerous = is_pos_dangerous_fuel;
+    } else {
+      parameter->is_pos_dangerous = NULL;
+    }
+    parameter->turn_mode = TM_WORST_TIME;
+    break;
+  case HELI_MOVING:
+    /* Helicoptors are treated similarly to airplanes. */
+    parameter->get_MC = single_airmove;
+    if (get_player_bonus(unit_owner(punit), EFT_UNIT_RECOVER)
+	>= unit_type(punit)->hp / 10) {
+      /* United nations cancels out helicoptor fuel loss. */
+      parameter->is_pos_dangerous = NULL;
+    } else {
+      /* Otherwise, don't risk fuel loss. */
+      parameter->is_pos_dangerous = is_pos_dangerous_fuel;
+      parameter->turn_mode = TM_WORST_TIME;
+    }
     break;
   default:
-    die("unknown move_type");
+    freelog(LOG_ERROR, "pft_fill_unit_parameter() impossible move type!");
+    break;
   }
 
   if (unit_type(punit)->move_type == LAND_MOVING 
-      && !unit_flag(punit, F_IGZOC)) {
+      && !unit_has_type_flag(punit, F_IGZOC)) {
     parameter->get_zoc = is_my_zoc;
   } else {
     parameter->get_zoc = NULL;
   }
 
-  if (unit_flag(punit, F_TRIREME)
+  if (unit_has_type_flag(punit, F_TRIREME)
       && base_trireme_loss_pct(unit_owner(punit), punit) > 0) {
     parameter->turn_mode = TM_WORST_TIME;
     parameter->is_pos_dangerous = trireme_is_pos_dangerous;
@@ -470,34 +706,49 @@ void pft_fill_unit_parameter(struct pf_parameter *parameter,
 }
 
 /**********************************************************************
-  Switch on one tile overlapping into the sea/land 
-  ("sea/land bombardment")
+  Switch on one tile overlapping into the sea/land.
+  For sea/land bombardment and for ferries.
 **********************************************************************/
 void pft_fill_unit_overlap_param(struct pf_parameter *parameter,
 				 struct unit *punit)
 {
+  const bool trireme_danger = unit_has_type_flag(punit, F_TRIREME)
+                     && base_trireme_loss_pct(unit_owner(punit), punit) > 0;
+  const bool danger
+    = base_unsafe_terrain_loss_pct(unit_owner(punit), punit) > 0;
+
   pft_fill_unit_default_parameter(parameter, punit);
 
   switch (unit_type(punit)->move_type) {
   case LAND_MOVING:
     parameter->get_MC = land_overlap_move;
     parameter->get_TB = dont_cross_ocean;
+
+    assert(!trireme_danger);
+    if (danger) {
+      parameter->is_pos_dangerous = is_pos_dangerous;
+    }
     break;
   case SEA_MOVING:
     parameter->get_MC = sea_overlap_move;
+
+    if (trireme_danger) {
+      parameter->is_pos_dangerous = trireme_is_pos_dangerous;
+    } else if (danger) {
+      parameter->is_pos_dangerous = is_overlap_pos_dangerous;
+    }
+    break;
+  case AIR_MOVING:
+  case HELI_MOVING:
+    assert(!danger && !trireme_danger);
+    parameter->get_MC = single_airmove; /* very crude */
     break;
   default:
-    die("Unsupported move_type");
+    freelog(LOG_ERROR, "pft_fill_unit_overlap_param() impossible move type!");
+    break;
   }
 
   parameter->get_zoc = NULL;
-
-  if (unit_flag(punit, F_TRIREME)
-      && base_trireme_loss_pct(unit_owner(punit), punit) > 0) {
-    parameter->is_pos_dangerous = trireme_is_pos_dangerous;
-  } else if (base_unsafe_terrain_loss_pct(unit_owner(punit), punit) > 0) {
-    parameter->is_pos_dangerous = is_pos_dangerous;
-  }
 }
 
 /**********************************************************************
@@ -515,12 +766,17 @@ void pft_fill_unit_attack_param(struct pf_parameter *parameter,
   case SEA_MOVING:
     parameter->get_MC = sea_attack_move;
     break;
+  case AIR_MOVING:
+  case HELI_MOVING:
+    parameter->get_MC = single_airmove; /* very crude */
+    break;
   default:
-    die("Unsupported move_type");
+    freelog(LOG_ERROR, "pft_fill_unit_attack_param() impossible move type!");
+    break;
   }
 
   if (unit_type(punit)->move_type == LAND_MOVING 
-      && !unit_flag(punit, F_IGZOC)) {
+      && !unit_has_type_flag(punit, F_IGZOC)) {
     parameter->get_zoc = is_my_zoc;
   } else {
     parameter->get_zoc = NULL;
@@ -530,6 +786,38 @@ void pft_fill_unit_attack_param(struct pf_parameter *parameter,
   parameter->is_pos_dangerous = NULL;
 }
 
+/****************************************************************************
+  Fill parameters for combined sea-land movement.
+  This is suitable for the case of a land unit riding a ferry.
+  The starting position of the ferry is taken to be the starting position for
+  the PF. The passenger is assumed to initailly be on the given ferry.
+  The destination may be inland, in which case the passenger will ride
+  the ferry to a beach head, disembark, then continue on land.
+  One complexity of amphibious movement is that the movement rate on land
+  might be different from that at sea. We therefore scale up the movement
+  rates (and the corresponding movement consts) to the product of the two
+  rates.
+****************************************************************************/
+void pft_fill_amphibious_parameter(struct pft_amphibious *parameter)
+{
+  const int move_rate = parameter->land.move_rate * parameter->sea.move_rate;
+
+  parameter->combined = parameter->sea;
+  parameter->land_scale = move_rate / parameter->land.move_rate;
+  parameter->sea_scale = move_rate / parameter->sea.move_rate;
+  parameter->combined.moves_left_initially *= parameter->sea_scale;
+  parameter->combined.move_rate = move_rate;
+  /* To ensure triremes behave correctly: */
+  parameter->combined.turn_mode = TM_WORST_TIME;
+  parameter->combined.get_MC = amphibious_move;
+  parameter->combined.get_TB = amphibious_behaviour;
+  parameter->combined.get_EC = amphibious_extra_cost;
+  parameter->combined.is_pos_dangerous = amphibious_is_pos_dangerous;
+  BV_CLR_ALL(parameter->combined.unit_flags);
+
+  parameter->combined.data = parameter;
+}
+
 /**********************************************************************
   Fill general use parameters to defaults
 ***********************************************************************/
@@ -537,6 +825,29 @@ static void pft_fill_unit_default_parameter(struct pf_parameter *parameter,
 					    struct unit *punit)
 {
   parameter->turn_mode = TM_CAPPED;
+  parameter->unknown_MC = SINGLE_MOVE;
+
+  switch (unit_type(punit)->move_type) {
+  case AIR_MOVING:
+  case HELI_MOVING:
+    /* no change */;
+    break;
+  case SEA_MOVING:
+    /* Sailing units explore less? */
+    parameter->unknown_MC *= 2;
+    break;
+  case LAND_MOVING:
+    terrain_type_iterate(pterrain) {
+      int mr = 2 * pterrain->movement_cost;
+
+      parameter->unknown_MC = MAX(mr, parameter->unknown_MC);
+    } terrain_type_iterate_end;
+    break;
+  default:
+    freelog(LOG_ERROR, "pft_fill_unit_default_parameter() impossible move type!");
+    break;
+  }
+
   parameter->get_TB = NULL;
   parameter->get_EC = NULL;
   parameter->is_pos_dangerous = NULL;
@@ -547,6 +858,13 @@ static void pft_fill_unit_default_parameter(struct pf_parameter *parameter,
   parameter->start_tile = punit->tile;
   parameter->moves_left_initially = punit->moves_left;
   parameter->move_rate = unit_move_rate(punit);
+  if (is_air_unit(punit)) {
+    parameter->fuel_left_initially = punit->fuel;
+    parameter->fuel = unit_type(punit)->fuel;
+  } else {
+    parameter->fuel = 1;
+    parameter->fuel_left_initially = 1;
+  }
   parameter->owner = unit_owner(punit);
   parameter->unit_flags = unit_type(punit)->flags;
 
@@ -587,3 +905,27 @@ struct pf_path *pft_concat(struct pf_path *dest_path,
   }
   return dest_path;
 }
+
+/****************************************************************************
+  Remove the part of a path leading up to a given tile.
+  The given tile must be on the path.  If it is on the path more than once
+  then the first occurrance will be the one used.
+****************************************************************************/
+void pft_advance_path(struct pf_path *path, struct tile *ptile)
+{
+  int i;
+  struct pf_position *new_positions;
+
+  for (i = 0; i < path->length; i++) {
+    if (path->positions[i].tile == ptile) {
+      break;
+    }
+  }
+  assert(i < path->length);
+  path->length -= i;
+  new_positions = fc_malloc(sizeof(*path->positions) * path->length);
+  memcpy(new_positions, path->positions + i,
+	 path->length * sizeof(*path->positions));
+  free(path->positions);
+  path->positions = new_positions;
+}
