@@ -21,19 +21,6 @@
 #include <assert.h>
 #include <limits.h>
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-#ifdef HAVE_WINSOCK
-#include <winsock.h>
-#endif
-
 #include "capability.h"
 #include "dataio.h"
 #include "events.h"
@@ -64,7 +51,7 @@
 #define JUMBO_BORDER 		(64*1024-COMPRESSION_BORDER-1)
 #endif
 
-#define BASIC_PACKET_LOG_LEVEL	LOG_DEBUG
+#define BASIC_PACKET_LOG_LEVEL	LOG_VERBOSE
 #define COMPRESS_LOG_LEVEL	LOG_DEBUG
 #define COMPRESS2_LOG_LEVEL	LOG_DEBUG
 
@@ -93,8 +80,7 @@
 ***********************************************************************/
 
 /**************************************************************************
-It returns the request id of the outgoing packet or 0 if the packet
-was no request (i.e. server sends packet).
+  It returns the request id of the outgoing packet (or 0 if pc->is_server).
 **************************************************************************/
 int send_packet_data(struct connection *pc, unsigned char *data, int len)
 {
@@ -104,11 +90,11 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
   freelog(BASIC_PACKET_LOG_LEVEL, "sending packet type=%s(%d) len=%d",
 	  get_packet_name(data[2]), data[2], len);
 
-  if (!is_server) {
+  if (!pc->is_server) {
     pc->client.last_request_id_used =
 	get_next_request_id(pc->client.last_request_id_used);
     result = pc->client.last_request_id_used;
-    freelog(LOG_DEBUG, "sending request %d", result);
+    freelog(BASIC_PACKET_LOG_LEVEL, "sending request %d", result);
   }
 
   if (pc->outgoing_packet_notify) {
@@ -133,6 +119,12 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
       compression_level_initialized = TRUE;
     }
 
+    /* TODO: PACKET_FREEZE_HINT and PACKET_THAW_HINT are meaningful
+     * only internally. They should not be sent to connection at all.
+     * Freezing could also be handled via separate functions, and
+     * not by special packets.
+     * Only problem is backward compatibility, so this cannot be
+     * changed in stable branch. */
     if (packet_type == PACKET_PROCESSING_STARTED
 	|| packet_type == PACKET_FREEZE_HINT) {
       if (pc->compression.frozen_level == 0) {
@@ -373,7 +365,7 @@ void *get_packet_from_connection(struct connection *pc,
      * We don't know the decompressed size. We assume a bad case
      * here: an expansion by an factor of 100. 
      */
-    uLongf decompressed_size = 100 * compressed_size;
+    unsigned long int decompressed_size = 100 * compressed_size;
     void *decompressed = fc_malloc(decompressed_size);
     int error;
     struct socket_packet_buffer *buffer = pc->buffer;
@@ -478,8 +470,7 @@ void *get_packet_from_connection(struct connection *pc,
     packets_stats[packet_type].size += size;
 
     packet_counter++;
-    if ((is_server && (packet_counter % 10 == 0))
-	|| (!is_server && (packet_counter % 1000 == 0))) {
+    if (packet_counter % 100 == 0) {
       int i, sum = 0;
 
       freelog(LOG_NORMAL, "Received packets:");
@@ -573,8 +564,10 @@ void generic_handle_player_attribute_chunk(struct player *pplayer,
 					   packet_player_attribute_chunk
 					   *chunk)
 {
-  freelog(LOG_DEBUG, "received attribute chunk %d/%d %d", chunk->offset,
-	  chunk->total_length, chunk->chunk_length);
+  freelog(BASIC_PACKET_LOG_LEVEL, "received attribute chunk %u/%u %u",
+	  (unsigned int) chunk->offset,
+	  (unsigned int) chunk->total_length,
+	  (unsigned int) chunk->chunk_length);
 
   if (chunk->total_length < 0
       || chunk->chunk_length < 0
@@ -653,6 +646,12 @@ void send_attribute_block(const struct player *pplayer,
 	   packet.chunk_length);
     bytes_left -= packet.chunk_length;
 
+    if (packet.chunk_length < ATTRIBUTE_CHUNK_SIZE) {
+      /* Last chunk is not full. Make sure that delta does
+       * not use random data. */
+      memset(packet.data, 0, ATTRIBUTE_CHUNK_SIZE - packet.chunk_length);
+    }
+
     send_packet_player_attribute_chunk(pconn, &packet);
   }
 
@@ -698,6 +697,9 @@ void post_receive_packet_chat_msg(struct connection *pc,
   }
 }
 
+/**************************************************************************
+  Test and log for sending player attribute_block
+**************************************************************************/
 void pre_send_packet_player_attribute_chunk(struct connection *pc,
 					    struct packet_player_attribute_chunk
 					    *packet)
@@ -710,16 +712,23 @@ void pre_send_packet_player_attribute_chunk(struct connection *pc,
   assert(packet->chunk_length <= packet->total_length);
   assert(packet->offset >= 0 && packet->offset < packet->total_length);
 
-  freelog(LOG_DEBUG, "sending attribute chunk %d/%d %d", packet->offset,
-	  packet->total_length, packet->chunk_length);
+  freelog(BASIC_PACKET_LOG_LEVEL, "sending attribute chunk %d/%d %d",
+	  packet->offset, packet->total_length, packet->chunk_length);
 
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
 void post_receive_packet_game_state(struct connection *pc,
 				    struct packet_game_state *packet)
 {
   conn_clear_packet_cache(pc);
 }
+
+/**************************************************************************
+  ...
+**************************************************************************/
 void post_send_packet_game_state(struct connection *pc,
 				 const struct packet_game_state *packet)
 {
