@@ -21,6 +21,7 @@
 #include "capstr.h"
 #include "events.h"
 #include "fcintl.h"
+#include "game.h"
 #include "log.h"
 #include "mem.h"
 #include "packets.h"
@@ -31,7 +32,6 @@
 #include "auth.h"
 #include "diplhand.h"
 #include "gamehand.h"
-#include "gamelog.h"
 #include "maphand.h"
 #include "meta.h"
 #include "plrhand.h"
@@ -47,11 +47,11 @@
   has started.  If pconn is NULL, is an AI, else a client.
 
   N.B. this only attachs a connection to a player if 
-       pconn->name == player->username
+       pconn->username == player->username
 **************************************************************************/
 void establish_new_connection(struct connection *pconn)
 {
-  struct conn_list *dest = &pconn->self;
+  struct conn_list *dest = pconn->self;
   struct player *pplayer;
   struct packet_server_join_reply packet;
   char hostname[512];
@@ -59,11 +59,10 @@ void establish_new_connection(struct connection *pconn)
   /* zero out the password */
   memset(pconn->server.password, 0, sizeof(pconn->server.password));
 
-  /* send off login_replay packet */
+  /* send join_reply packet */
   packet.you_can_join = TRUE;
   sz_strlcpy(packet.capability, our_capability);
-  my_snprintf(packet.message, sizeof(packet.message), _("%s Welcome"),
-              pconn->username);
+  sz_strlcpy(packet.message, pconn->username);
   sz_strlcpy(packet.challenge_file, new_challenge_filename(pconn));
   packet.conn_id = pconn->id;
   send_packet_server_join_reply(pconn, &packet);
@@ -72,168 +71,214 @@ void establish_new_connection(struct connection *pconn)
   pconn->established = TRUE;
   pconn->server.status = AS_ESTABLISHED;
 
+  conn_list_append(game.est_connections, pconn);
+  if (conn_list_size(game.est_connections) == 1) {
+    /* First connection
+     * Replace "restarting in x seconds" meta message */
+    maybe_automatic_meta_message(default_meta_message_string());
+    (void) send_server_info_to_metaserver(META_INFO);
+  }
+
   /* introduce the server to the connection */
   if (my_gethostname(hostname, sizeof(hostname)) == 0) {
-    notify_conn(dest, _("Welcome to the %s Server running at %s port %d."),
+    notify_conn(dest, NULL, E_CONNECTION,
+		_("Welcome to the %s Server running at %s port %d."),
                 freeciv_name_version(), hostname, srvarg.port);
   } else {
-    notify_conn(dest, _("Welcome to the %s Server at port %d."),
+    notify_conn(dest, NULL, E_CONNECTION,
+		_("Welcome to the %s Server at port %d."),
                 freeciv_name_version(), srvarg.port);
   }
 
   /* FIXME: this (getting messages about others logging on) should be a 
    * message option for the client with event */
 
-  /* notify the console and other established connections that you're here */
-  freelog(LOG_NORMAL, _("%s has connected from %s."),
-          pconn->username, pconn->addr);
+  /* TRANS: (connection) identifier: ... [network address] */
+  freelog(LOG_NORMAL, _("(%d) %s: connected [%s]"),
+          pconn->id, pconn->username, pconn->addr);
+
+  /* notify the other established connections that you're here */
   conn_list_iterate(game.est_connections, aconn) {
     if (aconn != pconn) {
-      notify_conn(&aconn->self, _("Server: %s has connected from %s."),
+      notify_conn(aconn->self, NULL, E_CONNECTION,
+		  _("Server: %s has connected from %s."),
                   pconn->username, pconn->addr);
     }
   } conn_list_iterate_end;
 
+  send_rulesets(dest);
+  send_server_settings(dest);
+
   /* a player has already been created for this user, reconnect him */
   if ((pplayer = find_player_by_user(pconn->username))) {
     attach_connection_to_player(pconn, pplayer);
 
-    if (server_state == RUN_GAME_STATE) {
-      /* Player and other info is only updated when the game is running.
-       * See the comment in lost_connection_to_client(). */
-      send_packet_freeze_hint(pconn);
-      send_rulesets(dest);
-      send_all_info(dest);
-      send_game_state(dest, CLIENT_GAME_RUNNING_STATE);
-      send_player_info(NULL,NULL);
-      send_diplomatic_meetings(pconn);
-      send_packet_thaw_hint(pconn);
-      send_packet_start_turn(pconn);
-    }
-
-    /* This must be done after the above info is sent, because it will
-     * generate a player-packet which can't be sent until (at least)
-     * rulesets are sent. */
-    if (game.auto_ai_toggle && pplayer->ai.control) {
+    if (game.info.auto_ai_toggle && pplayer->ai.control) {
       toggle_ai_player_direct(NULL, pplayer);
     }
 
-    gamelog(GAMELOG_PLAYER, pplayer);
-
-  } else if (server_state == PRE_GAME_STATE && game.is_new_game) {
-    if (!attach_connection_to_player(pconn, NULL)) {
-      notify_conn(dest, _("Couldn't attach your connection to new player."));
-      freelog(LOG_VERBOSE, "%s is not attached to a player", pconn->username);
+    if (S_S_RUNNING == server_state()) {
+      /* Player and other info is only updated when the game is running.
+       * See the comment in lost_connection_to_client(). */
+      send_packet_freeze_hint(pconn);
+      send_all_info(dest);
+      send_game_state(dest, C_S_RUNNING);
+      send_diplomatic_meetings(pconn);
+      send_packet_thaw_hint(pconn);
+      dsend_packet_start_phase(pconn, game.info.phase);
     } else {
-      sz_strlcpy(pconn->player->name, pconn->username);
+      /* send new player connection to everybody */
+      send_game_info(game.est_connections);
+      send_player_info_c(NULL, game.est_connections);
+      send_conn_info(game.est_connections, dest);
     }
+  } else {
+    if (S_S_INITIAL == server_state() && game.info.is_new_game) {
+      if (attach_connection_to_player(pconn, NULL)) {
+        sz_strlcpy(pconn->player->name, pconn->username);
+
+        /* send new player connection to everybody */
+        send_game_info(game.est_connections);
+        send_player_info_c(NULL, game.est_connections);
+      } else {
+        notify_conn(dest, NULL, E_CONNECTION,
+                    _("Couldn't attach your connection to new player."));
+        freelog(LOG_VERBOSE, "%s is not attached to a player", pconn->username);
+
+        /* send old player connections to self */
+        send_game_info(dest);
+        send_player_info_c(NULL, dest);
+      }
+    } else {
+      /* send old player connections to self */
+      send_game_info(dest);
+      send_player_info_c(NULL, dest);
+    }
+    send_conn_info(game.est_connections, dest);
   }
+  /* redundant self to self cannot be avoided */
+  send_conn_info(dest, game.est_connections);
 
   /* remind the connection who he is */
   if (!pconn->player) {
-    notify_conn(dest, _("You are logged in as '%s' connected to no player."),
+    notify_conn(dest, NULL, E_CONNECTION,
+		_("You are logged in as '%s' connected to no player."),
                 pconn->username);
-  } else if (strcmp(pconn->player->name, ANON_PLAYER_NAME) == 0) {
-    notify_conn(dest, _("You are logged in as '%s' connected to an "
-                        "anonymous player."),
+  } else if (strcmp(player_name(pconn->player), ANON_PLAYER_NAME) == 0) {
+    notify_conn(dest, NULL, E_CONNECTION,
+		_("You are logged in as '%s' connected to an "
+		  "anonymous player."),
 		pconn->username);
   } else {
-    notify_conn(dest, _("You are logged in as '%s' connected to %s."),
-                pconn->username, pconn->player->name);
+    notify_conn(dest, NULL, E_CONNECTION,
+		_("You are logged in as '%s' connected to %s."),
+                pconn->username,
+                player_name(pconn->player));
   }
 
-  /* if need be, tell who we're waiting on to end the game turn */
-  if (server_state == RUN_GAME_STATE && game.turnblock) {
+  /* if need be, tell who we're waiting on to end the game.info.turn */
+  if (S_S_RUNNING == server_state() && game.info.turnblock) {
     players_iterate(cplayer) {
       if (cplayer->is_alive
           && !cplayer->ai.control
-          && !cplayer->turn_done
+          && !cplayer->phase_done
           && cplayer != pconn->player) {  /* skip current player */
-        notify_conn(dest, _("Turn-blocking game play: "
-                            "waiting on %s to finish turn..."),
-                    cplayer->name);
+        notify_conn(dest, NULL, E_CONNECTION,
+		    _("Turn-blocking game play: "
+		      "waiting on %s to finish turn..."),
+                    player_name(cplayer));
       }
     } players_iterate_end;
   }
 
   /* if the game is running, players can just view the Players menu? --dwp */
-  if (server_state != RUN_GAME_STATE) {
+  if (S_S_RUNNING != server_state()) {
     show_players(pconn);
   }
 
-  send_conn_info(dest, &game.est_connections);
-  conn_list_insert_back(&game.est_connections, pconn);
-  send_conn_info(&game.est_connections, dest);
+  reset_all_start_commands();
   (void) send_server_info_to_metaserver(META_INFO);
 }
 
 /**************************************************************************
   send the rejection packet to the client.
 **************************************************************************/
-void reject_new_connection(const char *msg, struct connection *pconn)
+void reject_new_connection(struct connection *pconn, const char *message)
 {
   struct packet_server_join_reply packet;
 
   /* zero out the password */
   memset(pconn->server.password, 0, sizeof(pconn->server.password));
 
+  /* now send join_reply packet */
+  memset(&packet, 0, sizeof(packet));
   packet.you_can_join = FALSE;
+  sz_strlcpy(packet.message, message);
   sz_strlcpy(packet.capability, our_capability);
-  sz_strlcpy(packet.message, msg);
   packet.challenge_file[0] = '\0';
   packet.conn_id = -1;
+
   send_packet_server_join_reply(pconn, &packet);
-  freelog(LOG_NORMAL, _("Client rejected: %s."), conn_description(pconn));
   flush_connection_send_buffer_all(pconn);
 }
 
 /**************************************************************************
- Returns FALSE if the clients gets rejected and the connection should be
- closed. Returns TRUE if the client get accepted.
+ Returns FALSE when the clients are rejected and the connection should be
+ closed. Returns TRUE otherwise.
 **************************************************************************/
-bool handle_login_request(struct connection *pconn, 
-                          struct packet_server_join_req *req)
+bool server_join_request(struct connection *pconn, void *packet)
 {
-  char msg[MAX_LEN_MSG];
-  
-  freelog(LOG_NORMAL, _("Connection request from %s from %s"),
-          req->username, pconn->addr);
-  
-  /* print server and client capabilities to console */
-  freelog(LOG_NORMAL, _("%s has client version %d.%d.%d%s"),
-          pconn->username, req->major_version, req->minor_version,
-          req->patch_version, req->version_label);
-  freelog(LOG_VERBOSE, "Client caps: %s", req->capability);
-  freelog(LOG_VERBOSE, "Server caps: %s", our_capability);
-  sz_strlcpy(pconn->capability, req->capability);
-  
+  char username[MAX_LEN_NAME];
+#define req ((struct packet_server_join_req *)packet)
+
+  /* Never assume network data is valid and printable! */
+  sz_strlcpy(username,
+             is_valid_username(req->username)
+             ? req->username
+             : "?");
+  /* TRANS: (connection) identifier: ... [network address] */
+  freelog(LOG_NORMAL, _("(%d) %s: connection request [%s]"),
+          pconn->id,
+          username,
+          pconn->addr);
+
+  /* log client and server capabilities */
+  freelog(LOG_VERBOSE, "(%d) %s: client version %d.%d.%d%s",
+          pconn->id,
+          username,
+          req->major_version,
+          req->minor_version,
+          req->patch_version,
+          ('\0' == req->version_label[0] || is_ascii_name(req->version_label))
+          ? req->version_label
+          : "?");
+
+  sz_strlcpy(pconn->capability,
+             ('\0' == req->capability[0] || is_ascii_name(req->capability))
+             ? req->capability
+             : "?");
+  freelog(LOG_VERBOSE, "Client: %s", pconn->capability);
+  freelog(LOG_VERBOSE, "Server: %s", our_capability);
+
   /* Make sure the server has every capability the client needs */
-  if (!has_capabilities(our_capability, req->capability)) {
-    my_snprintf(msg, sizeof(msg),
-                _("The client is missing a capability that this server needs.\n"
-                   "Server version: %d.%d.%d%s Client version: %d.%d.%d%s."
-                   "  Upgrading may help!"),
-                MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, VERSION_LABEL,
-                req->major_version, req->minor_version,
-                req->patch_version, req->version_label);
-    reject_new_connection(msg, pconn);
-    freelog(LOG_NORMAL, _("%s was rejected: Mismatched capabilities."),
-            req->username);
+  if (!has_capabilities(our_capability, pconn->capability)) {
+    reject_new_connection(pconn,
+                          N_("The client is missing a capability"
+                             " that this server needs."));
+    freelog(LOG_NORMAL, _("(%d) %s rejected: mismatched capabilities!"),
+            pconn->id,
+            username);
     return FALSE;
   }
 
   /* Make sure the client has every capability the server needs */
-  if (!has_capabilities(req->capability, our_capability)) {
-    my_snprintf(msg, sizeof(msg),
-                _("The server is missing a capability that the client needs.\n"
-                   "Server version: %d.%d.%d%s Client version: %d.%d.%d%s."
-                   "  Upgrading may help!"),
-                MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, VERSION_LABEL,
-                req->major_version, req->minor_version,
-                req->patch_version, req->version_label);
-    reject_new_connection(msg, pconn);
-    freelog(LOG_NORMAL, _("%s was rejected: Mismatched capabilities."),
-            req->username);
+  if (!has_capabilities(pconn->capability, our_capability)) {
+    reject_new_connection(pconn,
+                          N_("This server is missing a capability"
+                             " that the client needs."));
+    freelog(LOG_NORMAL, _("(%d) %s rejected: mismatched capabilities!"),
+            pconn->id,
+            username);
     return FALSE;
   }
 
@@ -241,21 +286,26 @@ bool handle_login_request(struct connection *pconn,
 
   /* Name-sanity check: could add more checks? */
   if (!is_valid_username(req->username)) {
-    my_snprintf(msg, sizeof(msg), _("Invalid username '%s'"), req->username);
-    reject_new_connection(msg, pconn);
-    freelog(LOG_NORMAL, _("%s was rejected: Invalid name [%s]."),
-            req->username, pconn->addr);
+    reject_new_connection(pconn,
+                          N_("The username is invalid!"));
+    /* TRANS: (connection) identifier: ... [network address] */
+    freelog(LOG_NORMAL, _("(%d) %s rejected: invalid username [%s]"),
+            pconn->id,
+            username,
+            pconn->addr);
     return FALSE;
   } 
 
   /* don't allow duplicate logins */
   conn_list_iterate(game.all_connections, aconn) {
     if (mystrcasecmp(req->username, aconn->username) == 0) { 
-      my_snprintf(msg, sizeof(msg), _("'%s' already connected."), 
-                  req->username);
-      reject_new_connection(msg, pconn);
-      freelog(LOG_NORMAL, _("%s was rejected: Duplicate login name [%s]."),
-              req->username, pconn->addr);
+      reject_new_connection(pconn,
+                            N_("Your username is already connected!"));
+      /* TRANS: (connection) identifier: ... [network address] */
+      freelog(LOG_NORMAL, _("(%d) %s rejected: duplicate username [%s]"),
+              pconn->id,
+              username,
+              pconn->addr);
       return FALSE;
     }
   } conn_list_iterate_end;
@@ -272,7 +322,7 @@ bool handle_login_request(struct connection *pconn,
 /**************************************************************************
   High-level server stuff when connection to client is closed or lost.
   Reports loss to log, and to other players if the connection was a
-  player.  Also removes player if in pregame, applies auto_toggle, and
+  player.  Also removes player in pregame, applies auto_toggle, and
   does check for turn done (since can depend on connection/ai status).
   Note caller should also call close_connection() after this, to do
   lower-level close stuff.
@@ -288,9 +338,10 @@ void lost_connection_to_client(struct connection *pconn)
    * really lost (as opposed to server shutting it down) which would
    * trigger an error on send and recurse back to here.
    * Safe to unlink even if not in list: */
-  conn_list_unlink(&game.est_connections, pconn);
+  conn_list_unlink(game.est_connections, pconn);
   delayed_disconnect++;
-  notify_conn(&game.est_connections, _("Game: Lost connection: %s."), desc);
+  notify_conn(game.est_connections, NULL, E_CONNECTION,
+	      _("Lost connection: %s."), desc);
 
   if (!pplayer) {
     delayed_disconnect--;
@@ -298,43 +349,33 @@ void lost_connection_to_client(struct connection *pconn)
   }
 
   unattach_connection_from_player(pconn);
-
-  send_conn_info_remove(&pconn->self, &game.est_connections);
-  if (server_state == RUN_GAME_STATE) {
-    /* Player info is only updated when the game is running; this must be
-     * done consistently or the client will end up with inconsistent errors.
-     * At other times, the conn info (send_conn_info) is used by the client
-     * to display player information.  See establish_new_connection(). */
-    send_player_info(pplayer, NULL);
-  }
+  send_conn_info_remove(pconn->self, game.est_connections);
   notify_if_first_access_level_is_available();
 
-  /* Cancel diplomacy meetings */
-  if (!pplayer->is_connected) { /* may be still true if multiple connections */
-    players_iterate(other_player) {
-      if (find_treaty(pplayer, other_player)) {
-        handle_diplomacy_cancel_meeting_req(pplayer, other_player->player_no);
-      }
-    } players_iterate_end;
-  }
-
-  if (game.is_new_game
+  if (game.info.is_new_game
       && !pplayer->is_connected /* eg multiple controllers */
       && !pplayer->ai.control    /* eg created AI player */
-      && (server_state == PRE_GAME_STATE 
-          || server_state == SELECT_RACES_STATE)) {
+      && S_S_INITIAL == server_state()) {
     server_remove_player(pplayer);
   } else {
-    if (game.auto_ai_toggle
+    if (game.info.auto_ai_toggle
         && !pplayer->ai.control
         && !pplayer->is_connected /* eg multiple controllers */) {
       toggle_ai_player_direct(NULL, pplayer);
     }
 
-    gamelog(GAMELOG_PLAYER, pplayer);
-
     check_for_full_turn_done();
   }
+  /* send_player_info() was formerly updated by toggle_ai_player_direct(),
+   * so it must be safe to send here now?
+   *
+   * At other times, data from send_conn_info() is used by the client to
+   * display player information.  See establish_new_connection().
+   */
+  freelog(LOG_VERBOSE, "lost_connection_to_client() calls send_player_info_c()");
+  send_player_info_c(pplayer, game.est_connections);
+
+  reset_all_start_commands();
 
   delayed_disconnect--;
 }
@@ -366,8 +407,12 @@ static void send_conn_info_arg(struct conn_list *src,
                                struct conn_list *dest, bool remove)
 {
   struct packet_conn_info packet;
+
+  if (!dest) {
+    dest = game.est_connections;
+  }
   
-  conn_list_iterate(*src, psrc) {
+  conn_list_iterate(src, psrc) {
     package_conn_info(psrc, &packet);
     if (remove) {
       packet.used = FALSE;
@@ -402,30 +447,34 @@ void send_conn_info_remove(struct conn_list *src, struct conn_list *dest)
   If pplayer is NULL, take the next available player that is not already 
   associated.
   Note "observer" connections do not count for is_connected. You must set
-       pconn->obserber to TRUE before attaching!
+       pconn->observer to TRUE before attaching!
 **************************************************************************/
 bool attach_connection_to_player(struct connection *pconn,
                                  struct player *pplayer)
 {
   /* if pplayer is NULL, attach to first non-connected player slot */
   if (!pplayer) {
-    if (game.nplayers > game.max_players 
-        || game.nplayers > MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS) {
+    if (game.info.nplayers >= game.info.max_players 
+        || game.info.nplayers >= MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS) {
       return FALSE; 
     } else {
-      pplayer = &game.players[game.nplayers];
-      game.nplayers++;
+      pplayer = &game.players[game.info.nplayers];
+      server_player_init(pplayer, FALSE, TRUE);
+      game.info.nplayers++;
+      send_game_info(game.est_connections);
     }
   }
 
   if (!pconn->observer) {
     sz_strlcpy(pplayer->username, pconn->username);
+    pplayer->user_turns = 0; /* reset for a new user */
     pplayer->is_connected = TRUE;
   }
 
   pconn->player = pplayer;
-  conn_list_insert_back(&pplayer->connections, pconn);
-  conn_list_insert_back(&game.game_connections, pconn);
+  conn_list_append(pplayer->connections, pconn);
+
+  aifill(game.info.aifill);
 
   return TRUE;
 }
@@ -442,8 +491,7 @@ bool unattach_connection_from_player(struct connection *pconn)
     return FALSE; /* no player is attached to this conn */
   }
 
-  conn_list_unlink(&pconn->player->connections, pconn);
-  conn_list_unlink(&game.game_connections, pconn);
+  conn_list_unlink(pconn->player->connections, pconn);
 
   pconn->player->is_connected = FALSE;
   pconn->observer = FALSE;
