@@ -77,6 +77,7 @@
 #include "auth.h"
 #include "connecthand.h"
 #include "console.h"
+#include "ggzserver.h"
 #include "meta.h"
 #include "plrhand.h"
 #include "srv_main.h"
@@ -130,6 +131,10 @@ static void send_lanserver_response(void);
 
 static bool no_input = FALSE;
 
+/* Avoid compiler warning about defined, but unused function
+ * by defining it only when needed */
+#if defined(HAVE_LIBREADLINE) || \
+    (!defined(SOCKET_ZERO_ISNT_STDIN) && !defined(HAVE_LIBREADLINE))  
 /*****************************************************************************
   This happens if you type an EOF character with nothing on the current line.
 *****************************************************************************/
@@ -141,9 +146,11 @@ static void handle_stdin_close(void)
 #ifndef SOCKET_ZERO_ISNT_STDIN
   freelog(LOG_NORMAL, _("Server cannot read standard input. Ignoring input."));
   no_input = TRUE;
-#endif
+#endif /* SOCKET_ZERO_ISNT_STDIN */
 }
 
+#endif /* HAVE_LIBREADLINE || (!SOCKET_ZERO_ISNT_STDIN && !HAVE_LIBREADLINE) */
+
 #ifdef HAVE_LIBREADLINE
 /****************************************************************************/
 
@@ -195,12 +202,11 @@ void close_connection(struct connection *pconn)
     free_timer(timer);
   }
   assert(timer_list_size(pconn->server.ping_timers) == 0);
-  timer_list_unlink_all(pconn->server.ping_timers);
+  timer_list_free(pconn->server.ping_timers);
 
   /* safe to do these even if not in lists: */
-  conn_list_unlink(&game.all_connections, pconn);
-  conn_list_unlink(&game.est_connections, pconn);
-  conn_list_unlink(&game.game_connections, pconn);
+  conn_list_unlink(game.all_connections, pconn);
+  conn_list_unlink(game.est_connections, pconn);
 
   pconn->player = NULL;
   pconn->access_level = ALLOW_NONE;
@@ -213,22 +219,19 @@ void close_connection(struct connection *pconn)
 void close_connections_and_socket(void)
 {
   int i;
-  lsend_packet_server_shutdown(&game.all_connections);
+  lsend_packet_server_shutdown(game.all_connections);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     if(connections[i].used) {
       close_connection(&connections[i]);
     }
-    conn_list_unlink_all(&connections[i].self);
+    conn_list_unlink_all(connections[i].self);
+    conn_list_free(connections[i].self);
   }
 
   /* Remove the game connection lists and make sure they are empty. */
-  assert(conn_list_size(&game.all_connections) == 0);
-  conn_list_unlink_all(&game.all_connections);
-  assert(conn_list_size(&game.est_connections) == 0);
-  conn_list_unlink_all(&game.est_connections);
-  assert(conn_list_size(&game.game_connections) == 0);
-  conn_list_unlink_all(&game.game_connections);
+  conn_list_free(game.all_connections);
+  conn_list_free(game.est_connections);
 
   my_closesocket(sock);
   my_closesocket(socklan);
@@ -258,6 +261,30 @@ static void close_socket_callback(struct connection *pc)
   close_connection(pc);
 }
 
+/****************************************************************************
+  If a connection lags too much this function is called and we try to cut
+  it.
+****************************************************************************/
+static void cut_lagging_connection(struct connection *pconn)
+{
+  if (game.info.tcptimeout != 0
+      && pconn->last_write
+      && conn_list_size(game.all_connections) > 1
+      && pconn->access_level != ALLOW_HACK
+      && read_timer_seconds(pconn->last_write) > game.info.tcptimeout) {
+    /* Cut the connections to players who lag too much.  This
+     * usually happens because client animation slows the client
+     * too much and it can't keep up with the server.  We don't
+     * cut HACK connections, or cut in single-player games, since
+     * it wouldn't help the game progress.  For other connections
+     * the best thing to do when they lag too much is to be
+     * disconnected and reconnect. */
+    freelog(LOG_ERROR, "connection (%s) cut due to lagging player",
+	    conn_description(pconn));
+    close_socket_callback(pconn);
+  }
+}
+
 /****************************************************************************
   Attempt to flush all information in the send buffers for upto 'netwait'
   seconds.
@@ -273,7 +300,7 @@ void flush_packets(void)
   (void) time(&start);
 
   for(;;) {
-    tv.tv_sec=(game.netwait - (time(NULL) - start));
+    tv.tv_sec = (game.info.netwait - (time(NULL) - start));
     tv.tv_usec=0;
 
     if (tv.tv_sec < 0)
@@ -296,7 +323,7 @@ void flush_packets(void)
       return;
     }
 
-    if(select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
+    if(my_select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
       return;
     }
 
@@ -304,7 +331,7 @@ void flush_packets(void)
       struct connection *pconn = &connections[i];
       if(pconn->used) {
         if(FD_ISSET(pconn->sock, &exceptfs)) {
-	  freelog(LOG_NORMAL, "cut connection %s due to exception data",
+	  freelog(LOG_ERROR, "connection (%s) cut due to exception data",
 		  conn_description(pconn));
 	  close_socket_callback(pconn);
         } else {
@@ -312,12 +339,7 @@ void flush_packets(void)
 	    if(FD_ISSET(pconn->sock, &writefs)) {
 	      flush_connection_send_buffer_all(pconn);
 	    } else {
-	      if(game.tcptimeout != 0 && pconn->last_write != 0
-		 && (time(NULL)>pconn->last_write + game.tcptimeout)) {
-	        freelog(LOG_NORMAL, "cut connection %s due to lagging player",
-			conn_description(pconn));
-		close_socket_callback(pconn);
-	      }
+	      cut_lagging_connection(pconn);
 	    }
 	  }
         }
@@ -326,25 +348,88 @@ void flush_packets(void)
   }
 }
 
+struct packet_to_handle {
+  void *data;
+  enum packet_type type;
+};
+
+/*****************************************************************************
+  Simplify a loop by wrapping get_packet_from_connection.
+*****************************************************************************/
+static bool get_packet(struct connection *pconn, 
+                       struct packet_to_handle *ppacket)
+{
+  bool got_packet;
+
+  ppacket->data = get_packet_from_connection(pconn, &ppacket->type, 
+                                             &got_packet);
+  return got_packet;
+}
+
+/*****************************************************************************
+  Handle all incoming packets on a client connection.
+  Precondition - we have read_socket_data.
+  Postcondition - there are no more packets to handle on this connection.
+*****************************************************************************/
+static void incoming_client_packets(struct connection *pconn) 
+{
+  struct packet_to_handle packet;
+#if PROCESSING_TIME_STATISTICS
+  struct timer *request_time = NULL;
+#endif
+
+  while (get_packet(pconn, &packet)) {
+    bool command_ok;
+    int request_id;
+
+#if PROCESSING_TIME_STATISTICS
+    request_time = renew_timer_start(request_time, TIMER_USER, TIMER_ACTIVE);
+#endif
+
+    request_id = pconn->server.last_request_id_seen
+      = get_next_request_id(pconn->server.last_request_id_seen);
+
+    connection_do_buffer(pconn);
+    start_processing_request(pconn, pconn->server.last_request_id_seen);
+
+    command_ok = server_packet_input(pconn, packet.data, packet.type);
+    free(packet.data);
+
+    finish_processing_request(pconn);
+    connection_do_unbuffer(pconn);
+
+#if PROCESSING_TIME_STATISTICS
+    freelog(LOG_VERBOSE,
+            "processed request %d in %gms", request_id, 
+            read_timer_seconds(request_time) * 1000.0);
+#endif
+
+    if (!command_ok) {
+      close_connection(pconn);
+    }
+  }
+
+#if PROCESSING_TIME_STATISTICS
+  free_timer(request_time);
+#endif
+}
+
+
 /*****************************************************************************
 Get and handle:
 - new connections,
 - input from connections,
 - input from server operator in stdin
-Returns:
-  0 if went past end-of-turn timeout
-  2 if force_end_of_sniff found to be set
-  1 otherwise (got and processed something?)
+
 This function also handles prompt printing, via the con_prompt_*
 functions.  That is, other functions should not need to do so.  --dwp
 *****************************************************************************/
-int sniff_packets(void)
+enum server_events server_sniff_all_input(void)
 {
   int i;
   int max_desc;
   fd_set readfs, writefs, exceptfs;
   struct timeval tv;
-  static int year;
 #ifdef SOCKET_ZERO_ISNT_STDIN
   char *bufptr;    
 #endif
@@ -355,9 +440,10 @@ int sniff_packets(void)
   {
     if (!no_input && !readline_initialized) {
       char *home_dir = user_home_dir();
+
       if (home_dir) {
-	history_file =
-	  fc_malloc(strlen(home_dir) + 1 + strlen(HISTORY_FILENAME) + 1);
+	history_file
+	  = fc_malloc(strlen(home_dir) + 1 + strlen(HISTORY_FILENAME) + 1);
 	if (history_file) {
 	  strcpy(history_file, home_dir);
 	  strcat(history_file, "/");
@@ -368,7 +454,8 @@ int sniff_packets(void)
       }
 
       rl_initialize();
-      rl_callback_handler_install((char *) "> ", handle_readline_input_callback);
+      rl_callback_handler_install((char *) "> ",
+				  handle_readline_input_callback);
       rl_attempted_completion_function = freeciv_completion;
 
       readline_initialized = TRUE;
@@ -377,39 +464,35 @@ int sniff_packets(void)
   }
 #endif /* HAVE_LIBREADLINE */
 
-  if(year!=game.year) {
-    if (server_state == RUN_GAME_STATE) year=game.year;
-  }
-  if (game.timeout == 0) {
-    /* Just in case someone sets timeout we keep game.turn_start updated */
-    game.turn_start = time(NULL);
-  }
-  
-  while(TRUE) {
+  while (TRUE) {
     con_prompt_on();		/* accepting new input */
     
-    if(force_end_of_sniff) {
+    if (force_end_of_sniff) {
       force_end_of_sniff = FALSE;
       con_prompt_off();
-      return 2;
+      return S_E_FORCE_END_OF_SNIFF;
     }
 
     get_lanserver_announcement();
 
-    /* end server if no players for 'srvarg.quitidle' seconds */
-    if (srvarg.quitidle != 0 && server_state != PRE_GAME_STATE) {
+    /* end server if no players for 'srvarg.quitidle' seconds,
+     * but only if at least one player has previously connected. */
+    if (srvarg.quitidle != 0) {
       static time_t last_noplayers;
-      if(conn_list_size(&game.est_connections) == 0) {
+      static bool connections;
+
+      if (conn_list_size(game.est_connections) > 0) {
+	connections = TRUE;
+      }
+      if (connections && conn_list_size(game.est_connections) == 0) {
 	if (last_noplayers != 0) {
-	  if (time(NULL)>last_noplayers + srvarg.quitidle) {
-	    if (srvarg.exit_on_end) {
-	      save_game_auto();
-	    }
+	  if (time(NULL) > last_noplayers + srvarg.quitidle) {
+	    save_game_auto("Lost all connections");
 	    set_meta_message_string("restarting for lack of players");
 	    freelog(LOG_NORMAL, get_meta_message_string());
 	    (void) send_server_info_to_metaserver(META_INFO);
 
-            server_state = GAME_OVER_STATE;
+            set_server_state(S_S_OVER);
             force_end_of_sniff = TRUE;
             conn_list_iterate(game.est_connections, pconn) {
               lost_connection_to_client(pconn);
@@ -420,6 +503,9 @@ int sniff_packets(void)
 	      /* No need for anything more; just quit. */
 	      server_quit();
 	    }
+
+            /* Do not restart before someone has connected and left again */
+            connections = FALSE;
 	  }
 	} else {
           char buf[256];
@@ -438,22 +524,22 @@ int sniff_packets(void)
     }
 
     /* Pinging around for statistics */
-    if (time(NULL) > (game.last_ping + game.pingtime)) {
+    if (time(NULL) > (game.last_ping + game.info.pingtime)) {
       /* send data about the previous run */
       send_ping_times_to_all();
 
       conn_list_iterate(game.all_connections, pconn) {
 	if ((timer_list_size(pconn->server.ping_timers) > 0
-	     &&
-	     read_timer_seconds(timer_list_get(pconn->server.ping_timers, 0))
-	     > game.pingtimeout) || pconn->ping_time > game.pingtimeout) {
+	     && read_timer_seconds(timer_list_get(pconn->server.ping_timers, 0))
+	        > game.info.pingtimeout) 
+            || pconn->ping_time > game.info.pingtimeout) {
 	  /* cut mute players, except for hack-level ones */
 	  if (pconn->access_level == ALLOW_HACK) {
-	    freelog(LOG_NORMAL,
-		    "ignoring ping timeout to hack-level connection %s",
+	    freelog(LOG_ERROR,
+		    "connection (%s) [hack-level] ping timeout ignored",
 		    conn_description(pconn));
 	  } else {
-	    freelog(LOG_NORMAL, "cut connection %s due to ping timeout",
+	    freelog(LOG_ERROR, "connection (%s) cut due to ping timeout",
 		    conn_description(pconn));
 	    close_socket_callback(pconn);
 	  }
@@ -472,13 +558,13 @@ int sniff_packets(void)
     } conn_list_iterate_end
 
     /* Don't wait if timeout == -1 (i.e. on auto games) */
-    if (server_state != PRE_GAME_STATE && game.timeout == -1) {
+    if (S_S_INITIAL != server_state() && game.info.timeout == -1) {
       (void) send_server_info_to_metaserver(META_REFRESH);
-      return 0;
+      return S_E_END_OF_TURN_TIMEOUT;
     }
 
-    tv.tv_sec=1;
-    tv.tv_usec=0;
+    tv.tv_sec = 1;
+    tv.tv_usec = 0;
 
     MY_FD_ZERO(&readfs);
     MY_FD_ZERO(&writefs);
@@ -487,51 +573,73 @@ int sniff_packets(void)
     if (!no_input) {
 #ifdef SOCKET_ZERO_ISNT_STDIN
       my_init_console();
-#else
+#else /* SOCKET_ZERO_ISNT_STDIN */
 #   if !defined(__VMS)
       FD_SET(0, &readfs);
-#   endif	
-#endif
+#   endif /* VMS */	
+#endif /* SOCKET_ZERO_ISNT_STDIN */
     }
 
-    FD_SET(sock, &readfs);
-    FD_SET(sock, &exceptfs);
-    max_desc=sock;
+    if (with_ggz) {
+#ifdef GGZ_SERVER
+      int ggz_sock = get_ggz_socket();
 
-    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
-      if(connections[i].used) {
+      FD_SET(ggz_sock, &readfs);
+      max_desc = MAX(sock, ggz_sock);
+#endif /* GGZ_SERVER */
+    } else {
+      FD_SET(sock, &readfs);
+      FD_SET(sock, &exceptfs);
+      max_desc = sock;
+    }
+
+    for (i = 0; i < MAX_NUM_CONNECTIONS; i++) {
+      if (connections[i].used) {
 	FD_SET(connections[i].sock, &readfs);
 	if(connections[i].send_buffer->ndata > 0) {
 	  FD_SET(connections[i].sock, &writefs);
 	}
 	FD_SET(connections[i].sock, &exceptfs);
-        max_desc=MAX(connections[i].sock, max_desc);
+        max_desc = MAX(connections[i].sock, max_desc);
       }
     }
     con_prompt_off();		/* output doesn't generate a new prompt */
 
-    if(select(max_desc+1, &readfs, &writefs, &exceptfs, &tv)==0) { /* timeout */
+    if (my_select(max_desc + 1, &readfs, &writefs, &exceptfs, &tv) == 0) {
+      /* timeout */
       (void) send_server_info_to_metaserver(META_REFRESH);
-      if(game.timeout != 0
-	&& (time(NULL)>game.turn_start + game.timeout)
-	&& (server_state == RUN_GAME_STATE)){
+      if (game.info.timeout > 0
+	  && S_S_RUNNING == server_state()
+	  && game.phase_timer
+	  && (read_timer_seconds(game.phase_timer)
+	      > game.info.seconds_to_phasedone)) {
 	con_prompt_off();
-	return 0;
+	return S_E_END_OF_TURN_TIMEOUT;
       }
 
       if (!no_input) {
 #if defined(__VMS)
-      {
-	struct { short numchars; char firstchar; char reserved; int reserved2; } ttchar;
-	unsigned long status;
-	status = sys$qiow(EFN$C_ENF,tt_chan,IO$_SENSEMODE|IO$M_TYPEAHDCNT,0,0,0,
-			  &ttchar,sizeof(ttchar),0,0,0,0);
-	if (!$VMS_STATUS_SUCCESS(status)) lib$stop(status);
-	if (ttchar.numchars)
-	  FD_SET(0, &readfs);
-	else
-	  continue;
-      }
+	{
+	  struct {
+	    short numchars;
+	    char firstchar;
+	    char reserved;
+	    int reserved2;
+	  } ttchar;
+	  unsigned long status;
+
+	  status = sys$qiow(EFN$C_ENF, tt_chan,
+			    IO$_SENSEMODE | IO$M_TYPEAHDCNT, 0, 0, 0,
+			    &ttchar, sizeof(ttchar), 0, 0, 0, 0);
+	  if (!$VMS_STATUS_SUCCESS(status)) {
+	    lib$stop(status);
+	  }
+	  if (ttchar.numchars) {
+	    FD_SET(0, &readfs);
+	  } else {
+	    continue;
+	  }
+	}
 #else  /* !__VMS */
 #ifndef SOCKET_ZERO_ISNT_STDIN
       continue;
@@ -539,28 +647,39 @@ int sniff_packets(void)
 #endif /* !__VMS */
       }
     }
-    if (game.timeout == 0) {
-      /* Just in case someone sets timeout we keep game.turn_start updated */
-      game.turn_start = time(NULL);
-    }
 
-    if(FD_ISSET(sock, &exceptfs)) {	     /* handle Ctrl-Z suspend/resume */
-      continue;
-    }
-    if(FD_ISSET(sock, &readfs)) {	     /* new players connects */
-      freelog(LOG_VERBOSE, "got new connection");
-      if(server_accept_connection(sock)==-1) {
-	freelog(LOG_ERROR, "failed accepting connection");
+    if (!with_ggz) { /* No listening socket when using GGZ. */
+      if (FD_ISSET(sock, &exceptfs)) {	     /* handle Ctrl-Z suspend/resume */
+	continue;
+      }
+      if(FD_ISSET(sock, &readfs)) {	     /* new players connects */
+	freelog(LOG_VERBOSE, "got new connection");
+	if(server_accept_connection(sock)==-1) {
+	  freelog(LOG_ERROR, "failed accepting connection");
+	}
       }
     }
-    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {   /* check for freaky players */
+    for (i = 0; i < MAX_NUM_CONNECTIONS; i++) {
+      /* check for freaky players */
       struct connection *pconn = &connections[i];
-      if(pconn->used && FD_ISSET(pconn->sock, &exceptfs)) {
- 	freelog(LOG_ERROR, "cut connection %s due to exception data",
+
+      if (pconn->used && FD_ISSET(pconn->sock, &exceptfs)) {
+ 	freelog(LOG_ERROR, "connection (%s) cut due to exception data",
 		conn_description(pconn));
 	close_socket_callback(pconn);
       }
     }
+#ifdef GGZ_SERVER
+    if (with_ggz) {
+      /* This is intentionally after all the player socket handling because
+       * it may cut a client. */
+      int ggz_sock = get_ggz_socket();
+
+      if (FD_ISSET(ggz_sock, &readfs)) {
+	input_from_ggz(ggz_sock);
+      }
+    }
+#endif /* GGZ_SERVER */
     
 #ifdef SOCKET_ZERO_ISNT_STDIN
     if (!no_input && (bufptr = my_read_console())) {
@@ -581,7 +700,7 @@ int sniff_packets(void)
       continue;
 #else  /* !HAVE_LIBREADLINE */
       ssize_t didget;
-      char buf[BUF_SIZE+1];
+      char buf[BUF_SIZE + 1];
       char *buf_internal;
       
       didget = read(0, buf, BUF_SIZE);
@@ -590,93 +709,41 @@ int sniff_packets(void)
 	didget = 0; /* Avoid buffer underrun below. */
       }
 
-      *(buf+didget)='\0';
+      *(buf + didget)='\0';
       con_prompt_enter();	/* will need a new prompt, regardless */
       buf_internal = local_to_internal_string_malloc(buf);
       handle_stdin_input(NULL, buf_internal, FALSE);
       free(buf_internal);
 #endif /* !HAVE_LIBREADLINE */
-    }
-    else
+    } else
 #endif /* !SOCKET_ZERO_ISNT_STDIN */
      
     {                             /* input from a player */
-      for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
+      for(i = 0; i < MAX_NUM_CONNECTIONS; i++) {
   	struct connection *pconn = &connections[i];
-	if(pconn->used && FD_ISSET(pconn->sock, &readfs)) {
-	  if(read_socket_data(pconn->sock, pconn->buffer)>=0) {
-	    void *packet;
-	    enum packet_type type;
-	    bool result;
 
-	    while (TRUE) {
-	      packet = get_packet_from_connection(pconn, &type, &result);
-	      if (result) {
-		bool command_ok;
+        if (!pconn->used || !FD_ISSET(pconn->sock, &readfs)) {
+          continue;
+	}
 
-		pconn->server.last_request_id_seen =
-		    get_next_request_id(pconn->server.
-					last_request_id_seen);
-#if PROCESSING_TIME_STATISTICS
-		{
-		  int err;
-		  struct timeval start, end;
-		  struct timezone tz;
-		  long us;
-
-		  err = gettimeofday(&start, &tz);
-		  assert(!err);
-#endif
-		connection_do_buffer(pconn);
-		start_processing_request(pconn,
-					 pconn->server.
-					 last_request_id_seen);
-		command_ok = handle_packet_input(pconn, packet, type);
-		if (packet) {
-		  free(packet);
-		  packet = NULL;
-		}
-
-		finish_processing_request(pconn);
-		connection_do_unbuffer(pconn);
-		if (!command_ok) {
-		  close_connection(pconn);
-		}
-#if PROCESSING_TIME_STATISTICS
-		  err = gettimeofday(&end, &tz);
-		  assert(!err);
-
-		  us = (end.tv_sec - start.tv_sec) * 1000000 +
-		      end.tv_usec - start.tv_usec;
-
-		  freelog(LOG_NORMAL,
-			  "processed request %d in %ld.%03ldms",
-			  pconn->server.last_request_id_seen, us / 1000,
-			  us % 1000);
-                }
-#endif
-	      } else {
-		break;
-	      }
-	    }
-	  } else {
-	    close_socket_callback(pconn);
-	  }
+        if (read_socket_data(pconn->sock, pconn->buffer) >= 0) {
+          /* We read packets; now handle them. */
+          incoming_client_packets(pconn);
+	} else {
+          /* Read failure; the connection is closed. */
+	  close_socket_callback(pconn);
 	}
       }
 
-      for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
+      for (i = 0; i < MAX_NUM_CONNECTIONS; i++) {
         struct connection *pconn = &connections[i];
-        if(pconn->used && pconn->send_buffer && pconn->send_buffer->ndata > 0) {
-	  if(FD_ISSET(pconn->sock, &writefs)) {
+
+        if (pconn->used && pconn->send_buffer
+	    && pconn->send_buffer->ndata > 0) {
+	  if (FD_ISSET(pconn->sock, &writefs)) {
 	    flush_connection_send_buffer_all(pconn);
 	  } else {
-	    if(game.tcptimeout != 0 && pconn->last_write != 0
-	       && (time(NULL)>pconn->last_write + game.tcptimeout)) {
-	      freelog(LOG_NORMAL, "cut connection %s due to lagging player",
-		      conn_description(pconn));
-	      close_socket_callback(pconn);
-	    }
+	    cut_lagging_connection(pconn);
 	  }
         }
       }
@@ -685,10 +752,13 @@ int sniff_packets(void)
   }
   con_prompt_off();
 
-  if (game.timeout != 0 && (time(NULL) > game.turn_start + game.timeout)) {
-    return 0;
+  if (game.info.timeout > 0
+      && S_S_RUNNING == server_state()
+      && game.phase_timer
+      && read_timer_seconds(game.phase_timer) > game.info.seconds_to_phasedone) {
+    return S_E_END_OF_TURN_TIMEOUT;
   }
-  return 1;
+  return S_E_OTHERWISE;
 }
 
 /********************************************************************
@@ -727,16 +797,11 @@ static int server_accept_connection(int sockfd)
 {
   /* This used to have size_t for some platforms.  If this is necessary
    * it should be done with a configure check not a platform check. */
-#ifdef HAVE_SOCKLEN_T
   socklen_t fromlen;
-#else
-  int fromlen;
-#endif
 
   int new_sock;
   union my_sockaddr fromend;
   struct hostent *from;
-  int i;
 
   fromlen = sizeof(fromend);
 
@@ -745,12 +810,28 @@ static int server_accept_connection(int sockfd)
     return -1;
   }
 
-  my_nonblock(new_sock);
-
   from =
       gethostbyaddr((char *) &fromend.sockaddr_in.sin_addr,
 		    sizeof(fromend.sockaddr_in.sin_addr), AF_INET);
 
+  return server_make_connection(new_sock,
+				(from ? from->h_name
+				 : inet_ntoa(fromend.sockaddr_in.sin_addr)),
+				inet_ntoa(fromend.sockaddr_in.sin_addr));
+}
+
+/********************************************************************
+  Server accepts connection from client:
+  Low level socket stuff, and basic-initialize the connection struct.
+  Returns 0 on success, -1 on failure (bad accept(), or too many
+  connections).
+********************************************************************/
+int server_make_connection(int new_sock, const char *client_addr, const char *client_ip)
+{
+  int i;
+
+  my_nonblock(new_sock);
+
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     struct connection *pconn = &connections[i];
     if (!pconn->used) {
@@ -762,6 +843,7 @@ static int server_accept_connection(int sockfd)
       pconn->access_level = access_level_for_next_connection();
       pconn->delayed_disconnect = FALSE;
       pconn->notify_of_writable_data = NULL;
+      pconn->is_server = TRUE;
       pconn->server.currently_processed_request_id = 0;
       pconn->server.last_request_id_seen = 0;
       pconn->server.auth_tries = 0;
@@ -769,19 +851,16 @@ static int server_accept_connection(int sockfd)
       pconn->server.status = AS_NOT_ESTABLISHED;
       pconn->server.ping_timers =
 	  fc_malloc(sizeof(*pconn->server.ping_timers));
-      timer_list_init(pconn->server.ping_timers);
+      pconn->server.ping_timers = timer_list_new();
       pconn->ping_time = -1.0;
       pconn->incoming_packet_notify = NULL;
       pconn->outgoing_packet_notify = NULL;
 
       sz_strlcpy(pconn->username, makeup_connection_name(&pconn->id));
-      sz_strlcpy(pconn->addr,
-		 (from ? from->
-		  h_name : inet_ntoa(fromend.sockaddr_in.sin_addr)));
-      sz_strlcpy(pconn->server.ipaddr,
-                 inet_ntoa(fromend.sockaddr_in.sin_addr));
+      sz_strlcpy(pconn->addr, client_addr);
+      sz_strlcpy(pconn->server.ipaddr, client_ip);
 
-      conn_list_insert_back(&game.all_connections, pconn);
+      conn_list_append(game.all_connections, pconn);
   
       freelog(LOG_VERBOSE, "connection (%s) from %s (%s)", 
               pconn->username, pconn->addr, pconn->server.ipaddr);
@@ -877,15 +956,14 @@ void init_connections(void)
 {
   int i;
 
-  conn_list_init(&game.all_connections);
-  conn_list_init(&game.est_connections);
-  conn_list_init(&game.game_connections);
+  game.all_connections = conn_list_new();
+  game.est_connections = conn_list_new();
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) { 
     struct connection *pconn = &connections[i];
     pconn->used = FALSE;
-    conn_list_init(&pconn->self);
-    conn_list_insert(&pconn->self, pconn);
+    pconn->self = conn_list_new();
+    conn_list_prepend(pconn->self, pconn);
   }
 #if defined(__VMS)
   {
@@ -894,7 +972,7 @@ void init_connections(void)
     status = sys$assign(&tt_desc,&tt_chan,0,0);
     if (!$VMS_STATUS_SUCCESS(status)) lib$stop(status);
   }
-#endif
+#endif /* VMS */
 }
 
 /**************************************************************************
@@ -931,7 +1009,7 @@ static void ping_connection(struct connection *pconn)
   freelog(LOG_DEBUG, "sending ping to %s (open=%d)",
 	  conn_description(pconn),
 	  timer_list_size(pconn->server.ping_timers));
-  timer_list_insert_back(pconn->server.ping_timers,
+  timer_list_append(pconn->server.ping_timers,
 			 new_timer_start(TIMER_USER, TIMER_ACTIVE));
   send_packet_conn_ping(pconn);
 }
@@ -966,31 +1044,18 @@ static void send_ping_times_to_all(void)
   int i;
 
   i = 0;
-  conn_list_iterate(game.game_connections, pconn) {
-    if (!pconn->used) {
-      continue;
-    }
-    i++;
-  } conn_list_iterate_end;
-
-  packet.connections = i;
-  packet.old_connections = MIN(i, MAX_NUM_PLAYERS);
-
-  i = 0;
-  conn_list_iterate(game.game_connections, pconn) {
+  conn_list_iterate(game.est_connections, pconn) {
     if (!pconn->used) {
       continue;
     }
     assert(i < ARRAY_SIZE(packet.conn_id));
     packet.conn_id[i] = pconn->id;
     packet.ping_time[i] = pconn->ping_time;
-    if (i < packet.old_connections) {
-      packet.old_conn_id[i] = pconn->id;
-      packet.old_ping_time[i] = pconn->ping_time;
-    }
     i++;
   } conn_list_iterate_end;
-  lsend_packet_conn_ping_info(&game.est_connections, &packet);
+  packet.connections = i;
+
+  lsend_packet_conn_ping_info(game.est_connections, &packet);
 }
 
 /********************************************************************
@@ -1005,6 +1070,10 @@ static void get_lanserver_announcement(void)
   fd_set readfs, exceptfs;
   struct timeval tv;
 
+  if (with_ggz) {
+    return;
+  }
+
   FD_ZERO(&readfs);
   FD_ZERO(&exceptfs);
   FD_SET(socklan, &exceptfs);
@@ -1013,7 +1082,7 @@ static void get_lanserver_announcement(void)
   tv.tv_sec = 0;
   tv.tv_usec = 0;
 
-  while (select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
+  while (my_select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
     if (errno != EINTR) {
       freelog(LOG_ERROR, "select failed: %s", mystrerror());
       return;
@@ -1054,7 +1123,9 @@ static void send_lanserver_response(void)
   int socksend, setting = 1;
   const char *group;
   size_t size;
+#ifndef HAVE_WINSOCK
   unsigned char ttl;
+#endif
 
   /* Create a socket to broadcast to client. */
   if ((socksend = socket(AF_INET,SOCK_DGRAM, 0)) < 0) {
@@ -1069,6 +1140,9 @@ static void send_lanserver_response(void)
   addr.sockaddr_in.sin_addr.s_addr = inet_addr(group);
   addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT + 1);
 
+/* this setsockopt call fails on Windows 98, so we stick with the default
+ * value of 1 on Windows, which should be fine in most cases */
+#ifndef HAVE_WINSOCK
   /* Set the Time-to-Live field for the packet.  */
   ttl = SERVER_LAN_TTL;
   if (setsockopt(socksend, IPPROTO_IP, IP_MULTICAST_TTL, 
@@ -1076,6 +1150,7 @@ static void send_lanserver_response(void)
     freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
     return;
   }
+#endif
 
   if (setsockopt(socksend, SOL_SOCKET, SO_BROADCAST, 
                  (const char*)&setting, sizeof(setting))) {
@@ -1091,17 +1166,17 @@ static void send_lanserver_response(void)
   my_snprintf(version, sizeof(version), "%d.%d.%d%s",
               MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, VERSION_LABEL);
 
-  switch (server_state) {
-  case PRE_GAME_STATE:
+  switch (server_state()) {
+  case S_S_INITIAL:
     my_snprintf(status, sizeof(status), "Pregame");
     break;
-  case RUN_GAME_STATE:
+  case S_S_RUNNING:
     my_snprintf(status, sizeof(status), "Running");
     break;
-  case GAME_OVER_STATE:
+  case S_S_OVER:
     my_snprintf(status, sizeof(status), "Game over");
     break;
-  default:
+  case S_S_GENERATING_WAITING:
     my_snprintf(status, sizeof(status), "Waiting");
   }
 
