@@ -40,6 +40,15 @@
    qsort-type comparison function (but the hash table does not make
    use of the ordering information if the return value is non-zero).
 
+   Some constructors also accept following functions to be registered:
+
+     free_key_func: This is called when hash no longer needs key construct.
+   Note that one key construct gets freed even when it is replaced with
+   another that is considered identical by fcmp().
+
+     free_data_func: This is called when hash no longer needs data construct. 
+
+
    Implementation uses closed hashing with simple collision resolution.
    Deleted elements are marked as DELETED rather than UNUSED so that
    lookups on previously added entries work properly.
@@ -61,20 +70,13 @@
      the key is available to be freed (or can just be freed as part of
      deleting the "object").  (This is done in registry hsec, but the
      allocation is still via sbuffer.)
+   - Specify key free function pointers upon hash table creation which
+     handle the freeing.
    - Keep another hash from object pointers to key pointers?!  Seems a
      bit too perverse...
 
-   Possible implementation changes to avoid/reduce this problem:
-
-   - As noted above, could restrict key type (e.g., strings), or pass
-     in more function pointers and user data to allow duplicating and
-     freeing keys within hash table code.
-   - Have some mechanism to allow external access to the key pointers
-     in the hash table, especially when deleting/replacing elements
-     -- and when freeing the hash table?
-
-   Actually, there are some potential problems with memory management
-   of the user-data pointers too:
+   There are some potential problems with memory management of the
+   user-data pointers too:
    
    - If may have multiple keys pointing to the same data, cannot just
      free the data when deleted/replaced from hash.  (Eg, tilespec;
@@ -86,6 +88,7 @@
 
    - Don't have hash table as only access to user-data.
    - Allocate user-data using sbuffer.
+   - Specify user-data free function pointers upon hash table creation.
 
    
 ***************************************************************************/
@@ -121,6 +124,8 @@ struct hash_table {
   struct hash_bucket *buckets;
   hash_val_fn_t fval;
   hash_cmp_fn_t fcmp;
+  hash_free_fn_t free_key_func;
+  hash_free_fn_t free_data_func;
   unsigned int num_buckets;
   unsigned int num_entries;	/* does not included deleted entries */
   unsigned int num_deleted;
@@ -149,6 +154,8 @@ static void zero_htable(struct hash_table *h)
   h->buckets = NULL;
   h->fval = NULL;
   h->fcmp = NULL;
+  h->free_key_func = NULL;
+  h->free_data_func = NULL;
   h->num_buckets = h->num_entries = h->num_deleted = 0;
   h->frozen = FALSE;
 }
@@ -169,10 +176,10 @@ unsigned int hash_fval_int(const void *vkey, unsigned int num_buckets)
 **************************************************************************/
 int hash_fcmp_int(const void *vkey1, const void *vkey2)
 {
-  const int key1 = *(const int*)vkey1;
-  const int key2 = *(const int*)vkey2;
+  const int *key1 = vkey1, *key2 = vkey2;
+
   /* avoid overflow issues: */
-  return (key1 < key2) ? -1 : (key1 > key2) ? 1 : 0;
+  return (*key1 < *key2) ? -1 : (*key1 > *key2) ? 1 : 0;
 }
 
 
@@ -183,7 +190,7 @@ int hash_fcmp_int(const void *vkey1, const void *vkey2)
 unsigned int hash_fval_string(const void *vkey, unsigned int num_buckets)
 {
   const char *key = (const char*)vkey;
-  unsigned long result=0;
+  unsigned long result = 0;
 
   for (; *key != '\0'; key++) {
     result *= 5; 
@@ -256,8 +263,8 @@ static const unsigned long ht_sizes[] =
 **************************************************************************/
 static unsigned int calc_appropriate_nbuckets(unsigned int num_entries)
 {
-  const unsigned long *pframe=&ht_sizes[0], *pmid;
-  int fsize=NSIZES-1, lpart;
+  const unsigned long *pframe = &ht_sizes[0], *pmid;
+  int fsize = NSIZES - 1, lpart;
 
   num_entries <<= 1; /* breathing room */
 
@@ -275,10 +282,15 @@ static unsigned int calc_appropriate_nbuckets(unsigned int num_entries)
 }
 
 /**************************************************************************
-  Internal constructor, specifying exact number of buckets:
+  Internal constructor, specifying exact number of buckets.
+  Allows to specify functions to free the memory allocated for the key and
+  user-data that get called when removing the bucket from the hash table or
+  changing key/user-data values.
 **************************************************************************/
 static struct hash_table *hash_new_nbuckets(hash_val_fn_t fval,
 					    hash_cmp_fn_t fcmp,
+					    hash_free_fn_t free_key_func,
+					    hash_free_fn_t free_data_func,
 					    unsigned int nbuckets)
 {
   struct hash_table *h;
@@ -293,23 +305,55 @@ static struct hash_table *hash_new_nbuckets(hash_val_fn_t fval,
   h->num_entries = 0;
   h->fval = fval;
   h->fcmp = fcmp;
+  h->free_key_func = free_key_func;
+  h->free_data_func = free_data_func;
 
   h->buckets = (struct hash_bucket *)
       	       fc_malloc(h->num_buckets*sizeof(struct hash_bucket));
 
-  for(i=0; i<h->num_buckets; i++) {
+  for(i = 0; i < h->num_buckets; i++) {
     zero_hbucket(&h->buckets[i]);
   }
   return h;
 }
 
+/**************************************************************************
+  Constructor specifying number of entries.
+  Allows to specify functions to free the memory allocated for the key and
+  user-data that get called when removing the bucket from the hash table or
+  changing key/user-data values.
+**************************************************************************/
+struct hash_table *hash_new_nentries_full(hash_val_fn_t fval,
+					  hash_cmp_fn_t fcmp,
+					  hash_free_fn_t free_key_func,
+					  hash_free_fn_t free_data_func,
+					  unsigned int nentries)
+{
+  return hash_new_nbuckets(fval, fcmp, free_key_func, free_data_func,
+			   calc_appropriate_nbuckets(nentries));
+}
+
 /**************************************************************************
   Constructor specifying number of entries:
 **************************************************************************/
 struct hash_table *hash_new_nentries(hash_val_fn_t fval, hash_cmp_fn_t fcmp,
 				     unsigned int nentries)
 {
-  return hash_new_nbuckets(fval, fcmp, calc_appropriate_nbuckets(nentries));
+  return hash_new_nentries_full(fval, fcmp, NULL, NULL, nentries);
+}
+
+/**************************************************************************
+  Constructor with unspecified number of entries.
+  Allows to specify functions to free the memory allocated for the key and
+  user-data that get called when removing the bucket from the hash table or
+  changing key/user-data values.
+**************************************************************************/
+struct hash_table *hash_new_full(hash_val_fn_t fval,
+			         hash_cmp_fn_t fcmp,
+			         hash_free_fn_t free_key_func,
+			         hash_free_fn_t free_data_func)
+{
+  return hash_new_nentries_full(fval, fcmp, free_key_func, free_data_func, 0);
 }
 
 /**************************************************************************
@@ -317,7 +361,7 @@ struct hash_table *hash_new_nentries(hash_val_fn_t fval, hash_cmp_fn_t fcmp,
 **************************************************************************/
 struct hash_table *hash_new(hash_val_fn_t fval, hash_cmp_fn_t fcmp)
 {
-  return hash_new_nentries(fval, fcmp, 0);
+  return hash_new_full(fval, fcmp, NULL, NULL);
 }
 
 /**************************************************************************
@@ -328,7 +372,7 @@ static void hash_free_contents(struct hash_table *h)
 {
   unsigned i;
 
-  for(i=0; i<h->num_buckets; i++) {
+  for(i = 0; i < h->num_buckets; i++) {
     zero_hbucket(&h->buckets[i]);
   }
   free(h->buckets);
@@ -341,6 +385,7 @@ static void hash_free_contents(struct hash_table *h)
 **************************************************************************/
 void hash_free(struct hash_table *h)
 {
+  hash_delete_all_entries(h);
   hash_free_contents(h);
   free(h);
 }
@@ -356,7 +401,9 @@ static void hash_resize_table(struct hash_table *h, unsigned int new_nbuckets)
 
   assert(new_nbuckets >= h->num_entries);
 
-  h_new = hash_new_nbuckets(h->fval, h->fcmp, new_nbuckets);
+  h_new = hash_new_nbuckets(h->fval, h->fcmp,
+			    h->free_key_func, h->free_data_func,
+			    new_nbuckets);
   h_new->frozen = TRUE;
   
   for(i=0; i<h->num_buckets; i++) {
@@ -441,8 +488,8 @@ static struct hash_bucket *internal_lookup(const struct hash_table *h,
     case BUCKET_UNUSED:
       return deleted ? deleted : bucket;
     case BUCKET_USED:
-      if (bucket->hash_val==hash_val
-	  && h->fcmp(bucket->key, key)==0) { /* match */
+      if (bucket->hash_val == hash_val
+	  && h->fcmp(bucket->key, key) == 0) { /* match */
 	return bucket;
       }
       break;
@@ -455,10 +502,10 @@ static struct hash_bucket *internal_lookup(const struct hash_table *h,
       die("Bad value %d in switch(bucket->used)", (int) bucket->used);
     }
     i++;
-    if (i==h->num_buckets) {
-      i=0;
+    if (i == h->num_buckets) {
+      i = 0;
     }
-  } while (i!=hash_val);	/* catch loop all the way round  */
+  } while (i != hash_val);	/* catch loop all the way round  */
 
   if (deleted) {
     return deleted;
@@ -486,8 +533,16 @@ bool hash_insert(struct hash_table *h, const void *key, const void *data)
     assert(h->num_deleted>0);
     h->num_deleted--;
   }
+
+  if (h->free_key_func) {
+    h->free_key_func((void*)bucket->key);
+  }
   bucket->key = key;
+  if (h->free_data_func) {
+    h->free_data_func((void*)bucket->data);
+  }
   bucket->data = data;
+
   bucket->used = BUCKET_USED;
   bucket->hash_val = hash_val;
   h->num_entries++;
@@ -519,12 +574,61 @@ void *hash_replace(struct hash_table *h, const void *key, const void *data)
     h->num_entries++;
     bucket->used = BUCKET_USED;
   }
+
+  if (h->free_key_func) {
+    h->free_key_func((void*)bucket->key);
+  }
   bucket->key = key;
+  if (h->free_data_func) {
+    h->free_data_func((void*)bucket->data);
+  }
   bucket->data = data;
+
   bucket->hash_val = hash_val;
   return (void*)ret;
 }
 
+/**************************************************************************
+  Deletes a single bucket from the hash.  You may call this on an unused
+  bucket.  old_key and old_value, if non-NULL, will be set to contain the
+  pointers to the key and value that are being deleted.  free_key_func
+  and free_data_func will (if set) be called on the key and data so these
+  return values may already have been freed.
+**************************************************************************/
+static void hash_delete_bucket(struct hash_table *h,
+			       struct hash_bucket *bucket,
+			       void **old_key,
+			       void **old_value)
+{
+  if (bucket->used == BUCKET_USED) {
+    if (old_key) {
+      *old_key = (void*)bucket->key;
+    }
+    if (old_value) {
+      *old_value = (void*)bucket->data;
+    }
+    if (h->free_key_func) {
+      h->free_key_func((void*)bucket->key);
+    }
+    if (h->free_data_func) {
+      h->free_data_func((void*)bucket->data);
+    }
+    zero_hbucket(bucket);
+    bucket->used = BUCKET_DELETED;
+    h->num_deleted++;
+    assert(h->num_entries > 0);
+    h->num_entries--;
+  } else {
+    if (old_key) {
+      *old_key = NULL;
+    }
+    if (old_value) {
+      *old_value = NULL;
+    }
+  }
+
+}
+
 /**************************************************************************
   Delete an entry with specified key.  Returns user-data of deleted
   entry, or NULL if not found.
@@ -544,27 +648,12 @@ void *hash_delete_entry_full(struct hash_table *h, const void *key,
 			     void **old_key)
 {
   struct hash_bucket *bucket;
+  void *old_value;
 
   hash_maybe_shrink(h);  
   bucket = internal_lookup(h, key, HASH_VAL(h,key));
-  if (bucket->used == BUCKET_USED) {
-    const void *ret = bucket->data;
-
-    if (old_key) {
-      *old_key = (void *)bucket->key;
-    }
-    zero_hbucket(bucket);
-    bucket->used = BUCKET_DELETED;
-    h->num_deleted++;
-    assert(h->num_entries>0);
-    h->num_entries--;
-    return (void*) ret;
-  } else {
-    if (old_key) {
-      *old_key = NULL;
-    }
-    return NULL;
-  }
+  hash_delete_bucket(h, bucket, old_key, &old_value);
+  return old_value;
 }
 
 /**************************************************************************
@@ -572,8 +661,13 @@ void *hash_delete_entry_full(struct hash_table *h, const void *key,
 **************************************************************************/
 void hash_delete_all_entries(struct hash_table *h)
 {
-  while (hash_num_entries(h) > 0)
-    (void) hash_delete_entry(h, hash_key_by_number(h, 0));
+  unsigned int bucket_nr;
+
+  /* Modeled after hash_key_by_number and hash_delete_entry. */
+  for (bucket_nr = 0; bucket_nr < h->num_buckets; bucket_nr++) {
+    hash_delete_bucket(h, &h->buckets[bucket_nr], NULL, NULL);
+  }
+  hash_maybe_shrink(h);
 }
 
 /**************************************************************************
@@ -586,14 +680,47 @@ bool hash_key_exists(const struct hash_table *h, const void *key)
 }
 
 /**************************************************************************
-  Lookup: return user-data, or NULL.
+  Lookup: looks up a key in the hash table returning the original key, and
+  the associated user-data, and a boolean which is TRUE if the key was
+  found.
+  (This is useful if you need to free the memory allocated for the
+  original key, for example before calling hash_delete_entry.)
+**************************************************************************/
+bool hash_lookup(const struct hash_table *h, const void *key,
+                 const void **pkey, const void **pdata)
+{
+  struct hash_bucket *bucket = internal_lookup(h, key, HASH_VAL(h,key));
+
+  if (bucket->used == BUCKET_USED) {
+    if (pkey) {
+      *pkey = bucket->key;
+    }
+    if (pdata) {
+      *pdata = bucket->data;
+    }
+    return TRUE;
+  } else {
+    if (pkey) {
+      *pkey = NULL;
+    }
+    if (pdata) {
+      *pdata = NULL;
+    }
+    return FALSE;
+  }
+}
+
+/**************************************************************************
+  Lookup data: return user-data, or NULL.
   (Note that in other respects NULL is treated as a valid value, this is
   merely intended as a convenience when caller never uses NULL as value.)
 **************************************************************************/
 void *hash_lookup_data(const struct hash_table *h, const void *key)
 {
-  struct hash_bucket *bucket = internal_lookup(h, key, HASH_VAL(h,key));
-  return ((bucket->used == BUCKET_USED) ? (void*)bucket->data : NULL);
+  const void *data;
+
+  hash_lookup(h, key, NULL, &data);
+  return (void*)data;
 }
 
 /**************************************************************************
